/******************************************************************************
 *
 * File:            PdfTools_Toolbox_PtxGeomReal.h
 *
 * Description:     Sub Header file for Toolbox SDK
 *
 * Author:          PDF Tools AG
 *
 * Copyright:       Copyright (C) 2024 - 2024 PDF Tools AG, Switzerland
 *                  All rights reserved.
 *
 * Notice:          By downloading and using this artifact, you accept PDF Tools AG's
 *                  [license agreement](https://www.pdf-tools.com/license-agreement/),
 *                  [privacy policy](https://www.pdf-tools.com/privacy-policy/),
 *                  and allow PDF Tools AG to track your usage data.
 *
 *****************************************************************************/

#ifndef PDFTOOLS_TOOLBOX_PTXGEOMREAL_H__
#define PDFTOOLS_TOOLBOX_PTXGEOMREAL_H__

#ifndef PDFTOOLS_TOOLBOX_CALL
#if defined(WIN32)
#define PDFTOOLS_TOOLBOX_CALL __stdcall
#else
#define PDFTOOLS_TOOLBOX_CALL
#endif
#endif

#include "PdfTools_Toolbox_Types.h"
#include "PdfTools_Toolbox_PtxSys.h"

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef _UNICODE
#else
#endif

/******************************************************************************
 * QuadrilateralList
 *****************************************************************************/
/**
Create an empty list of \ref TPtxGeomReal_Quadrilateral ""s

 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxGeomReal_QuadrilateralList* PDFTOOLS_TOOLBOX_CALL PtxGeomReal_QuadrilateralList_New(void);

/**
 * @brief Get the number of elements in the list.
* @param[in,out] pQuadrilateralList Acts as a handle to the native object of type \ref TPtxGeomReal_QuadrilateralList.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL
PtxGeomReal_QuadrilateralList_GetCount(TPtxGeomReal_QuadrilateralList* pQuadrilateralList);
/**
 * @brief Returns the element at the specified position in the given list.
* @param[in,out] pQuadrilateralList Acts as a handle to the native object of type \ref TPtxGeomReal_QuadrilateralList.

* @param[in] iIndex
* @param[out] pQuadrilateral

 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxGeomReal_QuadrilateralList_Get(
    TPtxGeomReal_QuadrilateralList* pQuadrilateralList, int iIndex, TPtxGeomReal_Quadrilateral* pQuadrilateral);
/**
 * @brief Add an element to the end of the list.
* @param[in,out] pQuadrilateralList Acts as a handle to the native object of type \ref TPtxGeomReal_QuadrilateralList.

* @param[in] pQuadrilateral

 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxGeomReal_QuadrilateralList_Add(
    TPtxGeomReal_QuadrilateralList* pQuadrilateralList, const TPtxGeomReal_Quadrilateral* pQuadrilateral);
/**
 * @brief Clear list.
* @param[in,out] pQuadrilateralList Acts as a handle to the native object of type \ref TPtxGeomReal_QuadrilateralList.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxGeomReal_QuadrilateralList_Clear(TPtxGeomReal_QuadrilateralList* pQuadrilateralList);
/**
 * @brief Remove certain element from list.
* @param[in,out] pQuadrilateralList Acts as a handle to the native object of type \ref TPtxGeomReal_QuadrilateralList.

* @param[in] iIndex

 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxGeomReal_QuadrilateralList_Remove(TPtxGeomReal_QuadrilateralList* pQuadrilateralList, int iIndex);
/**
 * @brief Update certain element in list.
* @param[in,out] pQuadrilateralList Acts as a handle to the native object of type \ref TPtxGeomReal_QuadrilateralList.

* @param[in] iIndex
* @param[in] pValue

 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxGeomReal_QuadrilateralList_Set(
    TPtxGeomReal_QuadrilateralList* pQuadrilateralList, int iIndex, const TPtxGeomReal_Quadrilateral* pValue);

/******************************************************************************
 * AffineTransform Methods
 *****************************************************************************/
/**
 * @brief Translate.
Translations are specified as [1 0 0 1 tx ty],
where tx and ty are the distances to translate the origin of the
coordinate system in the horizontal and vertical dimensions, respectively.

* @param[in,out] pAffineTransform The pointer to the struct of type \ref TPtxGeomReal_AffineTransform to which this
function acts on.

* @param[in] tx horizontal translation

* @param[in] ty vertical translation


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState If the affine transform is singular, e.g. default initialized.
(Use \ref PtxGeomReal_AffineTransform_GetIdentity "" as an initial value.)


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxGeomReal_AffineTransform_Translate(TPtxGeomReal_AffineTransform* pAffineTransform, double tx, double ty);
/**
 * @brief Scale.
Scaling is obtained by [sx 0 0 sy 0 0].
This scales the coordinates so that 1 unit in the horizontal and vertical dimensions
of the new coordinate system is the same size as sx and sy units,
respectively, in the previous coordinate system.

* @param[in,out] pAffineTransform The pointer to the struct of type \ref TPtxGeomReal_AffineTransform to which this
function acts on.

* @param[in] sx horizontal scale factor

* @param[in] sy vertical scale factor


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState If the affine transform is singular, e.g. default initialized.
(Use \ref PtxGeomReal_AffineTransform_GetIdentity "" as an initial value.)


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxGeomReal_AffineTransform_Scale(TPtxGeomReal_AffineTransform* pAffineTransform, double sx, double sy);
/**
 * @brief Rotate.

Rotations are produced by [cos(a) sin(a) -sin(a) cos(a) 0 0],
which has the effect of rotating the coordinate system axes
by an angle "a" in counterclockwise direction around the origin.

If the given <b>pCenter</b> is not `NULL`,
then the rotation is performed around the given center point,
which is equivalent to the following sequence:
  - \ref PtxGeomReal_AffineTransform_Translate "" to <b>pCenter</b>.
  - \ref PtxGeomReal_AffineTransform_Rotate "" by <b>dAngle</b> around the origin.
  - \ref PtxGeomReal_AffineTransform_Translate "" "back" to the original origin.

* @param[in,out] pAffineTransform The pointer to the struct of type \ref TPtxGeomReal_AffineTransform to which this
function acts on.

* @param[in] dAngle The angle of the rotation in degrees.

* @param[in] pCenter The center of the rotation.
If `NULL` then the origin (0/0) is taken as center.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState If the affine transform is singular, e.g. default initialized.
(Use \ref PtxGeomReal_AffineTransform_GetIdentity "" as an initial value.)


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxGeomReal_AffineTransform_Rotate(
    TPtxGeomReal_AffineTransform* pAffineTransform, double dAngle, const TPtxGeomReal_Point* pCenter);
/**
 * @brief Skew.
Skew is specified by [1 tan a tan b 1 0 0],
which skews the x axis by an angle a and the y axis by an angle b.

* @param[in,out] pAffineTransform The pointer to the struct of type \ref TPtxGeomReal_AffineTransform to which this
function acts on.

* @param[in] dAlpha angle a in degrees

* @param[in] dBeta angle b in degrees


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState If the affine transform is singular, e.g. default initialized.
(Use \ref PtxGeomReal_AffineTransform_GetIdentity "" as an initial value.)

 * - \ref ePtx_Error_IllegalArgument if any of the given angles is too close to 90 + k*180 degrees for k = ..., -2, -1,
0, 1, 2, ...


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxGeomReal_AffineTransform_Skew(TPtxGeomReal_AffineTransform* pAffineTransform, double dAlpha, double dBeta);
/**
 * @brief Concatenate transform with other transform.
Concatenating a transform with an other transform is equivalent to left-multiplying the transform's matrix with with the
other transform's matrix.

* @param[in,out] pAffineTransform The pointer to the struct of type \ref TPtxGeomReal_AffineTransform to which this
function acts on.

* @param[in] pOther the transform to be concatenated to this transform


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState If the affine transform is singular, e.g. default initialized.
(Use \ref PtxGeomReal_AffineTransform_GetIdentity "" as an initial value.)

 * - \ref ePtx_Error_IllegalArgument If the <b>pOther</b> affine transform is singular, e.g. default initialized.
(Use \ref PtxGeomReal_AffineTransform_GetIdentity "" as an initial value.)


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxGeomReal_AffineTransform_Concatenate(
    TPtxGeomReal_AffineTransform* pAffineTransform, const TPtxGeomReal_AffineTransform* pOther);
/**
 * @brief Invert the transform
A transform usually maps from the transformed coordinate system to the untransformed coordinate system.
Use this method to create the reverse transform.

* @param[in,out] pAffineTransform The pointer to the struct of type \ref TPtxGeomReal_AffineTransform to which this
function acts on.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState If the affine transform is singular, e.g. default initialized.
(Use \ref PtxGeomReal_AffineTransform_GetIdentity "" as an initial value.)


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxGeomReal_AffineTransform_Invert(TPtxGeomReal_AffineTransform* pAffineTransform);
/**
 * @brief Transforms the given point.
* @param[in,out] pAffineTransform The pointer to the struct of type \ref TPtxGeomReal_AffineTransform to which this
function acts on.

* @param[in] pOriginal the point to be transformed

* @param[out] pPoint the transformed point


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState If the affine transform is singular, e.g. default initialized.
(Use \ref PtxGeomReal_AffineTransform_GetIdentity "" as an initial value.)


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxGeomReal_AffineTransform_TransformPoint(
    TPtxGeomReal_AffineTransform* pAffineTransform, const TPtxGeomReal_Point* pOriginal, TPtxGeomReal_Point* pPoint);
/**
 * @brief Transform the given rectangle
For a general affine transformation, the returned \ref TPtxGeomReal_Quadrilateral "" is a parallelogram.

* @param[in,out] pAffineTransform The pointer to the struct of type \ref TPtxGeomReal_AffineTransform to which this
function acts on.

* @param[in] pOriginal the rectangle to be transformed

* @param[out] pQuadrilateral the transformed rectangle. For a general affine transform this is a parallelogram.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the affine transform is singular, e.g. default initialized.
(Use \ref PtxGeomReal_AffineTransform_GetIdentity "" as an initial value.)


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxGeomReal_AffineTransform_TransformRectangle(
    TPtxGeomReal_AffineTransform* pAffineTransform, const TPtxGeomReal_Rectangle* pOriginal,
    TPtxGeomReal_Quadrilateral* pQuadrilateral);
/**
 * @brief Transform a given quadrilateral
If the input quadrilateral is a parallelogram, then the output is also a parallelogram.

* @param[in,out] pAffineTransform The pointer to the struct of type \ref TPtxGeomReal_AffineTransform to which this
function acts on.

* @param[in] pOriginal the quadrilateral to be transformed

* @param[out] pQuadrilateral the transformed quadrilateral. If the input is a parallelogram then the output is also a
parallelogram.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the affine transform is singular, e.g. default initialized.
(Use \ref PtxGeomReal_AffineTransform_GetIdentity "" as an initial value.)


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxGeomReal_AffineTransform_TransformQuadrilateral(
    TPtxGeomReal_AffineTransform* pAffineTransform, const TPtxGeomReal_Quadrilateral* pOriginal,
    TPtxGeomReal_Quadrilateral* pQuadrilateral);

/******************************************************************************
 * AffineTransform Properties
 *****************************************************************************/
/**
 * @brief The identity transform
* @param[out] pIdentity Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxGeomReal_AffineTransform_GetIdentity(TPtxGeomReal_AffineTransform* pIdentity);

#ifdef __cplusplus
}
#endif

#endif /* PDFTOOLS_TOOLBOX_PTXGEOMREAL_H__ */
