/******************************************************************************
 *
 * File:            PdfTools_Toolbox_PtxPdf.h
 *
 * Description:     Sub Header file for Toolbox SDK
 *
 * Author:          PDF Tools AG
 *
 * Copyright:       Copyright (C) 2024 - 2024 PDF Tools AG, Switzerland
 *                  All rights reserved.
 *
 * Notice:          By downloading and using this artifact, you accept PDF Tools AG's
 *                  [license agreement](https://www.pdf-tools.com/license-agreement/),
 *                  [privacy policy](https://www.pdf-tools.com/privacy-policy/),
 *                  and allow PDF Tools AG to track your usage data.
 *
 *****************************************************************************/

#ifndef PDFTOOLS_TOOLBOX_PTXPDF_H__
#define PDFTOOLS_TOOLBOX_PTXPDF_H__

#ifndef PDFTOOLS_TOOLBOX_CALL
#if defined(WIN32)
#define PDFTOOLS_TOOLBOX_CALL __stdcall
#else
#define PDFTOOLS_TOOLBOX_CALL
#endif
#endif

#include "PdfTools_Toolbox_Types.h"
#include "PdfTools_Toolbox_PtxSys.h"

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef _UNICODE
#define PtxPdf_Encryption_New              PtxPdf_Encryption_NewW
#define PtxPdf_Encryption_GetUserPassword  PtxPdf_Encryption_GetUserPasswordW
#define PtxPdf_Encryption_SetUserPassword  PtxPdf_Encryption_SetUserPasswordW
#define PtxPdf_Encryption_GetOwnerPassword PtxPdf_Encryption_GetOwnerPasswordW
#define PtxPdf_Encryption_SetOwnerPassword PtxPdf_Encryption_SetOwnerPasswordW

#define PtxPdf_Document_Open        PtxPdf_Document_OpenW
#define PtxPdf_Document_OpenWithFdf PtxPdf_Document_OpenWithFdfW

#define PtxPdf_Page_GetPageLabel PtxPdf_Page_GetPageLabelW

#define PtxPdf_Metadata_GetTitle    PtxPdf_Metadata_GetTitleW
#define PtxPdf_Metadata_SetTitle    PtxPdf_Metadata_SetTitleW
#define PtxPdf_Metadata_GetAuthor   PtxPdf_Metadata_GetAuthorW
#define PtxPdf_Metadata_SetAuthor   PtxPdf_Metadata_SetAuthorW
#define PtxPdf_Metadata_GetSubject  PtxPdf_Metadata_GetSubjectW
#define PtxPdf_Metadata_SetSubject  PtxPdf_Metadata_SetSubjectW
#define PtxPdf_Metadata_GetKeywords PtxPdf_Metadata_GetKeywordsW
#define PtxPdf_Metadata_SetKeywords PtxPdf_Metadata_SetKeywordsW
#define PtxPdf_Metadata_GetCreator  PtxPdf_Metadata_GetCreatorW
#define PtxPdf_Metadata_SetCreator  PtxPdf_Metadata_SetCreatorW
#define PtxPdf_Metadata_GetProducer PtxPdf_Metadata_GetProducerW

#define PtxPdf_FileReference_Create PtxPdf_FileReference_CreateW

#define PtxPdf_FileReference_GetAssociationRelationship PtxPdf_FileReference_GetAssociationRelationshipW
#define PtxPdf_FileReference_SetAssociationRelationship PtxPdf_FileReference_SetAssociationRelationshipW
#define PtxPdf_FileReference_GetDescription             PtxPdf_FileReference_GetDescriptionW

#define PtxPdf_FileReference_GetMediaType PtxPdf_FileReference_GetMediaTypeW

#define PtxPdf_FileReference_GetName PtxPdf_FileReference_GetNameW

#else
#define PtxPdf_Encryption_New              PtxPdf_Encryption_NewA
#define PtxPdf_Encryption_GetUserPassword  PtxPdf_Encryption_GetUserPasswordA
#define PtxPdf_Encryption_SetUserPassword  PtxPdf_Encryption_SetUserPasswordA
#define PtxPdf_Encryption_GetOwnerPassword PtxPdf_Encryption_GetOwnerPasswordA
#define PtxPdf_Encryption_SetOwnerPassword PtxPdf_Encryption_SetOwnerPasswordA

#define PtxPdf_Document_Open        PtxPdf_Document_OpenA
#define PtxPdf_Document_OpenWithFdf PtxPdf_Document_OpenWithFdfA

#define PtxPdf_Page_GetPageLabel PtxPdf_Page_GetPageLabelA

#define PtxPdf_Metadata_GetTitle    PtxPdf_Metadata_GetTitleA
#define PtxPdf_Metadata_SetTitle    PtxPdf_Metadata_SetTitleA
#define PtxPdf_Metadata_GetAuthor   PtxPdf_Metadata_GetAuthorA
#define PtxPdf_Metadata_SetAuthor   PtxPdf_Metadata_SetAuthorA
#define PtxPdf_Metadata_GetSubject  PtxPdf_Metadata_GetSubjectA
#define PtxPdf_Metadata_SetSubject  PtxPdf_Metadata_SetSubjectA
#define PtxPdf_Metadata_GetKeywords PtxPdf_Metadata_GetKeywordsA
#define PtxPdf_Metadata_SetKeywords PtxPdf_Metadata_SetKeywordsA
#define PtxPdf_Metadata_GetCreator  PtxPdf_Metadata_GetCreatorA
#define PtxPdf_Metadata_SetCreator  PtxPdf_Metadata_SetCreatorA
#define PtxPdf_Metadata_GetProducer PtxPdf_Metadata_GetProducerA

#define PtxPdf_FileReference_Create PtxPdf_FileReference_CreateA

#define PtxPdf_FileReference_GetAssociationRelationship PtxPdf_FileReference_GetAssociationRelationshipA
#define PtxPdf_FileReference_SetAssociationRelationship PtxPdf_FileReference_SetAssociationRelationshipA
#define PtxPdf_FileReference_GetDescription             PtxPdf_FileReference_GetDescriptionA

#define PtxPdf_FileReference_GetMediaType PtxPdf_FileReference_GetMediaTypeA

#define PtxPdf_FileReference_GetName PtxPdf_FileReference_GetNameA

#endif

/******************************************************************************
 * PageCopyOptions
 *****************************************************************************/
/**
 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_PageCopyOptions* PDFTOOLS_TOOLBOX_CALL PtxPdf_PageCopyOptions_New(void);

/**
 * @brief Copy strategy for links.

Specifies how links (document internal and external links) are treated
when copying a page.

Default value: \ref ePtxPdf_CopyStrategy_Copy ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_CopyStrategy PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetLinks(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Copy strategy for links.

Specifies how links (document internal and external links) are treated
when copying a page.

Default value: \ref ePtxPdf_CopyStrategy_Copy ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] iLinks Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_SetLinks(TPtxPdf_PageCopyOptions* pPageCopyOptions, TPtxPdf_CopyStrategy iLinks);
/**
 * @brief Copy strategy for form fields and widgets.

Specifies how form fields and widgets are treated when copying a page.

Default value: \ref ePtxPdfForms_FormFieldCopyStrategy_Copy ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfForms_FormFieldCopyStrategy PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetFormFields(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Copy strategy for form fields and widgets.

Specifies how form fields and widgets are treated when copying a page.

Default value: \ref ePtxPdfForms_FormFieldCopyStrategy_Copy ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] iFormFields Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_PageCopyOptions_SetFormFields(
    TPtxPdf_PageCopyOptions* pPageCopyOptions, TPtxPdfForms_FormFieldCopyStrategy iFormFields);
/**
 * @brief Removal strategy for signed signature fields.

Signed digital signatures are always invalidated when copying a page
and therefore have to be removed.
This property specifies, whether the visual representation of the signature
is preserved.

Default value: \ref ePtxPdf_RemovalStrategy_Remove ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_RemovalStrategy PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetSignedSignatures(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Removal strategy for signed signature fields.

Signed digital signatures are always invalidated when copying a page
and therefore have to be removed.
This property specifies, whether the visual representation of the signature
is preserved.

Default value: \ref ePtxPdf_RemovalStrategy_Remove ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] iSignedSignatures Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_PageCopyOptions_SetSignedSignatures(
    TPtxPdf_PageCopyOptions* pPageCopyOptions, TPtxPdf_RemovalStrategy iSignedSignatures);
/**
 * @brief Copy strategy for unsigned signature fields.

Specifies how signature fields are treated, that are not yet signed.

Default value: \ref ePtxPdf_CopyStrategy_Copy ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_CopyStrategy PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetUnsignedSignatures(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Copy strategy for unsigned signature fields.

Specifies how signature fields are treated, that are not yet signed.

Default value: \ref ePtxPdf_CopyStrategy_Copy ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] iUnsignedSignatures Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_PageCopyOptions_SetUnsignedSignatures(
    TPtxPdf_PageCopyOptions* pPageCopyOptions, TPtxPdf_CopyStrategy iUnsignedSignatures);
/**
 * @brief Copy strategy for annotations.

Specifies how interactive annotations (like sticky notes or text highlights)
are treated when copying a page.
This does not include links, form fields and signature fields which
are not considered annotations in this products.

Default value: \ref ePtxPdf_CopyStrategy_Copy ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_CopyStrategy PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetAnnotations(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Copy strategy for annotations.

Specifies how interactive annotations (like sticky notes or text highlights)
are treated when copying a page.
This does not include links, form fields and signature fields which
are not considered annotations in this products.

Default value: \ref ePtxPdf_CopyStrategy_Copy ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] iAnnotations Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_SetAnnotations(TPtxPdf_PageCopyOptions* pPageCopyOptions, TPtxPdf_CopyStrategy iAnnotations);
/**
 * @brief Copy outline items (bookmarks).

Specifies whether outline items (also known as bookmarks) pointing to the copied page
should be copied to the target document automatically.

Default value: \ref "TRUE"

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetCopyOutlineItems(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Copy outline items (bookmarks).

Specifies whether outline items (also known as bookmarks) pointing to the copied page
should be copied to the target document automatically.

Default value: \ref "TRUE"

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] bCopyOutlineItems Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_SetCopyOutlineItems(TPtxPdf_PageCopyOptions* pPageCopyOptions, BOOL bCopyOutlineItems);
/**
 * @brief Copy associated files.

Specifies whether embedded files associated with a page or any of its
subobjects are also copied when copying the page.

Default value: \ref "TRUE"

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetCopyAssociatedFiles(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Copy associated files.

Specifies whether embedded files associated with a page or any of its
subobjects are also copied when copying the page.

Default value: \ref "TRUE"

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] bCopyAssociatedFiles Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_SetCopyAssociatedFiles(TPtxPdf_PageCopyOptions* pPageCopyOptions, BOOL bCopyAssociatedFiles);
/**
 * @brief Copy the logical structure and tagging information.

Specifies whether the logical structure and tagging information associated
with a page or its content is also copied when copying the page.

This is required if the target document conformance is PDF/A Level a.

Default value: \ref "TRUE"

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetCopyLogicalStructure(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Copy the logical structure and tagging information.

Specifies whether the logical structure and tagging information associated
with a page or its content is also copied when copying the page.

This is required if the target document conformance is PDF/A Level a.

Default value: \ref "TRUE"

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] bCopyLogicalStructure Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_SetCopyLogicalStructure(TPtxPdf_PageCopyOptions* pPageCopyOptions, BOOL bCopyLogicalStructure);
/**
 * @brief Resolution of conflicting form field names.

Form field of different files can have the same name (identifier).
This property specifies how name conflicts are resolved,
when copying pages from multiple source files.

Default value: \ref ePtxPdf_NameConflictResolution_Merge ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_NameConflictResolution PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetFormFieldConflictResolution(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Resolution of conflicting form field names.

Form field of different files can have the same name (identifier).
This property specifies how name conflicts are resolved,
when copying pages from multiple source files.

Default value: \ref ePtxPdf_NameConflictResolution_Merge ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] iFormFieldConflictResolution Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_PageCopyOptions_SetFormFieldConflictResolution(
    TPtxPdf_PageCopyOptions* pPageCopyOptions, TPtxPdf_NameConflictResolution iFormFieldConflictResolution);
/**
* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 * @deprecated Deprecated in Version 3.3. Setting this property has no effect. When copying pages from multiple
documents, layers (optional content groups, OCG) are always merged.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_NameConflictResolution PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetOcgConflictResolution(
    TPtxPdf_PageCopyOptions*
        pPageCopyOptions); // Deprecated in Version 3.3. Setting this property has no effect. When copying pages from
                           // multiple documents, layers (optional content groups, OCG) are always merged.
/**
* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] iOcgConflictResolution Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 * @deprecated Deprecated in Version 3.3. Setting this property has no effect. When copying pages from multiple
documents, layers (optional content groups, OCG) are always merged.
 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_PageCopyOptions_SetOcgConflictResolution(
    TPtxPdf_PageCopyOptions* pPageCopyOptions,
    TPtxPdf_NameConflictResolution
        iOcgConflictResolution); // Deprecated in Version 3.3. Setting this property has no effect. When copying pages
                                 // from multiple documents, layers (optional content groups, OCG) are always merged.
/**
 * @brief Copy strategy for named destinations

Specify whether named destinations are resolved when copying a page.

Default value: \ref ePtxPdfNav_NamedDestinationCopyStrategy_Copy ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_NamedDestinationCopyStrategy PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetNamedDestinations(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Copy strategy for named destinations

Specify whether named destinations are resolved when copying a page.

Default value: \ref ePtxPdfNav_NamedDestinationCopyStrategy_Copy ""

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] iNamedDestinations Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_PageCopyOptions_SetNamedDestinations(
    TPtxPdf_PageCopyOptions* pPageCopyOptions, TPtxPdfNav_NamedDestinationCopyStrategy iNamedDestinations);
/**
 * @brief Find and merge redundant resources.

Find and merge redundant resources such as fonts and images.
This can lead to much smaller files, especially when copying
pages from multiple similar source files.
However it also decreases performance.

Default value: \ref "TRUE"

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_GetOptimizeResources(TPtxPdf_PageCopyOptions* pPageCopyOptions);
/**
 * @brief Find and merge redundant resources.

Find and merge redundant resources such as fonts and images.
This can lead to much smaller files, especially when copying
pages from multiple similar source files.
However it also decreases performance.

Default value: \ref "TRUE"

* @param[in,out] pPageCopyOptions Acts as a handle to the native object of type \ref TPtxPdf_PageCopyOptions.

* @param[in] bOptimizeResources Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_PageCopyOptions_SetOptimizeResources(TPtxPdf_PageCopyOptions* pPageCopyOptions, BOOL bOptimizeResources);

/******************************************************************************
 * Encryption
 *****************************************************************************/
/**
* @param[in] szUserPassword
* @param[in] szOwnerPassword
* @param[in] iPermissions

 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Encryption* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Encryption_NewA(const char* szUserPassword, const char* szOwnerPassword, TPtxPdf_Permission iPermissions);
/**
* @param[in] szUserPassword
* @param[in] szOwnerPassword
* @param[in] iPermissions

 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Encryption* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Encryption_NewW(const WCHAR* szUserPassword, const WCHAR* szOwnerPassword, TPtxPdf_Permission iPermissions);

/**
* @param[in,out] pEncryption Acts as a handle to the native object of type \ref TPtxPdf_Encryption.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Encryption_GetUserPasswordA(TPtxPdf_Encryption* pEncryption,
                                                                                        char*               pBuffer,
                                                                                        size_t nBufferSize);
/**
* @param[in,out] pEncryption Acts as a handle to the native object of type \ref TPtxPdf_Encryption.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Encryption_GetUserPasswordW(TPtxPdf_Encryption* pEncryption,
                                                                                        WCHAR*              pBuffer,
                                                                                        size_t nBufferSize);
/**
* @param[in,out] pEncryption Acts as a handle to the native object of type \ref TPtxPdf_Encryption.

* @param[in] szUserPassword Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Encryption_SetUserPasswordA(TPtxPdf_Encryption* pEncryption,
                                                                                      const char* szUserPassword);
/**
* @param[in,out] pEncryption Acts as a handle to the native object of type \ref TPtxPdf_Encryption.

* @param[in] szUserPassword Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Encryption_SetUserPasswordW(TPtxPdf_Encryption* pEncryption,
                                                                                      const WCHAR* szUserPassword);
/**
* @param[in,out] pEncryption Acts as a handle to the native object of type \ref TPtxPdf_Encryption.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdf_Encryption_GetOwnerPasswordA(TPtxPdf_Encryption* pEncryption, char* pBuffer, size_t nBufferSize);
/**
* @param[in,out] pEncryption Acts as a handle to the native object of type \ref TPtxPdf_Encryption.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdf_Encryption_GetOwnerPasswordW(TPtxPdf_Encryption* pEncryption, WCHAR* pBuffer, size_t nBufferSize);
/**
* @param[in,out] pEncryption Acts as a handle to the native object of type \ref TPtxPdf_Encryption.

* @param[in] szOwnerPassword Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Encryption_SetOwnerPasswordA(TPtxPdf_Encryption* pEncryption,
                                                                                       const char* szOwnerPassword);
/**
* @param[in,out] pEncryption Acts as a handle to the native object of type \ref TPtxPdf_Encryption.

* @param[in] szOwnerPassword Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Encryption_SetOwnerPasswordW(TPtxPdf_Encryption* pEncryption,
                                                                                       const WCHAR* szOwnerPassword);
/**
* @param[in,out] pEncryption Acts as a handle to the native object of type \ref TPtxPdf_Encryption.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Permission PDFTOOLS_TOOLBOX_CALL
PtxPdf_Encryption_GetPermissions(TPtxPdf_Encryption* pEncryption);
/**
* @param[in,out] pEncryption Acts as a handle to the native object of type \ref TPtxPdf_Encryption.

* @param[in] iPermissions Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Encryption_SetPermissions(TPtxPdf_Encryption* pEncryption,
                                                                                    TPtxPdf_Permission  iPermissions);

/******************************************************************************
 * PageList
 *****************************************************************************/
/**
 * @brief Copy a page list
Copy pages from an input document to the given <b>pTargetDocument</b>.
The returned list is associated with the given target document but not yet part of it.
It can be appended to the document's page list.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pPageList a page list of a different document

* @param[in,out] pOptions the copy options


 * @return   the copied page list, associated with the target document.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the source document or writing to the target document

 * - \ref ePtx_Error_Corrupt The source document is corrupt

 * - \ref ePtx_Error_Conformance The conformance level of the source document is not compatible
with the conformance level of the target document.

 * - \ref ePtx_Error_Conformance The explicitly requested conformance level is PDF/A Level A
(\ref ePtxPdf_Conformance_PdfA1A "", \ref ePtxPdf_Conformance_PdfA2A "",
or \ref ePtxPdf_Conformance_PdfA3A "")
and the copy option \ref PtxPdf_PageCopyOptions_GetCopyLogicalStructure "" is not set.

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pPageList</b> object is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument if the document associated with <b>pPageList</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the argument <b>pOptions</b> has \ref PtxPdf_PageCopyOptions_GetFormFields ""
set to \ref ePtxPdfForms_FormFieldCopyStrategy_Copy "" and the <b>pTargetDocument</b> contains form fields that have
either been copied explicitly with \ref PtxPdfForms_FieldNode_Copy "" or created with \ref PtxPdfForms_CheckBox_Create
"", \ref PtxPdfForms_ComboBox_Create "", \ref PtxPdfForms_CombTextField_Create "", \ref
PtxPdfForms_GeneralTextField_Create "", \ref PtxPdfForms_ListBox_Create "", \ref PtxPdfForms_RadioButtonGroup_Create "",
or \ref PtxPdfForms_SubForm_Create "".

 * - \ref ePtx_Error_IllegalArgument if the argument <b>pOptions</b> has \ref
PtxPdf_PageCopyOptions_GetUnsignedSignatures "" set to \ref ePtxPdf_CopyStrategy_Copy "" and the <b>pTargetDocument</b>
contains form fields that have either been copied explicitly with \ref PtxPdfForms_FieldNode_Copy "" or created with
\ref PtxPdfForms_CheckBox_Create "", \ref PtxPdfForms_ComboBox_Create "", \ref PtxPdfForms_CombTextField_Create "", \ref
PtxPdfForms_GeneralTextField_Create "", \ref PtxPdfForms_ListBox_Create "", \ref PtxPdfForms_RadioButtonGroup_Create "",
or \ref PtxPdfForms_SubForm_Create "".

 * - \ref ePtx_Error_IllegalArgument if <b>pOptions</b> has \ref PtxPdf_PageCopyOptions_GetCopyOutlineItems "" set to
\ref "TRUE" and the <b>pTargetDocument</b> contains outline items that have been copied explicitly with \ref
PtxPdfNav_OutlineItem_Copy "".


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_PageList* PDFTOOLS_TOOLBOX_CALL PtxPdf_PageList_Copy(TPtxPdf_Document* pTargetDocument,
                                                                                     TPtxPdf_PageList* pPageList,
                                                                                     TPtxPdf_PageCopyOptions* pOptions);
/**
 * @brief Get the number of elements in the list.
* @param[in,out] pPageList Acts as a handle to the native object of type \ref TPtxPdf_PageList.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL PtxPdf_PageList_GetCount(TPtxPdf_PageList* pPageList);
/**
 * @brief Returns the element at the specified position in the given list.
* @param[in,out] pPageList Acts as a handle to the native object of type \ref TPtxPdf_PageList.

* @param[in] iIndex

 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Page* PDFTOOLS_TOOLBOX_CALL PtxPdf_PageList_Get(TPtxPdf_PageList* pPageList,
                                                                                int               iIndex);
/**
 * @brief Get range of elements.
* @param[in,out] pPageList Acts as a handle to the native object of type \ref TPtxPdf_PageList.

* @param[in] iIndex
* @param[in] iCount

 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_PageList* PDFTOOLS_TOOLBOX_CALL PtxPdf_PageList_GetRange(TPtxPdf_PageList* pPageList,
                                                                                         int iIndex, int iCount);
/**
 * @brief Add an element to the end of the list.
* @param[in,out] pPageList Acts as a handle to the native object of type \ref TPtxPdf_PageList.

* @param[in,out] pPage

 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_PageList_Add(TPtxPdf_PageList* pPageList,
                                                                       TPtxPdf_Page*     pPage);
/**
 * @brief Add range of elements to th end of the list.
* @param[in,out] pPageList Acts as a handle to the native object of type \ref TPtxPdf_PageList.

* @param[in,out] pInput

 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_PageList_AddRange(TPtxPdf_PageList* pPageList,
                                                                            TPtxPdf_PageList* pInput);

/******************************************************************************
 * FileReferenceList
 *****************************************************************************/
/**
 * @brief Get the number of elements in the list.
* @param[in,out] pFileReferenceList Acts as a handle to the native object of type \ref TPtxPdf_FileReferenceList.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReferenceList_GetCount(TPtxPdf_FileReferenceList* pFileReferenceList);
/**
 * @brief Returns the element at the specified position in the given list.
* @param[in,out] pFileReferenceList Acts as a handle to the native object of type \ref TPtxPdf_FileReferenceList.

* @param[in] iIndex

 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_FileReference* PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReferenceList_Get(TPtxPdf_FileReferenceList* pFileReferenceList, int iIndex);
/**
 * @brief Add an element to the end of the list.
* @param[in,out] pFileReferenceList Acts as a handle to the native object of type \ref TPtxPdf_FileReferenceList.

* @param[in,out] pFileReference

 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReferenceList_Add(TPtxPdf_FileReferenceList* pFileReferenceList, TPtxPdf_FileReference* pFileReference);

/******************************************************************************
 * Document
 *****************************************************************************/
/**
 * @brief Open a PDF document.
Documents opened with this method are read-only and cannot be modified.

* @param[in] pStreamDesc The stream where the PDF document is stored.
Read access is required.

* @param[in] szPassword the password to open the PDF document


 * @return   the newly created document instance

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_Password if the file is encrypted and the password is not valid.

 * - \ref ePtx_Error_IO Error reading from the stream.

 * - \ref ePtx_Error_Corrupt if the file is corrupt or not a PDF.

 * - \ref ePtx_Error_UnsupportedFeature if the file is a PDF collection.

 * - \ref ePtx_Error_UnsupportedFeature if the PDF contains unrendered XFA fields.

 * - \ref ePtx_Error_Conformance if the document's conformance level is not supported


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Document* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_OpenA(const TPtxSys_StreamDescriptor* pStreamDesc, const char* szPassword);
/**
 * @brief Open a PDF document.
Documents opened with this method are read-only and cannot be modified.

* @param[in] pStreamDesc The stream where the PDF document is stored.
Read access is required.

* @param[in] szPassword the password to open the PDF document


 * @return   the newly created document instance

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_Password if the file is encrypted and the password is not valid.

 * - \ref ePtx_Error_IO Error reading from the stream.

 * - \ref ePtx_Error_Corrupt if the file is corrupt or not a PDF.

 * - \ref ePtx_Error_UnsupportedFeature if the file is a PDF collection.

 * - \ref ePtx_Error_UnsupportedFeature if the PDF contains unrendered XFA fields.

 * - \ref ePtx_Error_Conformance if the document's conformance level is not supported


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Document* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_OpenW(const TPtxSys_StreamDescriptor* pStreamDesc, const WCHAR* szPassword);
/**
 * @brief Open a PDF document together with an FDF file.
Documents opened with this method are read-only and cannot be modified.

* @param[in] pPdfStreamDesc The stream where the PDF document is stored.
Read access is required.

* @param[in] pFdfStreamDesc The stream where the FDF file is stored.
Read access is required.

* @param[in] szPassword the password to open the PDF document


 * @return   the newly created document instance

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_Password if the file is encrypted and the <b>szPassword</b> is not valid.

 * - \ref ePtx_Error_IO Error reading from the stream.

 * - \ref ePtx_Error_Corrupt if the file is corrupt or not a PDF.

 * - \ref ePtx_Error_UnsupportedFeature if the file is a PDF collection.

 * - \ref ePtx_Error_UnsupportedFeature if the PDF contains unrendered XFA fields.

 * - \ref ePtx_Error_Conformance if the document's conformance level is not supported


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Document* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_OpenWithFdfA(const TPtxSys_StreamDescriptor* pPdfStreamDesc,
                             const TPtxSys_StreamDescriptor* pFdfStreamDesc, const char* szPassword);
/**
 * @brief Open a PDF document together with an FDF file.
Documents opened with this method are read-only and cannot be modified.

* @param[in] pPdfStreamDesc The stream where the PDF document is stored.
Read access is required.

* @param[in] pFdfStreamDesc The stream where the FDF file is stored.
Read access is required.

* @param[in] szPassword the password to open the PDF document


 * @return   the newly created document instance

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_Password if the file is encrypted and the <b>szPassword</b> is not valid.

 * - \ref ePtx_Error_IO Error reading from the stream.

 * - \ref ePtx_Error_Corrupt if the file is corrupt or not a PDF.

 * - \ref ePtx_Error_UnsupportedFeature if the file is a PDF collection.

 * - \ref ePtx_Error_UnsupportedFeature if the PDF contains unrendered XFA fields.

 * - \ref ePtx_Error_Conformance if the document's conformance level is not supported


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Document* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_OpenWithFdfW(const TPtxSys_StreamDescriptor* pPdfStreamDesc,
                             const TPtxSys_StreamDescriptor* pFdfStreamDesc, const WCHAR* szPassword);
/**
 * @brief Create a new PDF document.
Documents with created with this method are writable and can be modified.

* @param[in,out] pStreamDesc The stream where the PDF document is stored.
Both, read and write access is required.

* @param[in] pConformance The required conformance level of the PDF document.
Adding pages or content from incompatible documents or using
incompatible features will lead to a conformance error.
When using `NULL`, the conformance is determined
automatically, based on the conformance of the input documents and the
requirements of the used features.
Note that for PDF/A document it is highly recommended to set an
output intent using \ref PtxPdf_Document_SetOutputIntent "".

* @param[in,out] pEncryption the optional encryption parameters


 * @return   the newly created document instance

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error writing to the stream.

 * - \ref ePtx_Error_Conformance If the conformance level is lower than 1.7 and Unicode passwords are specified.
In this context "a Unicode password" is essentially one containing characters that are not in the Windows ANSI encoding
(Windows Code Page 1252).


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Document* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_Create(const TPtxSys_StreamDescriptor* pStreamDesc, const TPtxPdf_Conformance* pConformance,
                       TPtxPdf_Encryption* pEncryption);
/**
 * @brief Create a new PDF document and an associated FDF.
Documents with created with this method are writable and can be modified.
When creating a document with this method,
all \ref TPtxPdfAnnots_MarkupAnnotation ""s - created or copied -
are stored as an FDF file to the <b>pFdfStreamDesc</b>.
In the output PDF (<b>pPdfStreamDesc</b>),
only annotations that are not \ref TPtxPdfAnnots_MarkupAnnotation ""s are stored.

* @param[in,out] pPdfStreamDesc The stream where the PDF document is stored.
Both, read and write access is required.
The resulting PDF document contains no \ref TPtxPdfAnnots_MarkupAnnotation ""s.

* @param[in,out] pFdfStreamDesc The stream where the document's \ref TPtxPdfAnnots_MarkupAnnotation ""s are stored in
the FDF format. Both, read and write access is required.

* @param[in] pConformance The required conformance level of the PDF document.
Adding pages or content from incompatible documents or using
incompatible features will lead to a conformance error.
When using `NULL`, the conformance is determined
automatically, based on the conformance of the input documents and the
requirements of the used features.
Note that for PDF/A document it is highly recommended to set an
output intent using \ref PtxPdf_Document_SetOutputIntent "".

* @param[in,out] pEncryption the optional encryption parameters


 * @return   the newly created document instance

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error writing to the <b>pPdfStreamDesc</b>

 * - \ref ePtx_Error_Conformance If the conformance level is lower than 1.7 and Unicode passwords are specified.
In this context "a Unicode password" is essentially one containing characters that are not in the Windows ANSI encoding
(Windows Code Page 1252).


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Document* PDFTOOLS_TOOLBOX_CALL PtxPdf_Document_CreateWithFdf(
    const TPtxSys_StreamDescriptor* pPdfStreamDesc, const TPtxSys_StreamDescriptor* pFdfStreamDesc,
    const TPtxPdf_Conformance* pConformance, TPtxPdf_Encryption* pEncryption);

/**
 * @brief the claimed conformance of the document.
This method only returns the claimed conformance level,
the document is not validated.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Conformance PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetConformance(TPtxPdf_Document* pDocument);
/**
 * @brief the metadata of the document.
If the document is writable,
the metadata object will be writable too and all changes to the
metadata object are reflected in the document.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdf_Metadata "" object is `NULL`

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdf_Metadata "" object belongs to a different document

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdf_Metadata "" object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Metadata* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetMetadata(TPtxPdf_Document* pDocument);
/**
 * @brief the metadata of the document.
If the document is writable,
the metadata object will be writable too and all changes to the
metadata object are reflected in the document.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.

* @param[in,out] pMetadata Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdf_Metadata "" object is `NULL`

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdf_Metadata "" object belongs to a different document

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdf_Metadata "" object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Document_SetMetadata(TPtxPdf_Document* pDocument,
                                                                               TPtxPdf_Metadata* pMetadata);
/**
 * @brief the pages of the document.
If the document is writable,
it is possible to append new pages to the end of the list.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_PageList* PDFTOOLS_TOOLBOX_CALL PtxPdf_Document_GetPages(TPtxPdf_Document* pDocument);
/**
 * @brief the output intent of the document.
The output intent specifies a color profile that characterizes the intended output device.
It is used to render device colors on devices other than the intended output device.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_UnsupportedOperation if an output intent has been set already

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_IccBasedColorSpace "" object is `NULL`

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_IccBasedColorSpace "" object has already been
closed

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_IccBasedColorSpace "" object> is associated with a
different document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_IccBasedColorSpace* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetOutputIntent(TPtxPdf_Document* pDocument);
/**
 * @brief the output intent of the document.
The output intent specifies a color profile that characterizes the intended output device.
It is used to render device colors on devices other than the intended output device.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.

* @param[in,out] pOutputIntent Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_UnsupportedOperation if an output intent has been set already

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_IccBasedColorSpace "" object is `NULL`

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_IccBasedColorSpace "" object has already been
closed

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_IccBasedColorSpace "" object> is associated with a
different document


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_SetOutputIntent(TPtxPdf_Document* pDocument, TPtxPdfContent_IccBasedColorSpace* pOutputIntent);
/**
 * @brief The form fields of the document
This list contains all AcroForm form fields that belong to this document.
Adding to this list results in an error:
  - <b>IllegalState</b> if the list has already been closed
  - <b>UnsupportedOperation</b> if the document is read-only
  - <b>IllegalArgument</b>
    - if the given form field node is `NULL`
    - if the given form field node has already been closed
    - if the given form field node does not belong to the same document as the list
    - if the given form field node has already been added to a form field node list
    - if the given form field node's identifier equals an identifier of one of the form field nodes in this list
This list does not support removing elements or setting elements or clearing.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_Corrupt if the document contains corrupt form fields


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfForms_FieldNodeMap* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetFormFields(TPtxPdf_Document* pDocument);
/**
 * @brief The signature fields of the document
Signature fields are a special kind of form fields,
that can contain digital signatures.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfForms_SignatureFieldList* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetSignatureFields(TPtxPdf_Document* pDocument);
/**
 * @brief plain embedded files

This list contains plain embedded files, i.e., files that are embedded
in this document without having a specific association (\ref PtxPdf_Document_GetAssociatedFiles ""),
and without being contained in any \ref TPtxPdfAnnots_FileAttachment "".

If the document is writable, then it is possible to append new file
references to the list.
Every file reference object can occur at most once in this list.

For PDF/A-3 documents, appending to this list results in a <b>Conformance</b> error.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_FileReferenceList* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetPlainEmbeddedFiles(TPtxPdf_Document* pDocument);
/**
 * @brief document-associated files

This list contains associated files, whose associated object is the
document.

If the document is writable, then it is possible to append new file
references to the list.
Every file reference object can occur at most once in this list.

Appending to this list results in a <b>Conformance</b> error
if the document's conformance is neither PDF/A-3 nor can be upgraded to PDF 2.0.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_FileReferenceList* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetAssociatedFiles(TPtxPdf_Document* pDocument);
/**
 * @brief plain embedded, associated, and attached files

This read-only list contains the union of all plain embedded files,
associated files, and files contained in file attachment annotations.
This is the list of files contained in a PDF as presented in a PDF viewer.

This list does not support appending.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_FileReferenceList* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetAllEmbeddedFiles(TPtxPdf_Document* pDocument);
/**
The document outline, also known as "Bookmarks".

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_OutlineItemList* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetOutline(TPtxPdf_Document* pDocument);
/**
The destination that is displayed when the document is opened.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the destination is associated with a different document

 * - \ref ePtx_Error_IllegalArgument if the destination has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_Destination* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetOpenDestination(TPtxPdf_Document* pDocument);
/**
The destination that is displayed when the document is opened.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.

* @param[in,out] pOpenDestination Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the destination is associated with a different document

 * - \ref ePtx_Error_IllegalArgument if the destination has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_SetOpenDestination(TPtxPdf_Document* pDocument, TPtxPdfNav_Destination* pOpenDestination);
/**
The permissions in force for this document.
This property is `NULL` if the document is not encrypted.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.

* @param[out] pPermissions Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Document_GetPermissions(TPtxPdf_Document*   pDocument,
                                                                                  TPtxPdf_Permission* pPermissions);
/**
The settings to use when opening the document in a viewer.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if viewer settings are associated with a different document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_ViewerSettings* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_GetViewerSettings(TPtxPdf_Document* pDocument);
/**
The settings to use when opening the document in a viewer.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.

* @param[in,out] pViewerSettings Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if viewer settings are associated with a different document


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_Document_SetViewerSettings(TPtxPdf_Document* pDocument, TPtxPdfNav_ViewerSettings* pViewerSettings);
/**

Whether the document is linearized.

Linearization is also known as "Fast Web View" and is a way of optimizing PDFs so they can be
streamed into a client application. This helps online documents open almost instantly,
without having to wait for a large document to completely download.

* @param[in,out] pDocument Acts as a handle to the native object of type \ref TPtxPdf_Document.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Document_IsLinearized(TPtxPdf_Document* pDocument);

/**
 * @brief Close object.
 *
 * Close disposable objects by invoking this function.
 *
 * @param[in] pObject Disposable object.
 *
 * @return   \ref TRUE if the object was closed successfully; \ref FALSE if an error occured while closing the object.
 *           Retrieve the error code by calling \ref Ptx_GetLastError .
 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Document_Close(TPtxPdf_Document* pObject);
/******************************************************************************
 * Page
 *****************************************************************************/
/**
 * @brief Create an empty page
The page is associated with the given target document but not yet part of it.
It can be appended to the document's page list.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] pSize the page size


 * @return   the newly created page object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Page* PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_Create(TPtxPdf_Document*        pTargetDocument,
                                                                               const TPtxGeomReal_Size* pSize);
/**
 * @brief Copy a page
Copy a page object from an input document to the given <b>pTargetDocument</b>.
The returned object is associated with the given target document but not yet part of it.
It can be appended to the document's page list.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pPage a page of a different document

* @param[in,out] pOptions the copy options


 * @return   the copied page, associated with the current document.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the source document or writing to the target document

 * - \ref ePtx_Error_Corrupt The source document is corrupt

 * - \ref ePtx_Error_Conformance The conformance level of the source document is not compatible
with the conformance level of the target document.

 * - \ref ePtx_Error_Conformance The explicitly requested conformance level is PDF/A Level A
(\ref ePtxPdf_Conformance_PdfA1A "", \ref ePtxPdf_Conformance_PdfA2A "",
or \ref ePtxPdf_Conformance_PdfA3A "")
and the copy option \ref PtxPdf_PageCopyOptions_GetCopyLogicalStructure "" is not set.

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pPage</b> object is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pPage</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the argument <b>pOptions</b> has \ref PtxPdf_PageCopyOptions_GetFormFields ""
set to \ref ePtxPdfForms_FormFieldCopyStrategy_Copy "" and the <b>pTargetDocument</b> contains form fields that have
either been copied explicitly with \ref PtxPdfForms_FieldNode_Copy "" or created with \ref PtxPdfForms_CheckBox_Create
"", \ref PtxPdfForms_ComboBox_Create "", \ref PtxPdfForms_CombTextField_Create "", \ref
PtxPdfForms_GeneralTextField_Create "", \ref PtxPdfForms_ListBox_Create "", \ref PtxPdfForms_RadioButtonGroup_Create "",
or \ref PtxPdfForms_SubForm_Create "".

 * - \ref ePtx_Error_IllegalArgument if the argument <b>pOptions</b> has \ref
PtxPdf_PageCopyOptions_GetUnsignedSignatures "" set to \ref ePtxPdf_CopyStrategy_Copy "" and the <b>pTargetDocument</b>
contains form fields that have either been copied explicitly with \ref PtxPdfForms_FieldNode_Copy "" or created with
\ref PtxPdfForms_CheckBox_Create "", \ref PtxPdfForms_ComboBox_Create "", \ref PtxPdfForms_CombTextField_Create "", \ref
PtxPdfForms_GeneralTextField_Create "", \ref PtxPdfForms_ListBox_Create "", \ref PtxPdfForms_RadioButtonGroup_Create "",
or \ref PtxPdfForms_SubForm_Create "".

 * - \ref ePtx_Error_IllegalArgument if <b>pOptions</b> has \ref PtxPdf_PageCopyOptions_GetCopyOutlineItems "" set to
\ref "TRUE" and the <b>pTargetDocument</b> contains outline items that have been copied explicitly with \ref
PtxPdfNav_OutlineItem_Copy "".


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Page* PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_Copy(TPtxPdf_Document*        pTargetDocument,
                                                                             TPtxPdf_Page*            pPage,
                                                                             TPtxPdf_PageCopyOptions* pOptions);
/**
 * @brief Update the page size to a specified rectangle.
Note that all page-related coordinates are normalized to the crop box
of the page. Updating the page size thus changes the coordinate system,
rendering all previously extracted coordinates invalid.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.

* @param[in] pRectangle the rectangle to update the page size to.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the owning document has already been closed

 * - \ref ePtx_Error_IllegalState if the page has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the page is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_UpdateSize(TPtxPdf_Page*                 pPage,
                                                                          const TPtxGeomReal_Rectangle* pRectangle);
/**
 * @brief Rotate the page by a multiple of 90 degrees.
* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.

* @param[in] iRotate the desired rotation


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the owning document has already been closed

 * - \ref ePtx_Error_IllegalState if the page has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the page is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_Rotate(TPtxPdf_Page* pPage, TPtxGeom_Rotation iRotate);

/**
 * @brief the visible size of the page (crop box).

The page size corresponds to the size of the crop box.
Since all coordinates are normalized to the origin of the crop box,
the normalized origin of the crop box is always (0,0) and thus
only the size is required.

The crop box defines the region to which the contents of
the page shall be clipped (cropped) when displayed or printed.
Unlike the other boxes,
the crop box has no defined meaning in terms of physical
page geometry or intended use;
it merely imposes clipping on the page contents.
However, in the absence of additional information
(such as imposition instructions specified in a JDF job ticket),
the crop box determines how the page's contents shall
be positioned on the output medium.
The default value is the page's media box.

This property cannot be `NULL`.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.

* @param[out] pSize Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the page has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetSize(TPtxPdf_Page* pPage, TPtxGeomReal_Size* pSize);
/**
 * @brief The media box of the page.

The media box defines the boundaries of the physical medium
on which the page is to be printed.
It may include any extended area surrounding the finished page for bleed,
printing marks, or other such purposes.
It may also include areas close to the edges of the medium that cannot be
marked because of physical limitations of the output device.
Content falling outside this boundary may safely be discarded without
affecting the meaning of the PDF file.

This property cannot be `NULL`.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.

* @param[out] pMediaBox Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the page has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetMediaBox(TPtxPdf_Page*           pPage,
                                                                           TPtxGeomReal_Rectangle* pMediaBox);
/**
 * @brief The bleed box of the page.

The bleed box (PDF 1.3) defines the region to which the contents of the page
shall be clipped when output in a production environment.
This may include any extra bleed area needed to accommodate
the physical limitations of cutting, folding, and trimming equipment.
The actual printed page may include printing marks that fall outside
the bleed box. The default value is the page's crop box.

This property is `NULL` if the page contains no explicit bleed box.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.

* @param[out] pBleedBox Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the page has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetBleedBox(TPtxPdf_Page*           pPage,
                                                                           TPtxGeomReal_Rectangle* pBleedBox);
/**
 * @brief The trim box of the page.

The trim box (PDF 1.3) defines the intended dimensions of
the finished page after trimming.
It may be smaller than the media box to allow for production-related content,
such as printing instructions, cut marks, or colour bars.
The default value is the page's crop box.

This property is `NULL` if the page contains no explicit trim box.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.

* @param[out] pTrimBox Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the page has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetTrimBox(TPtxPdf_Page*           pPage,
                                                                          TPtxGeomReal_Rectangle* pTrimBox);
/**
 * @brief The art box of the page.

The art box (PDF 1.3) defines the extent of the page's meaningful content
(including potential white-space) as intended by the page’s creator.
The default value is the page's crop box.

This property is `NULL` if the page contains no explicit art box.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.

* @param[out] pArtBox Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the page has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetArtBox(TPtxPdf_Page*           pPage,
                                                                         TPtxGeomReal_Rectangle* pArtBox);
/**
 * @brief the page content.
If the page is writable,
the content object can be used to apply new content on the page,
for example overlays or underlays.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the page has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Content* PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetContent(TPtxPdf_Page* pPage);
/**
 * @brief the list of this page's annotations.
Adding to this list results in an error:
  - <b>IllegalState</b> if the list has already been closed
  - <b>UnsupportedOperation</b> if the document is read-only
  - <b>IllegalArgument</b>
    - if the given annotation is `NULL`
    - if the given annotation object has already been closed
    - if the given annotation does not belong to the same document as the list
    - if the given annotation is already associated with a page
This list does not support removing or setting elements or clearing.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the page has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_AnnotationList* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Page_GetAnnotations(TPtxPdf_Page* pPage);
/**
 * @brief the list of this page's links.
Adding to this list results in an error:
  - <b>IllegalState</b> if the list has already been closed
  - <b>UnsupportedOperation</b> if the document is read-only
  - <b>IllegalArgument</b>
    - if the given link is `NULL`
    - if the given link object has already been closed
    - if the given link does not belong to the same document as the list
    - if the given link is already associated with a page
This list does not support removing or setting elements or clearing.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the page has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_LinkList* PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetLinks(TPtxPdf_Page* pPage);
/**
 * @brief the list of this page's form field widgets.
Adding to this list results in an error:
  - <b>IllegalState</b> if the list has already been closed
  - <b>UnsupportedOperation</b> if the document is read-only
  - <b>IllegalArgument</b>
    - if the given widget is `NULL`
    - if the given widget object has already been closed
    - if the given widget does not belong to the same document as the list
    - if the given widget is already associated with a page
This list does not support removing or setting elements or clearing.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the page has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfForms_WidgetList* PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetWidgets(TPtxPdf_Page* pPage);
/**
 * @brief the metadata of the page.

If the document is writable,
the metadata object will be writable too and all changes to the
metadata object are reflected in the document.

This property is `NULL` if the page has not metadata.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdf_Metadata "" object belongs to a different document

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdf_Metadata "" object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Metadata* PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetMetadata(TPtxPdf_Page* pPage);
/**
 * @brief the metadata of the page.

If the document is writable,
the metadata object will be writable too and all changes to the
metadata object are reflected in the document.

This property is `NULL` if the page has not metadata.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.

* @param[in,out] pMetadata Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdf_Metadata "" object belongs to a different document

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdf_Metadata "" object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_SetMetadata(TPtxPdf_Page*     pPage,
                                                                           TPtxPdf_Metadata* pMetadata);
/**
 * @brief Page label

The label for this page. A page does not have to have a label and
if it doesn't this property will be an empty string.

If it exists, a page label is designed to replace the
page number in visual presentations and consists of an
optional prefix and a number. Number can be in one of
several styles (arabic, Roman, alphabetic) and starts at
an arbitrary number for a range of pages.

Page labels are used to set distinct names or numbers, most often
for preface, appendices and similar sections of the document.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState If the document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetPageLabelA(TPtxPdf_Page* pPage, char* pBuffer,
                                                                               size_t nBufferSize);
/**
 * @brief Page label

The label for this page. A page does not have to have a label and
if it doesn't this property will be an empty string.

If it exists, a page label is designed to replace the
page number in visual presentations and consists of an
optional prefix and a number. Number can be in one of
several styles (arabic, Roman, alphabetic) and starts at
an arbitrary number for a range of pages.

Page labels are used to set distinct names or numbers, most often
for preface, appendices and similar sections of the document.

* @param[in,out] pPage Acts as a handle to the native object of type \ref TPtxPdf_Page.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState If the document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Page_GetPageLabelW(TPtxPdf_Page* pPage, WCHAR* pBuffer,
                                                                               size_t nBufferSize);

/******************************************************************************
 * Metadata
 *****************************************************************************/
/**
 * @brief Create a new metadata object
The newly created metadata object is associated with the target document but not
(yet) used as the document metadata.
The object can be used either as document metadata using \ref PtxPdf_Document_SetMetadata ""
or as page metadata using \ref PtxPdf_Page_SetMetadata "".

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] pXmp A stream containing an XMP file or `NULL` to
create an empty metadata object.


 * @return   the newly created metadata object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IO if the <b>pXmp</b> stream could not be read

 * - \ref ePtx_Error_Corrupt if the <b>pXmp</b> stream is corrupt


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Metadata* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Metadata_Create(TPtxPdf_Document* pTargetDocument, const TPtxSys_StreamDescriptor* pXmp);
/**
 * @brief Copy a metadata object
Copy a metadata object from an input document to the given <b>pTargetDocument</b>.
The returned object is associated with the target document but not
(yet) used as the document metadata.
The object can be used either as document metadata using \ref PtxPdf_Document_SetMetadata ""
or as page metadata using \ref PtxPdf_Page_SetMetadata "".

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pMetadata the metadata of a different document


 * @return   a metadata object with the same content, but associated with the current document.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the source document or writing to the target document

 * - \ref ePtx_Error_Corrupt The source document is corrupt

 * - \ref ePtx_Error_Conformance The conformance level of the source document is not compatible
with the conformance level of the target document.

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pMetadata</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pMetadata</b> object is not associated with an input document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Metadata* PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_Copy(TPtxPdf_Document* pTargetDocument,
                                                                                     TPtxPdf_Metadata* pMetadata);

/**
 * @brief The title of the document or resource.
This property corresponds to the "dc:title" entry
in the XMP metadata and to the "Title" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetTitleA(TPtxPdf_Metadata* pMetadata,
                                                                               char* pBuffer, size_t nBufferSize);
/**
 * @brief The title of the document or resource.
This property corresponds to the "dc:title" entry
in the XMP metadata and to the "Title" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetTitleW(TPtxPdf_Metadata* pMetadata,
                                                                               WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The title of the document or resource.
This property corresponds to the "dc:title" entry
in the XMP metadata and to the "Title" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] szTitle Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetTitleA(TPtxPdf_Metadata* pMetadata,
                                                                             const char*       szTitle);
/**
 * @brief The title of the document or resource.
This property corresponds to the "dc:title" entry
in the XMP metadata and to the "Title" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] szTitle Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetTitleW(TPtxPdf_Metadata* pMetadata,
                                                                             const WCHAR*      szTitle);
/**
 * @brief The name of the person who created the document or resource.
This property corresponds to the "dc:creator" entry
in the XMP metadata and to the "Author" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetAuthorA(TPtxPdf_Metadata* pMetadata,
                                                                                char* pBuffer, size_t nBufferSize);
/**
 * @brief The name of the person who created the document or resource.
This property corresponds to the "dc:creator" entry
in the XMP metadata and to the "Author" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetAuthorW(TPtxPdf_Metadata* pMetadata,
                                                                                WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The name of the person who created the document or resource.
This property corresponds to the "dc:creator" entry
in the XMP metadata and to the "Author" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] szAuthor Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetAuthorA(TPtxPdf_Metadata* pMetadata,
                                                                              const char*       szAuthor);
/**
 * @brief The name of the person who created the document or resource.
This property corresponds to the "dc:creator" entry
in the XMP metadata and to the "Author" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] szAuthor Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetAuthorW(TPtxPdf_Metadata* pMetadata,
                                                                              const WCHAR*      szAuthor);
/**
 * @brief The subject of the document or resource.
This property corresponds to the "dc:description" entry
in the XMP metadata and to the "Subject" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetSubjectA(TPtxPdf_Metadata* pMetadata,
                                                                                 char* pBuffer, size_t nBufferSize);
/**
 * @brief The subject of the document or resource.
This property corresponds to the "dc:description" entry
in the XMP metadata and to the "Subject" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetSubjectW(TPtxPdf_Metadata* pMetadata,
                                                                                 WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The subject of the document or resource.
This property corresponds to the "dc:description" entry
in the XMP metadata and to the "Subject" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] szSubject Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetSubjectA(TPtxPdf_Metadata* pMetadata,
                                                                               const char*       szSubject);
/**
 * @brief The subject of the document or resource.
This property corresponds to the "dc:description" entry
in the XMP metadata and to the "Subject" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] szSubject Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetSubjectW(TPtxPdf_Metadata* pMetadata,
                                                                               const WCHAR*      szSubject);
/**
 * @brief Keywords associated with the document or resource.

Keywords can be separated by:
  - carriage return / line feed
  - comma
  - semicolon
  - tab
  - double space

This property corresponds to the "pdf:Keywords" entry
in the XMP metadata and to the "Keywords" entry in
the document information dictionary.

Setting this property also sets the XMP property dc:subject
accordingly.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetKeywordsA(TPtxPdf_Metadata* pMetadata,
                                                                                  char* pBuffer, size_t nBufferSize);
/**
 * @brief Keywords associated with the document or resource.

Keywords can be separated by:
  - carriage return / line feed
  - comma
  - semicolon
  - tab
  - double space

This property corresponds to the "pdf:Keywords" entry
in the XMP metadata and to the "Keywords" entry in
the document information dictionary.

Setting this property also sets the XMP property dc:subject
accordingly.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetKeywordsW(TPtxPdf_Metadata* pMetadata,
                                                                                  WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief Keywords associated with the document or resource.

Keywords can be separated by:
  - carriage return / line feed
  - comma
  - semicolon
  - tab
  - double space

This property corresponds to the "pdf:Keywords" entry
in the XMP metadata and to the "Keywords" entry in
the document information dictionary.

Setting this property also sets the XMP property dc:subject
accordingly.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] szKeywords Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetKeywordsA(TPtxPdf_Metadata* pMetadata,
                                                                                const char*       szKeywords);
/**
 * @brief Keywords associated with the document or resource.

Keywords can be separated by:
  - carriage return / line feed
  - comma
  - semicolon
  - tab
  - double space

This property corresponds to the "pdf:Keywords" entry
in the XMP metadata and to the "Keywords" entry in
the document information dictionary.

Setting this property also sets the XMP property dc:subject
accordingly.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] szKeywords Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetKeywordsW(TPtxPdf_Metadata* pMetadata,
                                                                                const WCHAR*      szKeywords);
/**
 * @brief The original application that created the document.

The name of the first known tool used to create the document or resource.

This property corresponds to the "xmp:CreatorTool" entry
in the XMP metadata and to the "Creator" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetCreatorA(TPtxPdf_Metadata* pMetadata,
                                                                                 char* pBuffer, size_t nBufferSize);
/**
 * @brief The original application that created the document.

The name of the first known tool used to create the document or resource.

This property corresponds to the "xmp:CreatorTool" entry
in the XMP metadata and to the "Creator" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetCreatorW(TPtxPdf_Metadata* pMetadata,
                                                                                 WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The original application that created the document.

The name of the first known tool used to create the document or resource.

This property corresponds to the "xmp:CreatorTool" entry
in the XMP metadata and to the "Creator" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] szCreator Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetCreatorA(TPtxPdf_Metadata* pMetadata,
                                                                               const char*       szCreator);
/**
 * @brief The original application that created the document.

The name of the first known tool used to create the document or resource.

This property corresponds to the "xmp:CreatorTool" entry
in the XMP metadata and to the "Creator" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] szCreator Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetCreatorW(TPtxPdf_Metadata* pMetadata,
                                                                               const WCHAR*      szCreator);
/**
 * @brief The application that created the PDF

If the document was converted to PDF from another format,
the name of the PDF processor that converted it to PDF.

This property corresponds to the "pdf:Producer" entry
in the XMP metadata and to the "Producer" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetProducerA(TPtxPdf_Metadata* pMetadata,
                                                                                  char* pBuffer, size_t nBufferSize);
/**
 * @brief The application that created the PDF

If the document was converted to PDF from another format,
the name of the PDF processor that converted it to PDF.

This property corresponds to the "pdf:Producer" entry
in the XMP metadata and to the "Producer" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetProducerW(TPtxPdf_Metadata* pMetadata,
                                                                                  WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The date and time the document or resource was originally created.
This property corresponds to the "xmp:CreateDate" entry
in the XMP metadata and to the "CreationDate" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pCreationDate Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetCreationDate(TPtxPdf_Metadata* pMetadata,
                                                                                   TPtxSys_Date*     pCreationDate);
/**
 * @brief The date and time the document or resource was originally created.
This property corresponds to the "xmp:CreateDate" entry
in the XMP metadata and to the "CreationDate" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[in] pCreationDate Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_SetCreationDate(TPtxPdf_Metadata*   pMetadata,
                                                                                   const TPtxSys_Date* pCreationDate);
/**
 * @brief The date and time the document or resource was most recently modified.
This property corresponds to the "xmp:ModifyDate" entry
in the XMP metadata and to the "ModDate" entry in
the document information dictionary.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pModificationDate Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_Corrupt The date is corrupt.

 * - \ref ePtx_Error_IllegalState if the metadata have already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetModificationDate(TPtxPdf_Metadata* pMetadata,
                                                                                       TPtxSys_Date* pModificationDate);
/**
 * @brief The XMP metadata

The XMP metadata stream or `NULL` if there is none.

The stream is read-only.
To set the XMP stream of a metadata object  use the method
Document.CreateMetadata instead.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.

* @param[out] pOutStreamDesc Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_Metadata_GetXmp(TPtxPdf_Metadata*         pMetadata,
                                                                          TPtxSys_StreamDescriptor* pOutStreamDesc);
/**
 * @brief The custom entries in the document information dictionary

The standard entries "Title", "Author", "Subject", "Keywords",
"CreationDate", "ModDate", "Creator", "Producer" and "Trapped"
are not included in the map.
Any attempt to set a standard entry through this map will result in an error.
To get or set standard entries use the corresponding properties instead.

Note: The document information dictionary has been superseded by XMP metadata
and is deprecated in PDF 2.0.

* @param[in,out] pMetadata Acts as a handle to the native object of type \ref TPtxPdf_Metadata.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the metadata have already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtx_StringMap* PDFTOOLS_TOOLBOX_CALL
PtxPdf_Metadata_GetCustomEntries(TPtxPdf_Metadata* pMetadata);

/******************************************************************************
 * FileReference
 *****************************************************************************/
/**
 * @brief Create a new file reference object
The newly created file reference object belongs to the document but is not
(yet) used as an embedded file.
The object can be added to the list of embedded files or to the list of
associated files.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] pData A stream of the file to be added.
Read access is required.

* @param[in] szName The name to be used for the embedded file.
This name is presented to the user when viewing the list of embedded files.

* @param[in] szMediaType The mime type of the embedded file.
Default: "application/octet-stream".
Common values other than the default are "application/pdf", "application/xml", or "application/msword".

* @param[in] szDescription The description of the embedded file.
This is presented to the user when viewing the list of embedded files.

* @param[in] pModificationDate The modify date of the file.
Default: current time.


 * @return   the newly created file reference object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>szName</b> argument is an empty string

 * - \ref ePtx_Error_Conformance if the document's conformance is PDF/A-1

 * - \ref ePtx_Error_Conformance if the document's conformance is PDF/A-2 and the given <b>pData</b>
contains a file other than PDF/A-1 or PDF/A-2

 * - \ref ePtx_Error_IO Error reading from the stream.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_FileReference* PDFTOOLS_TOOLBOX_CALL PtxPdf_FileReference_CreateA(
    TPtxPdf_Document* pTargetDocument, const TPtxSys_StreamDescriptor* pData, const char* szName,
    const char* szMediaType, const char* szDescription, const TPtxSys_Date* pModificationDate);
/**
 * @brief Create a new file reference object
The newly created file reference object belongs to the document but is not
(yet) used as an embedded file.
The object can be added to the list of embedded files or to the list of
associated files.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] pData A stream of the file to be added.
Read access is required.

* @param[in] szName The name to be used for the embedded file.
This name is presented to the user when viewing the list of embedded files.

* @param[in] szMediaType The mime type of the embedded file.
Default: "application/octet-stream".
Common values other than the default are "application/pdf", "application/xml", or "application/msword".

* @param[in] szDescription The description of the embedded file.
This is presented to the user when viewing the list of embedded files.

* @param[in] pModificationDate The modify date of the file.
Default: current time.


 * @return   the newly created file reference object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>szName</b> argument is an empty string

 * - \ref ePtx_Error_Conformance if the document's conformance is PDF/A-1

 * - \ref ePtx_Error_Conformance if the document's conformance is PDF/A-2 and the given <b>pData</b>
contains a file other than PDF/A-1 or PDF/A-2

 * - \ref ePtx_Error_IO Error reading from the stream.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_FileReference* PDFTOOLS_TOOLBOX_CALL PtxPdf_FileReference_CreateW(
    TPtxPdf_Document* pTargetDocument, const TPtxSys_StreamDescriptor* pData, const WCHAR* szName,
    const WCHAR* szMediaType, const WCHAR* szDescription, const TPtxSys_Date* pModificationDate);
/**
 * @brief Copy a file reference object
Copy a file reference object from an input document to the given <b>pTargetDocument</b>.
The returned object is associated with the given target document but not yet part of it.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pFileReference a file reference object of a different document


 * @return   the copied file reference, associated with the current document

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pFileReference</b> argument is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument if the <b>pFileReference</b> object has already been closed

 * - \ref ePtx_Error_Conformance if the document's conformance is PDF/A-1

 * - \ref ePtx_Error_Conformance if the document's conformance is PDF/A-2 and the <b>pFileReference</b> object
contains an embedded file other than PDF/A-1 or PDF/A-2

 * - \ref ePtx_Error_IO Error reading from the input stream or writing to the output stream


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_FileReference* PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReference_Copy(TPtxPdf_Document* pTargetDocument, TPtxPdf_FileReference* pFileReference);

/**
 * @brief The file's association relationship
This property is `NULL` if the file is not associated with
any object.
When associating a file reference with an object such as the document
or a page, then this property defines the relationship between the
file and the object.
Typical values are:
  - <b>"Source"</b>:
used if this file is the original source material for the associated
content.
  - <b>"Data"</b>:
used if this file represents information used to derive a visual
presentation such as for a table or a graph.
  - <b>"Alternative"</b>:
used if this file is an alternative representation of content, for
example audio.
  - <b>"Supplement"</b>:
used if this file represents a supplemental representation of the
original source or data that may be more easily consumable (e.g., a
MathML version of an equation).
  - <b>"EncryptedPayload"</b>:
used if this file is an encrypted payload document that should be
displayed to the user if the PDF processor has the cryptographic filter
needed to decrypt the document.
  - <b>"FormData"</b>:
used if this file is the data associated with form fields of this PDF.
  - <b>"Schema"</b>:
used if this file is a schema definition for the associated object.
  - <b>"Unspecified"</b>:
used when the relationship is not known.

* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_FileReference_GetAssociationRelationshipA(
    TPtxPdf_FileReference* pFileReference, char* pBuffer, size_t nBufferSize);
/**
 * @brief The file's association relationship
This property is `NULL` if the file is not associated with
any object.
When associating a file reference with an object such as the document
or a page, then this property defines the relationship between the
file and the object.
Typical values are:
  - <b>"Source"</b>:
used if this file is the original source material for the associated
content.
  - <b>"Data"</b>:
used if this file represents information used to derive a visual
presentation such as for a table or a graph.
  - <b>"Alternative"</b>:
used if this file is an alternative representation of content, for
example audio.
  - <b>"Supplement"</b>:
used if this file represents a supplemental representation of the
original source or data that may be more easily consumable (e.g., a
MathML version of an equation).
  - <b>"EncryptedPayload"</b>:
used if this file is an encrypted payload document that should be
displayed to the user if the PDF processor has the cryptographic filter
needed to decrypt the document.
  - <b>"FormData"</b>:
used if this file is the data associated with form fields of this PDF.
  - <b>"Schema"</b>:
used if this file is a schema definition for the associated object.
  - <b>"Unspecified"</b>:
used when the relationship is not known.

* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdf_FileReference_GetAssociationRelationshipW(
    TPtxPdf_FileReference* pFileReference, WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The file's association relationship
This property is `NULL` if the file is not associated with
any object.
When associating a file reference with an object such as the document
or a page, then this property defines the relationship between the
file and the object.
Typical values are:
  - <b>"Source"</b>:
used if this file is the original source material for the associated
content.
  - <b>"Data"</b>:
used if this file represents information used to derive a visual
presentation such as for a table or a graph.
  - <b>"Alternative"</b>:
used if this file is an alternative representation of content, for
example audio.
  - <b>"Supplement"</b>:
used if this file represents a supplemental representation of the
original source or data that may be more easily consumable (e.g., a
MathML version of an equation).
  - <b>"EncryptedPayload"</b>:
used if this file is an encrypted payload document that should be
displayed to the user if the PDF processor has the cryptographic filter
needed to decrypt the document.
  - <b>"FormData"</b>:
used if this file is the data associated with form fields of this PDF.
  - <b>"Schema"</b>:
used if this file is a schema definition for the associated object.
  - <b>"Unspecified"</b>:
used when the relationship is not known.

* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[in] szAssociationRelationship Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_FileReference_SetAssociationRelationshipA(
    TPtxPdf_FileReference* pFileReference, const char* szAssociationRelationship);
/**
 * @brief The file's association relationship
This property is `NULL` if the file is not associated with
any object.
When associating a file reference with an object such as the document
or a page, then this property defines the relationship between the
file and the object.
Typical values are:
  - <b>"Source"</b>:
used if this file is the original source material for the associated
content.
  - <b>"Data"</b>:
used if this file represents information used to derive a visual
presentation such as for a table or a graph.
  - <b>"Alternative"</b>:
used if this file is an alternative representation of content, for
example audio.
  - <b>"Supplement"</b>:
used if this file represents a supplemental representation of the
original source or data that may be more easily consumable (e.g., a
MathML version of an equation).
  - <b>"EncryptedPayload"</b>:
used if this file is an encrypted payload document that should be
displayed to the user if the PDF processor has the cryptographic filter
needed to decrypt the document.
  - <b>"FormData"</b>:
used if this file is the data associated with form fields of this PDF.
  - <b>"Schema"</b>:
used if this file is a schema definition for the associated object.
  - <b>"Unspecified"</b>:
used when the relationship is not known.

* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[in] szAssociationRelationship Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdf_FileReference_SetAssociationRelationshipW(
    TPtxPdf_FileReference* pFileReference, const WCHAR* szAssociationRelationship);
/**
 * @brief The file's description
For embedded files, this is the description of the file presented to
the user in the list of embedded files.

* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReference_GetDescriptionA(TPtxPdf_FileReference* pFileReference, char* pBuffer, size_t nBufferSize);
/**
 * @brief The file's description
For embedded files, this is the description of the file presented to
the user in the list of embedded files.

* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReference_GetDescriptionW(TPtxPdf_FileReference* pFileReference, WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The file's MIME type
* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReference_GetMediaTypeA(TPtxPdf_FileReference* pFileReference, char* pBuffer, size_t nBufferSize);
/**
 * @brief The file's MIME type
* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReference_GetMediaTypeW(TPtxPdf_FileReference* pFileReference, WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The file's date of last modification
* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[out] pModificationDate Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReference_GetModificationDate(TPtxPdf_FileReference* pFileReference, TPtxSys_Date* pModificationDate);
/**
 * @brief The file name
For embedded files, this is the name presented to the user in a the list
of embedded files.

* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReference_GetNameA(TPtxPdf_FileReference* pFileReference, char* pBuffer, size_t nBufferSize);
/**
 * @brief The file name
For embedded files, this is the name presented to the user in a the list
of embedded files.

* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReference_GetNameW(TPtxPdf_FileReference* pFileReference, WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The file's stream
* @param[in,out] pFileReference Acts as a handle to the native object of type \ref TPtxPdf_FileReference.

* @param[out] pOutStreamDesc Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object or the owning document has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdf_FileReference_GetData(TPtxPdf_FileReference* pFileReference, TPtxSys_StreamDescriptor* pOutStreamDesc);

#ifdef __cplusplus
}
#endif

#endif /* PDFTOOLS_TOOLBOX_PTXPDF_H__ */
