/******************************************************************************
 *
 * File:            PdfTools_Toolbox_PtxPdfAnnots.h
 *
 * Description:     Sub Header file for Toolbox SDK
 *
 * Author:          PDF Tools AG
 *
 * Copyright:       Copyright (C) 2024 - 2024 PDF Tools AG, Switzerland
 *                  All rights reserved.
 *
 * Notice:          By downloading and using this artifact, you accept PDF Tools AG's
 *                  [license agreement](https://www.pdf-tools.com/license-agreement/),
 *                  [privacy policy](https://www.pdf-tools.com/privacy-policy/),
 *                  and allow PDF Tools AG to track your usage data.
 *
 *****************************************************************************/

#ifndef PDFTOOLS_TOOLBOX_PTXPDFANNOTS_H__
#define PDFTOOLS_TOOLBOX_PTXPDFANNOTS_H__

#ifndef PDFTOOLS_TOOLBOX_CALL
#if defined(WIN32)
#define PDFTOOLS_TOOLBOX_CALL __stdcall
#else
#define PDFTOOLS_TOOLBOX_CALL
#endif
#endif

#include "PdfTools_Toolbox_Types.h"
#include "PdfTools_Toolbox_PtxSys.h"

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef _UNICODE
#define PtxPdfAnnots_Annotation_GetId PtxPdfAnnots_Annotation_GetIdW

#define PtxPdfAnnots_MarkupInfo_GetAuthor  PtxPdfAnnots_MarkupInfo_GetAuthorW
#define PtxPdfAnnots_MarkupInfo_SetAuthor  PtxPdfAnnots_MarkupInfo_SetAuthorW
#define PtxPdfAnnots_MarkupInfo_GetSubject PtxPdfAnnots_MarkupInfo_GetSubjectW
#define PtxPdfAnnots_MarkupInfo_SetSubject PtxPdfAnnots_MarkupInfo_SetSubjectW
#define PtxPdfAnnots_MarkupInfo_GetContent PtxPdfAnnots_MarkupInfo_GetContentW
#define PtxPdfAnnots_MarkupInfo_SetContent PtxPdfAnnots_MarkupInfo_SetContentW

#define PtxPdfAnnots_StickyNote_Create PtxPdfAnnots_StickyNote_CreateW

#define PtxPdfAnnots_TextStamp_CreateRaw PtxPdfAnnots_TextStamp_CreateRawW

#define PtxPdfAnnots_FreeText_Create PtxPdfAnnots_FreeText_CreateW

#else
#define PtxPdfAnnots_Annotation_GetId PtxPdfAnnots_Annotation_GetIdA

#define PtxPdfAnnots_MarkupInfo_GetAuthor  PtxPdfAnnots_MarkupInfo_GetAuthorA
#define PtxPdfAnnots_MarkupInfo_SetAuthor  PtxPdfAnnots_MarkupInfo_SetAuthorA
#define PtxPdfAnnots_MarkupInfo_GetSubject PtxPdfAnnots_MarkupInfo_GetSubjectA
#define PtxPdfAnnots_MarkupInfo_SetSubject PtxPdfAnnots_MarkupInfo_SetSubjectA
#define PtxPdfAnnots_MarkupInfo_GetContent PtxPdfAnnots_MarkupInfo_GetContentA
#define PtxPdfAnnots_MarkupInfo_SetContent PtxPdfAnnots_MarkupInfo_SetContentA

#define PtxPdfAnnots_StickyNote_Create PtxPdfAnnots_StickyNote_CreateA

#define PtxPdfAnnots_TextStamp_CreateRaw PtxPdfAnnots_TextStamp_CreateRawA

#define PtxPdfAnnots_FreeText_Create PtxPdfAnnots_FreeText_CreateA

#endif

/******************************************************************************
 * Annotation
 *****************************************************************************/
/**
 * @brief Copy an annotation
Copy an annotation object from an input document to the given <b>pTargetDocument</b>.
The returned object is associated with the given target document but not yet part of it.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pAnnotation the annotation to be copied to the <b>pTargetDocument</b>


 * @return   the copied annotation, associated with the <b>pTargetDocument</b>

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pAnnotation</b> is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pAnnotation</b> object has already been
closed

 * - \ref ePtx_Error_Conformance The conformance level of the input document is not compatible
with the conformance level of the <b>pTargetDocument</b>.

 * - \ref ePtx_Error_IO Error reading from the input document or writing to the output document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Annotation* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Annotation_Copy(TPtxPdf_Document* pTargetDocument, TPtxPdfAnnots_Annotation* pAnnotation);

/**
 * @brief The location on the page
* @param[in,out] pAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_Annotation.

* @param[out] pBoundingBox Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Annotation_GetBoundingBox(TPtxPdfAnnots_Annotation* pAnnotation, TPtxGeomReal_Rectangle* pBoundingBox);
/**
 * @brief The annotation's visibility
If \ref "TRUE" then the annotation is present, but is invisible and not available for user interaction.
Depending on the \ref PtxPdfAnnots_Annotation_GetNoPrint "" property it will, however, still be visible when printing.

* @param[in,out] pAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_Annotation.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Annotation_GetHidden(TPtxPdfAnnots_Annotation* pAnnotation);
/**
 * @brief The annotation's visibility when printing
If \ref "TRUE" then the annotation is not present in a print output of the document.

* @param[in,out] pAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_Annotation.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Annotation_GetNoPrint(TPtxPdfAnnots_Annotation* pAnnotation);
/**
 * @brief The annotation's scaling behavior
If \ref "TRUE" then the annotation's visual appearance does not scale with the zoom factor of a PDF viewer.

* @param[in,out] pAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_Annotation.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Annotation_GetNoZoom(TPtxPdfAnnots_Annotation* pAnnotation);
/**
 * @brief The annotation's rotation behavior
If \ref "TRUE" then the annotation's visual appearance does not rotate with the rotation set in a PDF viewer.

* @param[in,out] pAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_Annotation.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Annotation_GetNoRotate(TPtxPdfAnnots_Annotation* pAnnotation);
/**
 * @brief The annotation identifier
A text string uniquely identifying it among all the annotations on its page.
When creating annotations using the SDK, a unique ID is set automatically.

* @param[in,out] pAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_Annotation.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Annotation_GetIdA(TPtxPdfAnnots_Annotation* pAnnotation, char* pBuffer, size_t nBufferSize);
/**
 * @brief The annotation identifier
A text string uniquely identifying it among all the annotations on its page.
When creating annotations using the SDK, a unique ID is set automatically.

* @param[in,out] pAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_Annotation.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Annotation_GetIdW(TPtxPdfAnnots_Annotation* pAnnotation, WCHAR* pBuffer, size_t nBufferSize);

/**
 * @brief Get actual derived type of base type \ref TPtxPdfAnnots_Annotation.
 *
 * This function is invoked prior to downcasting to ascertain the derived object type.
 *
 * @param[in,out] pAnnotation Acts as a handle to a native object.
 *
 * @return The item of the enumeration \ref TPtxPdfAnnots_AnnotationType that refers to the actual derived type. `0` in
 * case of an error.
 *
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 *       Get the error message with \ref Ptx_GetLastErrorMessage.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_AnnotationType PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Annotation_GetType(TPtxPdfAnnots_Annotation* pAnnotation);
/******************************************************************************
 * AnnotationList
 *****************************************************************************/
/**
 * @brief Get the number of elements in the list.
* @param[in,out] pAnnotationList Acts as a handle to the native object of type \ref TPtxPdfAnnots_AnnotationList.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_AnnotationList_GetCount(TPtxPdfAnnots_AnnotationList* pAnnotationList);
/**
 * @brief Returns the element at the specified position in the given list.
* @param[in,out] pAnnotationList Acts as a handle to the native object of type \ref TPtxPdfAnnots_AnnotationList.

* @param[in] iIndex

 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Annotation* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_AnnotationList_Get(TPtxPdfAnnots_AnnotationList* pAnnotationList, int iIndex);
/**
 * @brief Add an element to the end of the list.
* @param[in,out] pAnnotationList Acts as a handle to the native object of type \ref TPtxPdfAnnots_AnnotationList.

* @param[in,out] pAnnotation

 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_AnnotationList_Add(TPtxPdfAnnots_AnnotationList* pAnnotationList, TPtxPdfAnnots_Annotation* pAnnotation);

/******************************************************************************
 * MarkupInfo
 *****************************************************************************/
/**
 * @brief The date of creation
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[out] pCreationDate Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_GetCreationDate(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, TPtxSys_Date* pCreationDate);
/**
 * @brief The date of creation
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[in] pCreationDate Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_SetCreationDate(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, const TPtxSys_Date* pCreationDate);
/**
 * @brief The date of last modification
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[out] pModificationDate Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_GetModificationDate(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, TPtxSys_Date* pModificationDate);
/**
 * @brief The date of last modification
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[in] pModificationDate Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfAnnots_MarkupInfo_SetModificationDate(
    TPtxPdfAnnots_MarkupInfo* pMarkupInfo, const TPtxSys_Date* pModificationDate);
/**
 * @brief Whether the content can be modified
This does not restrict the modification of other aspects of the annotation or its deletion.

* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_GetLocked(TPtxPdfAnnots_MarkupInfo* pMarkupInfo);
/**
 * @brief Whether the content can be modified
This does not restrict the modification of other aspects of the annotation or its deletion.

* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[in] bLocked Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_SetLocked(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, BOOL bLocked);
/**
 * @brief The author
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_GetAuthorA(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, char* pBuffer, size_t nBufferSize);
/**
 * @brief The author
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_GetAuthorW(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The author
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[in] szAuthor Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_SetAuthorA(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, const char* szAuthor);
/**
 * @brief The author
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[in] szAuthor Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_SetAuthorW(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, const WCHAR* szAuthor);
/**
 * @brief The subject
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_GetSubjectA(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, char* pBuffer, size_t nBufferSize);
/**
 * @brief The subject
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_GetSubjectW(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The subject
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[in] szSubject Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_SetSubjectA(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, const char* szSubject);
/**
 * @brief The subject
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[in] szSubject Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_SetSubjectW(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, const WCHAR* szSubject);
/**
 * @brief The information content
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_GetContentA(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, char* pBuffer, size_t nBufferSize);
/**
 * @brief The information content
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_GetContentW(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The information content
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[in] szContent Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_SetContentA(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, const char* szContent);
/**
 * @brief The information content
* @param[in,out] pMarkupInfo Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfo.

* @param[in] szContent Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfo_SetContentW(TPtxPdfAnnots_MarkupInfo* pMarkupInfo, const WCHAR* szContent);

/******************************************************************************
 * MarkupInfoList
 *****************************************************************************/
/**
 * @brief Get the number of elements in the list.
* @param[in,out] pMarkupInfoList Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfoList.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfoList_GetCount(TPtxPdfAnnots_MarkupInfoList* pMarkupInfoList);
/**
 * @brief Returns the element at the specified position in the given list.
* @param[in,out] pMarkupInfoList Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupInfoList.

* @param[in] iIndex

 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_MarkupInfo* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupInfoList_Get(TPtxPdfAnnots_MarkupInfoList* pMarkupInfoList, int iIndex);

/******************************************************************************
 * Popup
 *****************************************************************************/
/**
 * @brief The pop-up's visibility state
* @param[in,out] pPopup Acts as a handle to the native object of type \ref TPtxPdfAnnots_Popup.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfAnnots_Popup_IsOpen(TPtxPdfAnnots_Popup* pPopup);
/**
 * @brief The pop-up location
* @param[in,out] pPopup Acts as a handle to the native object of type \ref TPtxPdfAnnots_Popup.

* @param[out] pBoundingBox Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Popup_GetBoundingBox(TPtxPdfAnnots_Popup* pPopup, TPtxGeomReal_Rectangle* pBoundingBox);

/******************************************************************************
 * MarkupAnnotation
 *****************************************************************************/
/**
 * @brief Whether the markup annotation can be modified
This does not restrict modification of the markup annotation's content.

* @param[in,out] pMarkupAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupAnnotation.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupAnnotation_GetLocked(TPtxPdfAnnots_MarkupAnnotation* pMarkupAnnotation);
/**
 * @brief The information content of this markup
* @param[in,out] pMarkupAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupAnnotation.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_MarkupInfo* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupAnnotation_GetInfo(TPtxPdfAnnots_MarkupAnnotation* pMarkupAnnotation);
/**
 * @brief The replies to this markup
* @param[in,out] pMarkupAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_MarkupAnnotation.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_MarkupInfoList* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupAnnotation_GetReplies(TPtxPdfAnnots_MarkupAnnotation* pMarkupAnnotation);

/**
 * @brief Get actual derived type of base type \ref TPtxPdfAnnots_MarkupAnnotation.
 *
 * This function is invoked prior to downcasting to ascertain the derived object type.
 *
 * @param[in,out] pMarkupAnnotation Acts as a handle to a native object.
 *
 * @return The item of the enumeration \ref TPtxPdfAnnots_MarkupAnnotationType that refers to the actual derived type.
 * `0` in case of an error.
 *
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 *       Get the error message with \ref Ptx_GetLastErrorMessage.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_MarkupAnnotationType PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_MarkupAnnotation_GetType(TPtxPdfAnnots_MarkupAnnotation* pMarkupAnnotation);
/******************************************************************************
 * StickyNote
 *****************************************************************************/
/**
 * @brief Create a sticky note annotation.
The returned sticky note annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in] pTopLeft The location of the annotation's upper left corner on the page

* @param[in] szContent The text content

* @param[in,out] pPaint The paint for the sticky note icon.


 * @return   The newly created sticky note annotation

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the color space of the <b>pPaint</b> argument is not a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> has a non-`NULL`\ref PtxPdfContent_Paint_GetTransparency ""
with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument is not associated with the <b>pTargetDocument</b>


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_StickyNote* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_StickyNote_CreateA(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Point* pTopLeft,
                                const char* szContent, TPtxPdfContent_Paint* pPaint);
/**
 * @brief Create a sticky note annotation.
The returned sticky note annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in] pTopLeft The location of the annotation's upper left corner on the page

* @param[in] szContent The text content

* @param[in,out] pPaint The paint for the sticky note icon.


 * @return   The newly created sticky note annotation

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the color space of the <b>pPaint</b> argument is not a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> has a non-`NULL`\ref PtxPdfContent_Paint_GetTransparency ""
with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument is not associated with the <b>pTargetDocument</b>


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_StickyNote* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_StickyNote_CreateW(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Point* pTopLeft,
                                const WCHAR* szContent, TPtxPdfContent_Paint* pPaint);

/**
 * @brief The paint for the icon and the popup
* @param[in,out] pStickyNote Acts as a handle to the native object of type \ref TPtxPdfAnnots_StickyNote.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt and the annotation's paint cannot be read


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_StickyNote_GetPaint(TPtxPdfAnnots_StickyNote* pStickyNote);
/**
 * @brief The pop-up
* @param[in,out] pStickyNote Acts as a handle to the native object of type \ref TPtxPdfAnnots_StickyNote.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Popup* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_StickyNote_GetPopup(TPtxPdfAnnots_StickyNote* pStickyNote);

/******************************************************************************
 * FileAttachment
 *****************************************************************************/
/**
 * @brief Create a file attachment annotation.
The returned file attachment annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in] pTopLeft The location of the annotation's upper left corner on the page.

* @param[in,out] pAttachedFile The file to be attached.

* @param[in,out] pPaint This paint for the file attachment icon.


 * @return   The newly created file attachment annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the color space of the <b>pPaint</b> argument is not a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> has a non-`NULL`\ref PtxPdfContent_Paint_GetTransparency ""
with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument is not associated with the <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pAttachedFile</b> argument is not associated with the
<b>pTargetDocument</b>

 * - \ref ePtx_Error_Exists if the <b>pAttachedFile</b> argument is already used as a plain embedded file, an associated
file, or in a file attachment.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_FileAttachment* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_FileAttachment_Create(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Point* pTopLeft,
                                   TPtxPdf_FileReference* pAttachedFile, TPtxPdfContent_Paint* pPaint);

/**
 * @brief The displayed icon
* @param[in,out] pFileAttachment Acts as a handle to the native object of type \ref TPtxPdfAnnots_FileAttachment.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt the document is corrupt and the file reference cannot be read


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_FileAttachmentIcon PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_FileAttachment_GetIcon(TPtxPdfAnnots_FileAttachment* pFileAttachment);
/**
 * @brief The paint for the icon and the popup
* @param[in,out] pFileAttachment Acts as a handle to the native object of type \ref TPtxPdfAnnots_FileAttachment.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt and the annotation's paint cannot be read


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_FileAttachment_GetPaint(TPtxPdfAnnots_FileAttachment* pFileAttachment);
/**
 * @brief The embedded file
* @param[in,out] pFileAttachment Acts as a handle to the native object of type \ref TPtxPdfAnnots_FileAttachment.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_FileReference* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_FileAttachment_GetAttachedFile(TPtxPdfAnnots_FileAttachment* pFileAttachment);
/**
 * @brief The pop-up
* @param[in,out] pFileAttachment Acts as a handle to the native object of type \ref TPtxPdfAnnots_FileAttachment.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Popup* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_FileAttachment_GetPopup(TPtxPdfAnnots_FileAttachment* pFileAttachment);

/******************************************************************************
 * Stamp
 *****************************************************************************/
/**
 * @brief The pop-up
* @param[in,out] pStamp Acts as a handle to the native object of type \ref TPtxPdfAnnots_Stamp.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Popup* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Stamp_GetPopup(TPtxPdfAnnots_Stamp* pStamp);
/**
 * @brief The paint for the popup
* @param[in,out] pStamp Acts as a handle to the native object of type \ref TPtxPdfAnnots_Stamp.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt and the annotation's paint cannot be read


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Stamp_GetPopupPaint(TPtxPdfAnnots_Stamp* pStamp);

/**
 * @brief Get actual derived type of base type \ref TPtxPdfAnnots_Stamp.
 *
 * This function is invoked prior to downcasting to ascertain the derived object type.
 *
 * @param[in,out] pStamp Acts as a handle to a native object.
 *
 * @return The item of the enumeration \ref TPtxPdfAnnots_StampType that refers to the actual derived type. `0` in case
 * of an error.
 *
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 *       Get the error message with \ref Ptx_GetLastErrorMessage.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_StampType PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_Stamp_GetType(TPtxPdfAnnots_Stamp* pStamp);
/******************************************************************************
 * TextStamp
 *****************************************************************************/
/**
 * @brief Create a text stamp annotation.
The width of the annotation is computed from the given stamp text.
The returned text stamp annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in] pTopLeft The location of the annotation's upper left corner on the page.

* @param[in] pHeight The height of the annotation.

* @param[in] iTextType The text stamp type.

* @param[in] szText The text to be shown in this stamp.


 * @return   The newly created text stamp annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>iTextType</b> argument is \ref
ePtxPdfAnnots_TextStampType_CustomStampText ""

 * - \ref ePtx_Error_IllegalArgument if the <b>szText</b> argument is empty


 * @deprecated Deprecated in Version 4.2. Use static method TextStamp.Create instead.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_TextStamp* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_TextStamp_CreateRawA(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Point* pTopLeft,
                                  const double* pHeight, TPtxPdfAnnots_TextStampType iTextType, const char* szText);
/**
 * @brief Create a text stamp annotation.
The width of the annotation is computed from the given stamp text.
The returned text stamp annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in] pTopLeft The location of the annotation's upper left corner on the page.

* @param[in] pHeight The height of the annotation.

* @param[in] iTextType The text stamp type.

* @param[in] szText The text to be shown in this stamp.


 * @return   The newly created text stamp annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>iTextType</b> argument is \ref
ePtxPdfAnnots_TextStampType_CustomStampText ""

 * - \ref ePtx_Error_IllegalArgument if the <b>szText</b> argument is empty


 * @deprecated Deprecated in Version 4.2. Use static method TextStamp.Create instead.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_TextStamp* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_TextStamp_CreateRawW(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Point* pTopLeft,
                                  const double* pHeight, TPtxPdfAnnots_TextStampType iTextType, const WCHAR* szText);
/**
 * @brief Create a text stamp annotation.
The width of the annotation is computed based on the given <b>iTextType</b>.
The returned text stamp annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in] pTopLeft The location of the annotation's upper left corner on the page.

* @param[in] pHeight The height of the annotation.

* @param[in] iTextType The text stamp type.


 * @return   The newly created text stamp annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>iTextType</b> argument is \ref
ePtxPdfAnnots_TextStampType_CustomStampText ""


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_TextStamp* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_TextStamp_Create(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Point* pTopLeft,
                              const double* pHeight, TPtxPdfAnnots_TextStampType iTextType);

/**
 * @brief The displayed text
This defines a predefined text for this text stamp.

* @param[in,out] pTextStamp Acts as a handle to the native object of type \ref TPtxPdfAnnots_TextStamp.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_TextStampType PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_TextStamp_GetTextType(TPtxPdfAnnots_TextStamp* pTextStamp);

/******************************************************************************
 * CustomStamp
 *****************************************************************************/
/**
 * @brief Create a custom stamp annotation.
The returned custom stamp annotation's appearance has an empty content with size equal to the given <b>pBoundingBox</b>,
and with coordinate origin located at the bottom left corner.
Use a \ref TPtxPdfContent_ContentGenerator "" to generate the stamp's content prior to adding the stamp annotation to a
page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in] pBoundingBox The location of the annotation on the page.


 * @return   The newly created custom stamp annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_CustomStamp* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_CustomStamp_Create(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Rectangle* pBoundingBox);

/**
 * @brief The custom stamp's visual appearance
* @param[in,out] pCustomStamp Acts as a handle to the native object of type \ref TPtxPdfAnnots_CustomStamp.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Group* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_CustomStamp_GetAppearance(TPtxPdfAnnots_CustomStamp* pCustomStamp);

/******************************************************************************
 * FreeText
 *****************************************************************************/
/**
 * @brief Create a free-text annotation.
The returned free-text annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in] pBoundingBox The location on the page.

* @param[in] szContent The text content.

* @param[in,out] pPaint This paint used for the background of the free text annotation.
If `NULL`, the background is transparent.

* @param[in,out] pStroke The stroking parameters used for stroking the rectangle.
The stroking paint is used as the annotation's main paint.


 * @return   The newly created free text annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> has a \ref PtxPdfContent_Paint_GetColorSpace "" other than a
device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> has a non-`NULL`\ref PtxPdfContent_Paint_GetTransparency ""
with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument is not associated with the <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a \ref
PtxPdfContent_Paint_GetColorSpace "" other than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetLineCapStyle "" is other than
\ref ePtxPdfContent_LineCapStyle_Butt ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetLineJoinStyle "" is other than
\ref ePtxPdfContent_LineJoinStyle_Miter ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetMiterLimit "" is other than 10

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetDashPhase "" is other than 0


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_FreeText* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_FreeText_CreateA(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Rectangle* pBoundingBox,
                              const char* szContent, TPtxPdfContent_Paint* pPaint, TPtxPdfContent_Stroke* pStroke);
/**
 * @brief Create a free-text annotation.
The returned free-text annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in] pBoundingBox The location on the page.

* @param[in] szContent The text content.

* @param[in,out] pPaint This paint used for the background of the free text annotation.
If `NULL`, the background is transparent.

* @param[in,out] pStroke The stroking parameters used for stroking the rectangle.
The stroking paint is used as the annotation's main paint.


 * @return   The newly created free text annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> has a \ref PtxPdfContent_Paint_GetColorSpace "" other than a
device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> has a non-`NULL`\ref PtxPdfContent_Paint_GetTransparency ""
with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument is not associated with the <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a \ref
PtxPdfContent_Paint_GetColorSpace "" other than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetLineCapStyle "" is other than
\ref ePtxPdfContent_LineCapStyle_Butt ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetLineJoinStyle "" is other than
\ref ePtxPdfContent_LineJoinStyle_Miter ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetMiterLimit "" is other than 10

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetDashPhase "" is other than 0


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_FreeText* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_FreeText_CreateW(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Rectangle* pBoundingBox,
                              const WCHAR* szContent, TPtxPdfContent_Paint* pPaint, TPtxPdfContent_Stroke* pStroke);

/**
 * @brief The font size

Default value: 12.

Note: the font size has no effect when rich text is used

* @param[in,out] pFreeText Acts as a handle to the native object of type \ref TPtxPdfAnnots_FreeText.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the given value is smaller than <b>0.0</b>


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_FreeText_GetFontSize(TPtxPdfAnnots_FreeText* pFreeText);
/**
 * @brief The font size

Default value: 12.

Note: the font size has no effect when rich text is used

* @param[in,out] pFreeText Acts as a handle to the native object of type \ref TPtxPdfAnnots_FreeText.

* @param[in] dFontSize Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the given value is smaller than <b>0.0</b>


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfAnnots_FreeText_SetFontSize(TPtxPdfAnnots_FreeText* pFreeText,
                                                                                     double                  dFontSize);
/**
 * @brief The text alignment
* @param[in,out] pFreeText Acts as a handle to the native object of type \ref TPtxPdfAnnots_FreeText.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxGeom_HorizontalAlignment PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_FreeText_GetAlignment(TPtxPdfAnnots_FreeText* pFreeText);
/**
 * @brief The paint for the text background and the popup
* @param[in,out] pFreeText Acts as a handle to the native object of type \ref TPtxPdfAnnots_FreeText.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt and the annotation's paint cannot be read


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_FreeText_GetPaint(TPtxPdfAnnots_FreeText* pFreeText);

/******************************************************************************
 * DrawingAnnotation
 *****************************************************************************/
/**
 * @brief The paint for line stroking and for the popup
* @param[in,out] pDrawingAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_DrawingAnnotation.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt and the annotation's paint cannot be read


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_DrawingAnnotation_GetPaint(TPtxPdfAnnots_DrawingAnnotation* pDrawingAnnotation);
/**
 * @brief The pop-up
* @param[in,out] pDrawingAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_DrawingAnnotation.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Popup* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_DrawingAnnotation_GetPopup(TPtxPdfAnnots_DrawingAnnotation* pDrawingAnnotation);

/**
 * @brief Get actual derived type of base type \ref TPtxPdfAnnots_DrawingAnnotation.
 *
 * This function is invoked prior to downcasting to ascertain the derived object type.
 *
 * @param[in,out] pDrawingAnnotation Acts as a handle to a native object.
 *
 * @return The item of the enumeration \ref TPtxPdfAnnots_DrawingAnnotationType that refers to the actual derived type.
 * `0` in case of an error.
 *
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 *       Get the error message with \ref Ptx_GetLastErrorMessage.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_DrawingAnnotationType PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_DrawingAnnotation_GetType(TPtxPdfAnnots_DrawingAnnotation* pDrawingAnnotation);
/******************************************************************************
 * LineAnnotation
 *****************************************************************************/
/**
 * @brief Create a line annotation.
The returned line annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in] pStart The line's start point.

* @param[in] pEnd The line's end point.

* @param[in,out] pStroke The stroking parameters used for stroking the line.
The stroking paint is used as the annotation's main paint.


 * @return   The newly created line annotation.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument is not associated with the <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a \ref
PtxPdfContent_Paint_GetColorSpace "" other than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineCapStyle "" is
other than \ref ePtxPdfContent_LineCapStyle_Butt ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineJoinStyle "" is
other than \ref ePtxPdfContent_LineJoinStyle_Miter ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetMiterLimit "" is
other than 10

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetDashPhase "" is other
than 0


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_LineAnnotation* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_LineAnnotation_Create(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Point* pStart,
                                   const TPtxGeomReal_Point* pEnd, TPtxPdfContent_Stroke* pStroke);

/**
 * @brief The line's starting point
* @param[in,out] pLineAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_LineAnnotation.

* @param[out] pStart Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_LineAnnotation_GetStart(TPtxPdfAnnots_LineAnnotation* pLineAnnotation, TPtxGeomReal_Point* pStart);
/**
 * @brief The line's ending point
* @param[in,out] pLineAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_LineAnnotation.

* @param[out] pEnd Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_LineAnnotation_GetEnd(TPtxPdfAnnots_LineAnnotation* pLineAnnotation, TPtxGeomReal_Point* pEnd);
/**
 * @brief The starting point's style
* @param[in,out] pLineAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_LineAnnotation.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_LineEnding PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_LineAnnotation_GetStartStyle(TPtxPdfAnnots_LineAnnotation* pLineAnnotation);
/**
 * @brief The ending point's style
* @param[in,out] pLineAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_LineAnnotation.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_LineEnding PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_LineAnnotation_GetEndStyle(TPtxPdfAnnots_LineAnnotation* pLineAnnotation);
/**
 * @brief The line ending filling paint
This paint applies to both the starting end the ending point.

* @param[in,out] pLineAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_LineAnnotation.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_LineAnnotation_GetLineEndingFill(TPtxPdfAnnots_LineAnnotation* pLineAnnotation);

/******************************************************************************
 * InkAnnotation
 *****************************************************************************/
/**
 * @brief Create a ink annotation.
The returned ink annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in,out] pPath The path of the free-hand drawing.

* @param[in,out] pStroke The stroking parameters used for stroking the line.
The stroking paint is used as the annotation's main paint.


 * @return   The newly created ink annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pPath</b> argument has not been constructed with a \ref
TPtxPdfContent_PathGenerator ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b>'s conformance is not PDF 2.0 and the <b>pPath</b>
argument contains curve-to operations

 * - \ref ePtx_Error_IllegalArgument if the <b>pPath</b> argument contains rectangle operations

 * - \ref ePtx_Error_UnsupportedFeature if the <b>pTargetDocument</b>'s conformance is PDF 2.0 and the <b>pPath</b>
argument contains curve-to operations

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a \ref
PtxPdfContent_Paint_GetColorSpace "" other than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineCapStyle "" is
other than \ref ePtxPdfContent_LineCapStyle_Butt ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineJoinStyle "" is
other than \ref ePtxPdfContent_LineJoinStyle_Miter ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetMiterLimit "" is
other than 10

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's dash array is not empty

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetDashPhase "" is other
than 0

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument is not associated with the <b>pTargetDocument</b>


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_InkAnnotation* PDFTOOLS_TOOLBOX_CALL PtxPdfAnnots_InkAnnotation_Create(
    TPtxPdf_Document* pTargetDocument, TPtxPdfContent_Path* pPath, TPtxPdfContent_Stroke* pStroke);

/******************************************************************************
 * PolyLineAnnotation
 *****************************************************************************/
/**
 * @brief Create a poly-line annotation.
The returned poly-line annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in,out] pPath The path of the poly line.

* @param[in,out] pStroke The stroking parameters used for stroking the poly-line.
The stroking paint is used as the annotation's main paint.


 * @return   The newly created poly-line annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b>'s conformance is not PDF 2.0 and the <b>pPath</b>
argument contains curve-to operations

 * - \ref ePtx_Error_IllegalArgument if the <b>pPath</b> argument contains close-subpath or rectangle operations

 * - \ref ePtx_Error_UnsupportedFeature if the <b>pTargetDocument</b>'s conformance is PDF 2.0 and the <b>pPath</b>
argument contains curve-to operations

 * - \ref ePtx_Error_IllegalArgument if the <b>pPath</b> argument has not been constructed with a \ref
TPtxPdfContent_PathGenerator ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument is not associated with the <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a \ref
PtxPdfContent_Paint_GetColorSpace "" other than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineCapStyle "" is
other than \ref ePtxPdfContent_LineCapStyle_Butt ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineJoinStyle "" is
other than \ref ePtxPdfContent_LineJoinStyle_Miter ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetMiterLimit "" is
other than 10

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetDashPhase "" is other
than 0

 * - \ref ePtx_Error_Conformance if the explicitly requested PDF conformance is smaller than PDF 1.5


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_PolyLineAnnotation* PDFTOOLS_TOOLBOX_CALL PtxPdfAnnots_PolyLineAnnotation_Create(
    TPtxPdf_Document* pTargetDocument, TPtxPdfContent_Path* pPath, TPtxPdfContent_Stroke* pStroke);

/**
 * @brief The starting point's style
* @param[in,out] pPolyLineAnnotation Acts as a handle to the native object of type \ref
TPtxPdfAnnots_PolyLineAnnotation.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_LineEnding PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_PolyLineAnnotation_GetStartStyle(TPtxPdfAnnots_PolyLineAnnotation* pPolyLineAnnotation);
/**
 * @brief The ending point's style
* @param[in,out] pPolyLineAnnotation Acts as a handle to the native object of type \ref
TPtxPdfAnnots_PolyLineAnnotation.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_LineEnding PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_PolyLineAnnotation_GetEndStyle(TPtxPdfAnnots_PolyLineAnnotation* pPolyLineAnnotation);
/**
 * @brief The line ending filling paint for both start and end
* @param[in,out] pPolyLineAnnotation Acts as a handle to the native object of type \ref
TPtxPdfAnnots_PolyLineAnnotation.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_PolyLineAnnotation_GetLineEndingFill(TPtxPdfAnnots_PolyLineAnnotation* pPolyLineAnnotation);

/******************************************************************************
 * PolygonAnnotation
 *****************************************************************************/
/**
 * @brief Create a polygon annotation.
The returned polygon annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in,out] pPath The path of the polygon.

* @param[in,out] pStroke The stroking parameters used for stroking the polygon.
The stroking paint is used as the annotation's main paint.
If `NULL` then the polygon is not filled.

* @param[in,out] pFill The paint used for filling the polygon.
If `NULL` then the polygon is not filled.


 * @return   The newly created polygon annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pPath</b> argument has not been constructed with a \ref
TPtxPdfContent_PathGenerator ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b>'s conformance is not PDF 2.0 and the <b>pPath</b>
argument contains curve-to operations

 * - \ref ePtx_Error_IllegalArgument if the <b>pPath</b> argument contains close-subpath or rectangle operations

 * - \ref ePtx_Error_UnsupportedFeature if the <b>pTargetDocument</b>'s conformance is PDF 2.0 and the <b>pPath</b>
argument contains curve-to operations

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> and <b>pFill</b> arguments are both `NULL`

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument is not associated with <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a \ref
PtxPdfContent_Paint_GetColorSpace "" other than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineCapStyle "" is
other than \ref ePtxPdfContent_LineCapStyle_Butt ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineJoinStyle "" is
other than \ref ePtxPdfContent_LineJoinStyle_Miter ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetMiterLimit "" is
other than 10

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetDashPhase "" is other
than 0

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> has a \ref PtxPdfContent_Paint_GetColorSpace "" other than a
device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> has a non-`NULL`\ref PtxPdfContent_Paint_GetTransparency ""
with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> argument is not associated with the <b>pTargetDocument</b>

 * - \ref ePtx_Error_Conformance if the explicitly requested PDF conformance is smaller than PDF 1.5


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_PolygonAnnotation* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_PolygonAnnotation_Create(TPtxPdf_Document* pTargetDocument, TPtxPdfContent_Path* pPath,
                                      TPtxPdfContent_Stroke* pStroke, TPtxPdfContent_Paint* pFill);

/**
 * @brief The filling paint
* @param[in,out] pPolygonAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_PolygonAnnotation.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_PolygonAnnotation_GetFill(TPtxPdfAnnots_PolygonAnnotation* pPolygonAnnotation);

/******************************************************************************
 * RectangleAnnotation
 *****************************************************************************/
/**
 * @brief Create a rectangle annotation.
The returned rectangle annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in] pBoundingBox The location on the page.

* @param[in,out] pStroke The stroking parameters used for stroking the rectangle.
The stroking paint is used as the annotation's main paint.
If `NULL` then the rectangle is not stroked.

* @param[in,out] pFill The paint used for filling the rectangle.
If `NULL` then the rectangle is not filled.


 * @return   The newly created rectangle annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is `NULL`

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> and <b>pFill</b> arguments are both `NULL`

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument is not associated with the <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a \ref
PtxPdfContent_Paint_GetColorSpace "" other than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineCapStyle "" is
other than \ref ePtxPdfContent_LineCapStyle_Butt ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineJoinStyle "" is
other than \ref ePtxPdfContent_LineJoinStyle_Miter ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetMiterLimit "" is
other than 10

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetDashPhase "" is other
than 0

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> has a \ref PtxPdfContent_Paint_GetColorSpace "" other than a
device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> has a non-`NULL`\ref PtxPdfContent_Paint_GetTransparency ""
with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> argument is not associated with the <b>pTargetDocument</b>


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_RectangleAnnotation* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_RectangleAnnotation_Create(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Rectangle* pBoundingBox,
                                        TPtxPdfContent_Stroke* pStroke, TPtxPdfContent_Paint* pFill);

/**
 * @brief The filling paint
* @param[in,out] pRectangleAnnotation Acts as a handle to the native object of type \ref
TPtxPdfAnnots_RectangleAnnotation.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_RectangleAnnotation_GetFill(TPtxPdfAnnots_RectangleAnnotation* pRectangleAnnotation);

/******************************************************************************
 * EllipseAnnotation
 *****************************************************************************/
/**
 * @brief Create an ellipse annotation.
The returned ellipse annotation is not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The output document with which the returned object is associated.

* @param[in] pBoundingBox The location on the page.

* @param[in,out] pStroke The stroking parameters used for stroking the ellipse.
The stroking paint is used as the annotation's main paint.
If `NULL` then the ellipse is not stroked.

* @param[in,out] pFill The paint used for filling the ellipse.
If `NULL` then the ellipse is not filled.


 * @return   The newly created ellipse annotation.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> and <b>pFill</b> arguments are both `NULL`

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument is not associated with <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a \ref
PtxPdfContent_Paint_GetColorSpace "" other than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b>'s \ref PtxPdfContent_Stroke_GetPaint "" has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineCapStyle "" is
other than \ref ePtxPdfContent_LineCapStyle_Butt ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetLineJoinStyle "" is
other than \ref ePtxPdfContent_LineJoinStyle_Miter ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetMiterLimit "" is
other than 10

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument's \ref PtxPdfContent_Stroke_GetDashPhase "" is other
than 0

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> has a \ref PtxPdfContent_Paint_GetColorSpace "" other than a
device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> has a non-`NULL`\ref PtxPdfContent_Paint_GetTransparency ""
with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref ePtxPdfContent_BlendMode_Normal ""

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> argument is not associated with <b>pTargetDocument</b>


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_EllipseAnnotation* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_EllipseAnnotation_Create(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Rectangle* pBoundingBox,
                                      TPtxPdfContent_Stroke* pStroke, TPtxPdfContent_Paint* pFill);

/**
 * @brief The filling paint
* @param[in,out] pEllipseAnnotation Acts as a handle to the native object of type \ref TPtxPdfAnnots_EllipseAnnotation.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_EllipseAnnotation_GetFill(TPtxPdfAnnots_EllipseAnnotation* pEllipseAnnotation);

/******************************************************************************
 * TextMarkup
 *****************************************************************************/
/**
 * @brief The paint the annotation and the popup
* @param[in,out] pTextMarkup Acts as a handle to the native object of type \ref TPtxPdfAnnots_TextMarkup.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt and the annotation's paint cannot be read


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_TextMarkup_GetPaint(TPtxPdfAnnots_TextMarkup* pTextMarkup);
/**
 * @brief The pop-up
* @param[in,out] pTextMarkup Acts as a handle to the native object of type \ref TPtxPdfAnnots_TextMarkup.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Popup* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_TextMarkup_GetPopup(TPtxPdfAnnots_TextMarkup* pTextMarkup);
/**
 * @brief The markup area
This list of \ref TPtxGeomReal_Quadrilateral ""s defines the markup area on the page.

* @param[in,out] pTextMarkup Acts as a handle to the native object of type \ref TPtxPdfAnnots_TextMarkup.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxGeomReal_QuadrilateralList* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_TextMarkup_GetMarkupArea(TPtxPdfAnnots_TextMarkup* pTextMarkup);

/**
 * @brief Get actual derived type of base type \ref TPtxPdfAnnots_TextMarkup.
 *
 * This function is invoked prior to downcasting to ascertain the derived object type.
 *
 * @param[in,out] pTextMarkup Acts as a handle to a native object.
 *
 * @return The item of the enumeration \ref TPtxPdfAnnots_TextMarkupType that refers to the actual derived type. `0` in
 * case of an error.
 *
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 *       Get the error message with \ref Ptx_GetLastErrorMessage.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_TextMarkupType PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_TextMarkup_GetType(TPtxPdfAnnots_TextMarkup* pTextMarkup);
/******************************************************************************
 * Highlight
 *****************************************************************************/
/**
 * @brief Create a new highlight with defined area
The highlight area is defined by the given <b>pMarkupArea</b>.
The returned object is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in,out] pMarkupArea The area on the page to be highlighted.

* @param[in,out] pPaint The highlight paint


 * @return   The newly created object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pMarkupArea</b> is empty

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument is not associated with <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument has a \ref PtxPdfContent_Paint_GetColorSpace "" other
than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Highlight* PDFTOOLS_TOOLBOX_CALL PtxPdfAnnots_Highlight_CreateFromQuadrilaterals(
    TPtxPdf_Document* pTargetDocument, TPtxGeomReal_QuadrilateralList* pMarkupArea, TPtxPdfContent_Paint* pPaint);

/******************************************************************************
 * Underline
 *****************************************************************************/
/**
 * @brief Create a new underline with defined area
The area to be underlined is defined by the given <b>pMarkupArea</b>.
The returned object is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in,out] pMarkupArea The area on the page to be underlined.

* @param[in,out] pPaint The paint used for drawing the underline


 * @return   The newly created object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pMarkupArea</b> is empty

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument is not associated with <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument has a \ref PtxPdfContent_Paint_GetColorSpace "" other
than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Underline* PDFTOOLS_TOOLBOX_CALL PtxPdfAnnots_Underline_CreateFromQuadrilaterals(
    TPtxPdf_Document* pTargetDocument, TPtxGeomReal_QuadrilateralList* pMarkupArea, TPtxPdfContent_Paint* pPaint);

/******************************************************************************
 * StrikeThrough
 *****************************************************************************/
/**
 * @brief Create a new strike-through with defined area
The area to be striked through is defined by the given <b>pMarkupArea</b>.
The returned object is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in,out] pMarkupArea The area on the page to be striked through.

* @param[in,out] pPaint The paint used for drawing the strike through


 * @return   The newly created object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pMarkupArea</b> is empty

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument is not associated with <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument has a \ref PtxPdfContent_Paint_GetColorSpace "" other
than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_StrikeThrough* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_StrikeThrough_CreateFromQuadrilaterals(TPtxPdf_Document*               pTargetDocument,
                                                    TPtxGeomReal_QuadrilateralList* pMarkupArea,
                                                    TPtxPdfContent_Paint*           pPaint);

/******************************************************************************
 * Squiggly
 *****************************************************************************/
/**
 * @brief Create a new squiggly underline with defined area
The area to be underlined is defined by the given <b>pMarkupArea</b>.
The returned object is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of annotations.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in,out] pMarkupArea The area on the page to be underlined.

* @param[in,out] pPaint The paint used for drawing the squiggly underline


 * @return   The newly created object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pMarkupArea</b> is empty

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument is not associated with <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument has a \ref PtxPdfContent_Paint_GetColorSpace "" other
than a device color space

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> argument has a non-`NULL`\ref
PtxPdfContent_Paint_GetTransparency "" with \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal ""


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Squiggly* PDFTOOLS_TOOLBOX_CALL PtxPdfAnnots_Squiggly_CreateFromQuadrilaterals(
    TPtxPdf_Document* pTargetDocument, TPtxGeomReal_QuadrilateralList* pMarkupArea, TPtxPdfContent_Paint* pPaint);

/******************************************************************************
 * TextInsert
 *****************************************************************************/
/**
 * @brief The paint for the annotation and the popup
* @param[in,out] pTextInsert Acts as a handle to the native object of type \ref TPtxPdfAnnots_TextInsert.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt and the annotation's paint cannot be read


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_TextInsert_GetPaint(TPtxPdfAnnots_TextInsert* pTextInsert);
/**
 * @brief The pop-up
* @param[in,out] pTextInsert Acts as a handle to the native object of type \ref TPtxPdfAnnots_TextInsert.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfAnnots_Popup* PDFTOOLS_TOOLBOX_CALL
PtxPdfAnnots_TextInsert_GetPopup(TPtxPdfAnnots_TextInsert* pTextInsert);

#ifdef __cplusplus
}
#endif

#endif /* PDFTOOLS_TOOLBOX_PTXPDFANNOTS_H__ */
