/******************************************************************************
 *
 * File:            PdfTools_Toolbox_PtxPdfContent.h
 *
 * Description:     Sub Header file for Toolbox SDK
 *
 * Author:          PDF Tools AG
 *
 * Copyright:       Copyright (C) 2024 - 2024 PDF Tools AG, Switzerland
 *                  All rights reserved.
 *
 * Notice:          By downloading and using this artifact, you accept PDF Tools AG's
 *                  [license agreement](https://www.pdf-tools.com/license-agreement/),
 *                  [privacy policy](https://www.pdf-tools.com/privacy-policy/),
 *                  and allow PDF Tools AG to track your usage data.
 *
 *****************************************************************************/

#ifndef PDFTOOLS_TOOLBOX_PTXPDFCONTENT_H__
#define PDFTOOLS_TOOLBOX_PTXPDFCONTENT_H__

#ifndef PDFTOOLS_TOOLBOX_CALL
#if defined(WIN32)
#define PDFTOOLS_TOOLBOX_CALL __stdcall
#else
#define PDFTOOLS_TOOLBOX_CALL
#endif
#endif

#include "PdfTools_Toolbox_Types.h"
#include "PdfTools_Toolbox_PtxSys.h"

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef _UNICODE
#define PtxPdfContent_Font_CreateFromSystem PtxPdfContent_Font_CreateFromSystemW

#define PtxPdfContent_Font_GetBaseFont PtxPdfContent_Font_GetBaseFontW

#define PtxPdfContent_TextGenerator_GetWidth PtxPdfContent_TextGenerator_GetWidthW
#define PtxPdfContent_TextGenerator_Show     PtxPdfContent_TextGenerator_ShowW
#define PtxPdfContent_TextGenerator_ShowLine PtxPdfContent_TextGenerator_ShowLineW

#define PtxPdfContent_Glyph_GetText PtxPdfContent_Glyph_GetTextW

#define PtxPdfContent_TextFragment_GetText PtxPdfContent_TextFragment_GetTextW

#else
#define PtxPdfContent_Font_CreateFromSystem PtxPdfContent_Font_CreateFromSystemA

#define PtxPdfContent_Font_GetBaseFont PtxPdfContent_Font_GetBaseFontA

#define PtxPdfContent_TextGenerator_GetWidth PtxPdfContent_TextGenerator_GetWidthA
#define PtxPdfContent_TextGenerator_Show     PtxPdfContent_TextGenerator_ShowA
#define PtxPdfContent_TextGenerator_ShowLine PtxPdfContent_TextGenerator_ShowLineA

#define PtxPdfContent_Glyph_GetText PtxPdfContent_Glyph_GetTextA

#define PtxPdfContent_TextFragment_GetText PtxPdfContent_TextFragment_GetTextA

#endif

/******************************************************************************
 * Transparency
 *****************************************************************************/
/**
* @param[in] dAlpha

 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the given value smaller than 0.0 or greater than 1.0.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Transparency* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Transparency_New(double dAlpha);

/**
* @param[in,out] pTransparency Acts as a handle to the native object of type \ref TPtxPdfContent_Transparency.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_BlendMode PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Transparency_GetBlendMode(TPtxPdfContent_Transparency* pTransparency);
/**
* @param[in,out] pTransparency Acts as a handle to the native object of type \ref TPtxPdfContent_Transparency.

* @param[in] iBlendMode Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Transparency_SetBlendMode(
    TPtxPdfContent_Transparency* pTransparency, TPtxPdfContent_BlendMode iBlendMode);
/**
* @param[in,out] pTransparency Acts as a handle to the native object of type \ref TPtxPdfContent_Transparency.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `-1` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the given value smaller than 0.0 or greater than 1.0.


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Transparency_GetAlpha(TPtxPdfContent_Transparency* pTransparency);
/**
* @param[in,out] pTransparency Acts as a handle to the native object of type \ref TPtxPdfContent_Transparency.

* @param[in] dAlpha Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the given value smaller than 0.0 or greater than 1.0.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Transparency_SetAlpha(TPtxPdfContent_Transparency* pTransparency, double dAlpha);

/******************************************************************************
 * Stroke
 *****************************************************************************/
/**
* @param[in,out] pPaint
* @param[in] dLineWidth

 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pPaint</b> argument has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Stroke* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Stroke_New(TPtxPdfContent_Paint* pPaint, double dLineWidth);

/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the document associated with the \ref TPtxPdfContent_Paint "" object has already
been closed

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_Paint "" object is associated with a different
document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Stroke_GetPaint(TPtxPdfContent_Stroke* pStroke);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.

* @param[in,out] pPaint Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the document associated with the \ref TPtxPdfContent_Paint "" object has already
been closed

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_Paint "" object is associated with a different
document


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Stroke_SetPaint(TPtxPdfContent_Stroke* pStroke,
                                                                                 TPtxPdfContent_Paint*  pPaint);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `-1.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Stroke_GetLineWidth(TPtxPdfContent_Stroke* pStroke);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.

* @param[in] dLineWidth Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Stroke_SetLineWidth(TPtxPdfContent_Stroke* pStroke,
                                                                                     double                 dLineWidth);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_LineCapStyle PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Stroke_GetLineCapStyle(TPtxPdfContent_Stroke* pStroke);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.

* @param[in] iLineCapStyle Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Stroke_SetLineCapStyle(TPtxPdfContent_Stroke* pStroke, TPtxPdfContent_LineCapStyle iLineCapStyle);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_LineJoinStyle PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Stroke_GetLineJoinStyle(TPtxPdfContent_Stroke* pStroke);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.

* @param[in] iLineJoinStyle Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Stroke_SetLineJoinStyle(TPtxPdfContent_Stroke* pStroke, TPtxPdfContent_LineJoinStyle iLineJoinStyle);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved array `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `-1` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Stroke_GetDashArray(TPtxPdfContent_Stroke* pStroke,
                                                                                       double*                pBuffer,
                                                                                       size_t nBufferSize);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.

* @param[in] pDashArray Set value.

* @param[in] nDashArrays Size of the array `pDashArray`.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Stroke_SetDashArray(TPtxPdfContent_Stroke* pStroke,
                                                                                     const double*          pDashArray,
                                                                                     size_t nDashArrays);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `-1.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Stroke_GetDashPhase(TPtxPdfContent_Stroke* pStroke);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.

* @param[in] dDashPhase Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Stroke_SetDashPhase(TPtxPdfContent_Stroke* pStroke,
                                                                                     double                 dDashPhase);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `-1.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Stroke_GetMiterLimit(TPtxPdfContent_Stroke* pStroke);
/**
* @param[in,out] pStroke Acts as a handle to the native object of type \ref TPtxPdfContent_Stroke.

* @param[in] dMiterLimit Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Stroke "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Stroke_SetMiterLimit(TPtxPdfContent_Stroke* pStroke,
                                                                                      double dMiterLimit);

/******************************************************************************
 * Fill
 *****************************************************************************/
/**
* @param[in,out] pPaint

 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pPaint</b> argument has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Fill* PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Fill_New(TPtxPdfContent_Paint* pPaint);

/**
* @param[in,out] pFill Acts as a handle to the native object of type \ref TPtxPdfContent_Fill.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Fill "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the document associated with the given \ref TPtxPdfContent_Paint "" object has
already been closed

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_Paint "" object is associated with a different
document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Fill_GetPaint(TPtxPdfContent_Fill* pFill);
/**
* @param[in,out] pFill Acts as a handle to the native object of type \ref TPtxPdfContent_Fill.

* @param[in,out] pPaint Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Fill "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the document associated with the given \ref TPtxPdfContent_Paint "" object has
already been closed

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_Paint "" object is associated with a different
document


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Fill_SetPaint(TPtxPdfContent_Fill*  pFill,
                                                                               TPtxPdfContent_Paint* pPaint);
/**
* @param[in,out] pFill Acts as a handle to the native object of type \ref TPtxPdfContent_Fill.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Fill "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_InsideRule PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Fill_GetInsideRule(TPtxPdfContent_Fill* pFill);
/**
* @param[in,out] pFill Acts as a handle to the native object of type \ref TPtxPdfContent_Fill.

* @param[in] iInsideRule Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the \ref TPtxPdfContent_Fill "" object has already
been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Fill_SetInsideRule(TPtxPdfContent_Fill* pFill, TPtxPdfContent_InsideRule iInsideRule);

/******************************************************************************
 * Image
 *****************************************************************************/
/**
 * @brief Create an image object from image data.
Supported formats are:
  - BMP
  - DIB
  - JPEG
  - JPEG2000
  - JBIG2
  - PNG
  - GIF
The returned image object is not yet painted on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] pStreamDesc the image data stream


 * @return   the newly created image object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the image or writing to the document

 * - \ref ePtx_Error_UnknownFormat The image data has an unknown format

 * - \ref ePtx_Error_Corrupt The image data is corrupt

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pStreamDesc</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Image* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Image_Create(TPtxPdf_Document* pTargetDocument, const TPtxSys_StreamDescriptor* pStreamDesc);
/**
 * @brief Extract embedded image from PDF

Facilitate the extraction of images from a specified page within a PDF, outputting them in the <b>pImageType</b> format.

By default, the method determines the format of the extracted image based on the embedded image data present within the
PDF. Users can ascertain the default image format through \ref PtxPdfContent_Image_GetDefaultImageType "". It's
important to note that not all image types or conversion processes are universally supported, hence adhering to the
default \ref TPtxPdfContent_ImageType "" is advisable for optimal compatibility.

Key considerations include:
  - The extraction process isolates the image from the page's resources, neglecting any contextual attributes from the
PDF page. Consequently, the original resolution and modifications—such as scaling, rotation, or cropping—that influence
the image's appearance on the page are not preserved in the extracted image.
  - In instances where a \ref ePtx_Error_Generic "" error arises, the output file may be compromised and rendered
unusable.

This method is designed to efficiently retrieve images without their page-specific modifications, ensuring a
straightforward extraction process.

* @param[in,out] pImage Acts as a handle to the native object of type \ref TPtxPdfContent_Image.

* @param[in,out] pStreamDesc The image data stream.

* @param[in] pImageType The desired image type of the extracted image stream.
If the embedded image data cannot be directly extracted to the chosen ImageType, the data is first recompressed and then
extracted to the chosen ImageType. In this case, extraction is slower and there can be some loss of image quality. The
default image type can be retrieved by calling \ref PtxPdfContent_Image_GetDefaultImageType "".


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pStreamDesc</b> argument is null

 * - \ref ePtx_Error_IllegalState if the image has already been closed

 * - \ref ePtx_Error_Generic if image extraction fails


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Image_Extract(TPtxPdfContent_Image* pImage, const TPtxSys_StreamDescriptor* pStreamDesc,
                            const TPtxPdfContent_ImageType* pImageType);

/**
 * @brief Default extracted image type.
The default image type that will be extracted, based on the way that the image data is compressed and stored in the PDF
file. The type of the output image is \ref ePtxPdfContent_ImageType_Jpeg "" for embedded JPEG and JPEG2000 images. In
all other cases the image type will be \ref ePtxPdfContent_ImageType_Tiff "".

* @param[in,out] pImage Acts as a handle to the native object of type \ref TPtxPdfContent_Image.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the image has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ImageType PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Image_GetDefaultImageType(TPtxPdfContent_Image* pImage);
/**
 * @brief The size of the image in samples.
Samples are often also called pixels.

* @param[in,out] pImage Acts as a handle to the native object of type \ref TPtxPdfContent_Image.

* @param[out] pSize Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the image has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Image_GetSize(TPtxPdfContent_Image* pImage,
                                                                               TPtxGeomInt_Size*     pSize);
/**
 * @brief The raw content of the image.

The samples (pixels) are given in order, top to bottom,
left to right. Each sample is given component by component.
There is no padding between components or samples, except
that each row of sample data  begins on a byte boundary.
If the number of data bits per row is not a multiple of 8,
the end of the row is padded with extra bits to fill out the
last byte. Padding bits should be ignored.

Most often, each component is 8 bits, so there's no packing/unpacking
or alignment/padding. Components with 2 or 4 bits are very rare.

If the image is compressed, it will be decompressed in order
to get the samples. For very large images, this may take some
time.

* @param[in,out] pImage Acts as a handle to the native object of type \ref TPtxPdfContent_Image.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved array `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `-1` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the image has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Image_GetSamples(TPtxPdfContent_Image* pImage,
                                                                                    unsigned char*        pBuffer,
                                                                                    size_t                nBufferSize);
/**
 * @brief the number of bits per component.
The number of bits used to represent each color component.
Only a single value may be specified; the number of bits is the same for all color components.
Valid values are 1, 2, 4, and 8.

* @param[in,out] pImage Acts as a handle to the native object of type \ref TPtxPdfContent_Image.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the image has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Image_GetBitsPerComponent(TPtxPdfContent_Image* pImage);
/**
 * @brief the color space in which image samples are specified.
* @param[in,out] pImage Acts as a handle to the native object of type \ref TPtxPdfContent_Image.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the image has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ColorSpace* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Image_GetColorSpace(TPtxPdfContent_Image* pImage);

/******************************************************************************
 * ImageMask
 *****************************************************************************/
/**
 * @brief Create an image mask object from image data.
Supported formats are:
  - BMP
  - DIB
  - JBIG2
  - PNG
  - GIF
The returned image mask object is not yet painted on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] pStreamDesc the image data stream


 * @return   the newly created image mask object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the image or writing to the document

 * - \ref ePtx_Error_UnknownFormat The image data has an unknown format or the format is not suitable for an image mask

 * - \ref ePtx_Error_Corrupt The image data is corrupt

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pStreamDesc</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ImageMask* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ImageMask_Create(TPtxPdf_Document* pTargetDocument, const TPtxSys_StreamDescriptor* pStreamDesc);
/**
 * @brief Extract image mask from PDF

Facilitate the extraction of image masks from a specified page within a PDF, outputting them in the <b>pImageType</b>
format.

By default <b>pImageType</b> takes the value \ref ePtxPdfContent_ImageType_Tiff "".

Key considerations include:
  - The extraction process isolates the image from the page's resources, neglecting any contextual attributes from the
PDF page. Consequently, the original resolution and modifications—such as scaling, rotation, or cropping—that influence
the image's appearance on the page are not preserved in the extracted image mask.
  - In instances where a \ref ePtx_Error_Generic "" error arises, the output file may be compromised and rendered
unusable.

This method is designed to efficiently retrieve image masks without their page-specific modifications, ensuring a
straightforward extraction process.

* @param[in,out] pImageMask Acts as a handle to the native object of type \ref TPtxPdfContent_ImageMask.

* @param[in,out] pStreamDesc The image mask data stream.

* @param[in] pImageType

 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pStreamDesc</b> argument is null

 * - \ref ePtx_Error_IllegalState if the image has already been closed

 * - \ref ePtx_Error_Generic if image extraction fails


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ImageMask_Extract(TPtxPdfContent_ImageMask* pImageMask, const TPtxSys_StreamDescriptor* pStreamDesc,
                                const TPtxPdfContent_ImageType* pImageType);

/**
 * @brief The size of the image mask in samples.
* @param[in,out] pImageMask Acts as a handle to the native object of type \ref TPtxPdfContent_ImageMask.

* @param[out] pSize Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the image has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ImageMask_GetSize(TPtxPdfContent_ImageMask* pImageMask,
                                                                                   TPtxGeomInt_Size*         pSize);

/******************************************************************************
 * Font
 *****************************************************************************/
/**
 * @brief Create a new font object from font file data.
Supported formats are:
  - Type1
  - CFF
  - TrueType
  - OpenType
The returned font object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] pStreamDesc the font file data stream

* @param[in] bEmbedded \ref "TRUE" if the font shall be embedded in the document.
Note that this parameter must be \ref "TRUE" for PDF/A documents.


 * @return   the newly created font object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the font file or writing to the document

 * - \ref ePtx_Error_UnknownFormat The font data has an unknown format

 * - \ref ePtx_Error_Corrupt The font data is corrupt

 * - \ref ePtx_Error_Conformance Parameter <b>bEmbedded</b> is \ref "FALSE" for PDF/A document

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pStreamDesc</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Font* PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_Create(
    TPtxPdf_Document* pTargetDocument, const TPtxSys_StreamDescriptor* pStreamDesc, BOOL bEmbedded);
/**
 * @brief Create a new font object from an installed font.
The returned font object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] szFamily the font family name (e.g. "Arial")

* @param[in] szStyle the font style (e.g. "Bold")

* @param[in] bEmbedded \ref "TRUE" if the font shall be embedded in the document.
Note that this parameter must be \ref "TRUE" for PDF/A documents.


 * @return   the newly created font object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_NotFound There is no such font installed

 * - \ref ePtx_Error_IO Error reading the font file or writing to the document

 * - \ref ePtx_Error_UnknownFormat The font data has an unknown format

 * - \ref ePtx_Error_Corrupt The font data is corrupt

 * - \ref ePtx_Error_Conformance Parameter <b>bEmbedded</b> is \ref "FALSE" for PDF/A document

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>szFamily</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Font* PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_CreateFromSystemA(
    TPtxPdf_Document* pTargetDocument, const char* szFamily, const char* szStyle, BOOL bEmbedded);
/**
 * @brief Create a new font object from an installed font.
The returned font object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] szFamily the font family name (e.g. "Arial")

* @param[in] szStyle the font style (e.g. "Bold")

* @param[in] bEmbedded \ref "TRUE" if the font shall be embedded in the document.
Note that this parameter must be \ref "TRUE" for PDF/A documents.


 * @return   the newly created font object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_NotFound There is no such font installed

 * - \ref ePtx_Error_IO Error reading the font file or writing to the document

 * - \ref ePtx_Error_UnknownFormat The font data has an unknown format

 * - \ref ePtx_Error_Corrupt The font data is corrupt

 * - \ref ePtx_Error_Conformance Parameter <b>bEmbedded</b> is \ref "FALSE" for PDF/A document

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>szFamily</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Font* PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_CreateFromSystemW(
    TPtxPdf_Document* pTargetDocument, const WCHAR* szFamily, const WCHAR* szStyle, BOOL bEmbedded);
/**
 * @brief the width of a single glyph.

The width of a unicode character (in pt) relative to a font size of 1 pt.

If an error occurs (because the font or the owning document has already been closed) this method returns 0.
But a return value of 0 is not generally an indication for failure.

* @param[in,out] pFont Acts as a handle to the native object of type \ref TPtxPdfContent_Font.

* @param[in] iCharacter the unicode character code.


 * @return   the width of the character

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_GetCharacterWidth(TPtxPdfContent_Font* pFont,
                                                                                          int iCharacter);

/**
 * @brief the PostScript name of the font
* @param[in,out] pFont Acts as a handle to the native object of type \ref TPtxPdfContent_Font.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the font has already been closed

 * - \ref ePtx_Error_Corrupt if the font's base name is missing or cannot be decoded


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_GetBaseFontA(TPtxPdfContent_Font* pFont,
                                                                                     char* pBuffer, size_t nBufferSize);
/**
 * @brief the PostScript name of the font
* @param[in,out] pFont Acts as a handle to the native object of type \ref TPtxPdfContent_Font.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the font has already been closed

 * - \ref ePtx_Error_Corrupt if the font's base name is missing or cannot be decoded


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_GetBaseFontW(TPtxPdfContent_Font* pFont,
                                                                                     WCHAR*               pBuffer,
                                                                                     size_t               nBufferSize);
/**
 * @brief the visual weight of the font.
Indicates the visual weight (degree of blackness or thickness of strokes) of the characters in the font.
If the font does not define this value, `NULL` is returned.

* @param[in,out] pFont Acts as a handle to the native object of type \ref TPtxPdfContent_Font.

* @param[out] pWeight Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the font has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_GetWeight(TPtxPdfContent_Font*       pFont,
                                                                                TPtxPdfContent_FontWeight* pWeight);
/**
 * @brief the italic angle of the font.
The angle is expressed in degrees counterclockwise from the vertical,
of the dominant vertical strokes of the font.
(For example, the 9-o’clock position is 90 degrees, and the 3-o’clock position is –90 degrees.)
The value is negative for fonts that slope to the right,
as almost all italic fonts do.

* @param[in,out] pFont Acts as a handle to the native object of type \ref TPtxPdfContent_Font.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the font has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_GetItalicAngle(TPtxPdfContent_Font* pFont);
/**
 * @brief the ascent of the font.
The maximum height above the baseline reached by glyphs in this font,
excluding the height of glyphs for accented characters.

* @param[in,out] pFont Acts as a handle to the native object of type \ref TPtxPdfContent_Font.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the font has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_GetAscent(TPtxPdfContent_Font* pFont);
/**
 * @brief the descent of the font.
The maximum depth below the baseline reached by glyphs in this font.
The value is a negative number.

* @param[in,out] pFont Acts as a handle to the native object of type \ref TPtxPdfContent_Font.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the font has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_GetDescent(TPtxPdfContent_Font* pFont);
/**
 * @brief the cap height of the font.
The vertical coordinate of the top of flat capital letters,
measured from the baseline.

* @param[in,out] pFont Acts as a handle to the native object of type \ref TPtxPdfContent_Font.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the font has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_GetCapHeight(TPtxPdfContent_Font* pFont);
/**
 * @brief the leading of the font.

The vertical distance between two adjacent base lines in multiline text.

This is a recomended value when generating several lines of text.
If the font does not define this value, `NULL` is returned.

Note that \ref PtxPdfContent_TextGenerator_New "" uses a fixed value of 1.2
instead of this property to initialize the \ref PtxPdfContent_TextGenerator_SetLeading ""
in order to maintain backward compatibility with earlier versions.

* @param[in,out] pFont Acts as a handle to the native object of type \ref TPtxPdfContent_Font.

* @param[out] pLeading Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the font has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Font_GetLeading(TPtxPdfContent_Font* pFont,
                                                                                 double*              pLeading);

/******************************************************************************
 * Group
 *****************************************************************************/
/**
 * @brief Create an empty group object.
The returned group object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] pSize the size of the group


 * @return   the newly created group object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Group* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Group_Create(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Size* pSize);
/**
 * @brief Create a group object from a page.

From a given page in an input document, create a group object in the given target document.
The returned object is associated with the target document but not yet part of it.

A group that contains interactive elements can be painted once only.
Interactive elements are annotations, group fields, outlines or logical structure information.
If a group needs to be painted multiple times,
interactive elements can be flattened or the group can be copied multiple times from the page.

There are some interactive elements such as form fields or text annotations that cannot be rotated.
So if you plan to rotate the group, it is recommended to flatten form fields and annotations.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pPage a page of a different document

* @param[in,out] pOptions the copy options


 * @return   the copied group, associated with the current document.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the input document or writing to the output document

 * - \ref ePtx_Error_Corrupt The input document is corrupt

 * - \ref ePtx_Error_Conformance The conformance level of the source document is not compatible
with the conformance level of the target document.

 * - \ref ePtx_Error_Conformance The explicitly requested conformance level is PDF/A Level A
(\ref ePtxPdf_Conformance_PdfA1A "", \ref ePtxPdf_Conformance_PdfA2A "",
or \ref ePtxPdf_Conformance_PdfA3A "")
and the copy option \ref PtxPdf_PageCopyOptions_GetCopyLogicalStructure "" is not set.

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pPage</b> object is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pPage</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the argument <b>pOptions</b> has \ref PtxPdf_PageCopyOptions_GetFormFields ""
set to \ref ePtxPdfForms_FormFieldCopyStrategy_Copy "" and the <b>pTargetDocument</b> contains form fields that have
either been copied explicitly with \ref PtxPdfForms_FieldNode_Copy "" or created with \ref PtxPdfForms_CheckBox_Create
"", \ref PtxPdfForms_ComboBox_Create "", \ref PtxPdfForms_CombTextField_Create "", \ref
PtxPdfForms_GeneralTextField_Create "", \ref PtxPdfForms_ListBox_Create "", \ref PtxPdfForms_RadioButtonGroup_Create "",
or \ref PtxPdfForms_SubForm_Create "".

 * - \ref ePtx_Error_IllegalArgument if the argument <b>pOptions</b> has \ref
PtxPdf_PageCopyOptions_GetUnsignedSignatures "" set to \ref ePtxPdf_CopyStrategy_Copy "" and the <b>pTargetDocument</b>
contains form fields that have either been copied explicitly with \ref PtxPdfForms_FieldNode_Copy "" or created with
\ref PtxPdfForms_CheckBox_Create "", \ref PtxPdfForms_ComboBox_Create "", \ref PtxPdfForms_CombTextField_Create "", \ref
PtxPdfForms_GeneralTextField_Create "", \ref PtxPdfForms_ListBox_Create "", \ref PtxPdfForms_RadioButtonGroup_Create "",
or \ref PtxPdfForms_SubForm_Create "".

 * - \ref ePtx_Error_IllegalArgument if <b>pOptions</b> has \ref PtxPdf_PageCopyOptions_GetCopyOutlineItems "" set to
\ref "TRUE" and the <b>pTargetDocument</b> contains outline items that have been copied explicitly with \ref
PtxPdfNav_OutlineItem_Copy "".


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Group* PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Group_CopyFromPage(
    TPtxPdf_Document* pTargetDocument, TPtxPdf_Page* pPage, TPtxPdf_PageCopyOptions* pOptions);

/**
 * @brief the size of the group.
* @param[in,out] pGroup Acts as a handle to the native object of type \ref TPtxPdfContent_Group.

* @param[out] pSize Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the group has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Group_GetSize(TPtxPdfContent_Group* pGroup,
                                                                               TPtxGeomReal_Size*    pSize);
/**
 * @brief the group content.
If the group is writable, the content object can be used to apply new content on the group.

* @param[in,out] pGroup Acts as a handle to the native object of type \ref TPtxPdfContent_Group.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the group has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Content* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Group_GetContent(TPtxPdfContent_Group* pGroup);
/**
 * @brief the transparency isolation behavior
* @param[in,out] pGroup Acts as a handle to the native object of type \ref TPtxPdfContent_Group.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the group has already been closed

 * - \ref ePtx_Error_IllegalState if the group has already been painted

 * - \ref ePtx_Error_UnsupportedOperation The document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Group_GetIsolated(TPtxPdfContent_Group* pGroup);
/**
 * @brief the transparency isolation behavior
* @param[in,out] pGroup Acts as a handle to the native object of type \ref TPtxPdfContent_Group.

* @param[in] bIsolated Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the group has already been closed

 * - \ref ePtx_Error_IllegalState if the group has already been painted

 * - \ref ePtx_Error_UnsupportedOperation The document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Group_SetIsolated(TPtxPdfContent_Group* pGroup,
                                                                                   BOOL                  bIsolated);
/**
 * @brief the transparency knockout behavior
* @param[in,out] pGroup Acts as a handle to the native object of type \ref TPtxPdfContent_Group.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the group has already been closed

 * - \ref ePtx_Error_IllegalState if the group has already been painted

 * - \ref ePtx_Error_UnsupportedOperation The document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Group_GetKnockout(TPtxPdfContent_Group* pGroup);
/**
 * @brief the transparency knockout behavior
* @param[in,out] pGroup Acts as a handle to the native object of type \ref TPtxPdfContent_Group.

* @param[in] bKnockout Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the group has already been closed

 * - \ref ePtx_Error_IllegalState if the group has already been painted

 * - \ref ePtx_Error_UnsupportedOperation The document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Group_SetKnockout(TPtxPdfContent_Group* pGroup,
                                                                                   BOOL                  bKnockout);

/******************************************************************************
 * ColorSpace
 *****************************************************************************/
/**
 * @brief Get the canonical grayscale, RGB, or CMYK color space.
Depending on the PDF/A compliance and the output intent,
this is either a device color space (\ref TPtxPdfContent_DeviceGrayColorSpace "", \ref
TPtxPdfContent_DeviceRgbColorSpace "", \ref TPtxPdfContent_DeviceCmykColorSpace ""), a calibrated color space (\ref
TPtxPdfContent_CalibratedGrayColorSpace "", \ref TPtxPdfContent_CalibratedRgbColorSpace ""), or an ICC-based color space
(4-channel \ref TPtxPdfContent_IccBasedColorSpace ""). The returned color space object is not yet used on any page, but
it is associated with the given target document.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] iType the color space type


 * @return   newly created color space object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Unable to read a required ICC profile or writing to the document

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ColorSpace* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ColorSpace_CreateProcessColorSpace(TPtxPdf_Document*                    pTargetDocument,
                                                 TPtxPdfContent_ProcessColorSpaceType iType);
/**
 * @brief Copy a color space
Copy a color space object from an input document to the given <b>pTargetDocument</b>.
The returned object is associated with the given target document but not yet part of it.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pColorSpace a color space of a different document


 * @return   the copied color space, associated with the current document.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the source document or writing to the target document

 * - \ref ePtx_Error_Corrupt The source document is corrupt

 * - \ref ePtx_Error_Conformance The conformance level of the source document is not compatible
with the conformance level of the target document.

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pColorSpace</b> object is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument if the <b>pColorSpace</b> object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ColorSpace* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ColorSpace_Copy(TPtxPdf_Document* pTargetDocument, TPtxPdfContent_ColorSpace* pColorSpace);

/**
 * @brief the number of components in the color space.
* @param[in,out] pColorSpace Acts as a handle to the native object of type \ref TPtxPdfContent_ColorSpace.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the color space has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ColorSpace_GetComponentCount(TPtxPdfContent_ColorSpace* pColorSpace);

/**
 * @brief Get actual derived type of base type \ref TPtxPdfContent_ColorSpace.
 *
 * This function is invoked prior to downcasting to ascertain the derived object type.
 *
 * @param[in,out] pColorSpace Acts as a handle to a native object.
 *
 * @return The item of the enumeration \ref TPtxPdfContent_ColorSpaceType that refers to the actual derived type. `0` in
 * case of an error.
 *
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 *       Get the error message with \ref Ptx_GetLastErrorMessage.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ColorSpaceType PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ColorSpace_GetType(TPtxPdfContent_ColorSpace* pColorSpace);
/******************************************************************************
 * IccBasedColorSpace
 *****************************************************************************/
/**
 * @brief Create an new ICC-based color space from an ICC color profile.
The returned color space object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in] pProfile the color profile data stream


 * @return   newly created color profile object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the profile or writing to the document

 * - \ref ePtx_Error_UnknownFormat The profile data has an unknown format

 * - \ref ePtx_Error_Corrupt The profile data is corrupt

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pProfile</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_IccBasedColorSpace* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_IccBasedColorSpace_Create(TPtxPdf_Document* pTargetDocument, const TPtxSys_StreamDescriptor* pProfile);
/**
 * @brief Copy an ICC-based color space
Copy an ICC-based color space object from an input document to the given <b>pTargetDocument</b>.
The returned object is associated with the given target document but not yet part of it.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pColorSpace an ICC-based color space of a different document


 * @return   the copied color space, associated with the current document.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the source document or writing to the target document

 * - \ref ePtx_Error_Corrupt The source document is corrupt

 * - \ref ePtx_Error_Conformance The conformance level of the source document is not compatible
with the conformance level of the target document.

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pColorSpace</b> object is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument if the document associated with <b>pColorSpace</b> has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_IccBasedColorSpace* PDFTOOLS_TOOLBOX_CALL PtxPdfContent_IccBasedColorSpace_Copy(
    TPtxPdf_Document* pTargetDocument, TPtxPdfContent_IccBasedColorSpace* pColorSpace);

/******************************************************************************
 * Subpath
 *****************************************************************************/
/**
 * @brief Get the number of elements in the list.
* @param[in,out] pSubpath Acts as a handle to the native object of type \ref TPtxPdfContent_Subpath.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Subpath_GetCount(TPtxPdfContent_Subpath* pSubpath);
/**
 * @brief Returns the element at the specified position in the given list.
* @param[in,out] pSubpath Acts as a handle to the native object of type \ref TPtxPdfContent_Subpath.

* @param[in] iIndex
* @param[out] pPathSegment

 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Subpath_Get(TPtxPdfContent_Subpath*     pSubpath,
                                                                             int                         iIndex,
                                                                             TPtxPdfContent_PathSegment* pPathSegment);

/**
The start point of the \ref TPtxPdfContent_Subpath "".

* @param[in,out] pSubpath Acts as a handle to the native object of type \ref TPtxPdfContent_Subpath.

* @param[out] pStartPoint Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Subpath_GetStartPoint(TPtxPdfContent_Subpath* pSubpath,
                                                                                       TPtxGeomReal_Point* pStartPoint);
/**
If \ref "TRUE", the \ref TPtxPdfContent_Subpath "" represents a closed curve.

* @param[in,out] pSubpath Acts as a handle to the native object of type \ref TPtxPdfContent_Subpath.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Subpath_IsClosed(TPtxPdfContent_Subpath* pSubpath);

/******************************************************************************
 * Path
 *****************************************************************************/
/**
 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Path* PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Path_New(void);

/**
 * @brief Get an iterator for this enumerable.
* @param[in,out] pPath Acts as a handle to the native object of type \ref TPtxPdfContent_Path.


 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_PathIterator* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Path_GetIterator(TPtxPdfContent_Path* pPath);

/******************************************************************************
 * PathIterator
 *****************************************************************************/
/**
 * @brief Get the current element or `NULL`if no elements are left.
* @param[in,out] pPathIterator Acts as a handle to the native object of type \ref TPtxPdfContent_PathIterator.


 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Subpath* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathIterator_GetValue(TPtxPdfContent_PathIterator* pPathIterator);
/**
 * @brief Move the iterator to the next element.
* @param[in,out] pPathIterator Acts as a handle to the native object of type \ref TPtxPdfContent_PathIterator.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathIterator_MoveNext(TPtxPdfContent_PathIterator* pPathIterator);

/******************************************************************************
 * ContentGenerator
 *****************************************************************************/
/**
 * @brief Create a new content generator for appending or prepending to the content of a group.
* @param[in,out] pContent the content object of a page or group

* @param[in] bPrepend \ref "TRUE" for prepending to the content (apply content to background of page), \ref "FALSE" for
appending (apply content to foreground of page)


 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the document associated with <b>pContent</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the page or group associated with the <b>pContent</b> has already been appended
or closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ContentGenerator* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentGenerator_New(TPtxPdfContent_Content* pContent, BOOL bPrepend);

/**
 * @brief Save the current graphics state
The graphics state is stored on the graphics state stack.
The following properties are affected:
  - The current transform matrix
  - The current clip path

* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the page/group associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentGenerator_Save(TPtxPdfContent_ContentGenerator* pContentGenerator);
/**
 * @brief Restore the graphics state.
The most recently saved state is restored and removed from the graphics state stack.
The following properties are affected:
  - The current transform matrix
  - The current clip path

* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the page/group associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentGenerator_Restore(TPtxPdfContent_ContentGenerator* pContentGenerator);
/**
 * @brief Modify the current transform matrix by concatenating the specified matrix.
* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.

* @param[in] pTransform the transform that is applied to the current transform


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the page/group associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTransform</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTransform</b> is non-invertible


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ContentGenerator_Transform(
    TPtxPdfContent_ContentGenerator* pContentGenerator, const TPtxGeomReal_AffineTransform* pTransform);
/**
 * @brief Paint an image.
* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.

* @param[in,out] pImage the image to be painted

* @param[in] pTargetRect the target rectangle in the current coordinate system. If targetRect is `NULL`, the unit
rectangle <b>[0, 0, 1, 1]</b> is used.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the page/group associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the document associated with <b>pImage</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pImage</b> is associated with a different document


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentGenerator_PaintImage(TPtxPdfContent_ContentGenerator* pContentGenerator,
                                          TPtxPdfContent_Image* pImage, const TPtxGeomReal_Rectangle* pTargetRect);
/**
 * @brief Paint an image (stencil) mask.

An image mask is a monochrome image, in which each sample is specified by a single bit.
However, instead of being painted in opaque black and white,
the image mask is treated as a stencil mask that is partly opaque and partly transparent.
Sample values in the image do not represent black and white pixels;
rather, they designate places on the content that should either be marked
with the given paint or masked out (not marked at all).
Areas that are masked out retain their former content.

The effect is like applying paint in the current color through a cut-out stencil,
which allows the paint to reach the page in some places and masks it out in others.

* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.

* @param[in,out] pImageMask the image (stencil) mask

* @param[in] pTargetRect the target rectangle in the current coordinate system.
If targetRect is `NULL`, the unit rectangle <b>[0, 0, 1, 1]</b> is used.

* @param[in,out] pPaint the paint for filling marked pixels


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the page/group associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the document associated with <b>pImageMask</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pImageMask</b> object is not an image mask

 * - \ref ePtx_Error_IllegalArgument if the <b>pImageMask</b> is associated with a different document

 * - \ref ePtx_Error_IllegalArgument if the document associated with <b>pPaint</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pPaint</b> is associated with a different document


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ContentGenerator_PaintImageMask(
    TPtxPdfContent_ContentGenerator* pContentGenerator, TPtxPdfContent_ImageMask* pImageMask,
    const TPtxGeomReal_Rectangle* pTargetRect, TPtxPdfContent_Paint* pPaint);
/**
 * @brief Paint a path.
The path is first filled and then stroked
The blend mode for filling and stroking must be the same.

* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.

* @param[in,out] pPath the path to be painted

* @param[in,out] pFill the fill properties or `NULL` if the path should not be filled

* @param[in,out] pStroke the stroke properties or `NULL` if the path should not be stroked


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the page/group associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> and <b>pStroke</b> arguments are both `NULL`.

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pFill</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pFill</b> object belongs to a different document

 * - \ref ePtx_Error_IllegalArgument if the property \ref PtxPdfContent_Stroke_GetPaint "" of argument <b>pStroke</b> is
`NULL`.

 * - \ref ePtx_Error_IllegalArgument if the document associated with the property \ref PtxPdfContent_Stroke_GetPaint ""
of argument <b>pStroke</b> has already been closed.

 * - \ref ePtx_Error_IllegalArgument if the <b>pStroke</b> argument belongs to a different document

 * - \ref ePtx_Error_UnsupportedOperation if the \ref TPtxPdfContent_Paint "" objects for filling and stroking use
different blend modes


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentGenerator_PaintPath(TPtxPdfContent_ContentGenerator* pContentGenerator, TPtxPdfContent_Path* pPath,
                                         TPtxPdfContent_Fill* pFill, TPtxPdfContent_Stroke* pStroke);
/**
 * @brief Paint text.
* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.

* @param[in,out] pText the text to be painted


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the page/group associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pText</b> is associated with a different document

 * - \ref ePtx_Error_Generic if for some of the requested characters to paint the font's encoding is not defined or no
glyph exists in the font


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ContentGenerator_PaintText(
    TPtxPdfContent_ContentGenerator* pContentGenerator, TPtxPdfContent_Text* pText);
/**
 * @brief Intersect clip path with path.
Update the current clip path by intersecting with the given path.

* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.

* @param[in,out] pPath the path to intersect with the current clip path

* @param[in] iInsideRule the inside rule of the path argument


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the page/group associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pPath</b> is associated with a different document


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentGenerator_ClipWithPath(TPtxPdfContent_ContentGenerator* pContentGenerator,
                                            TPtxPdfContent_Path* pPath, TPtxPdfContent_InsideRule iInsideRule);
/**
 * @brief Intersect clip path with text.
Update the current  clip path by intersecting with the given text.

* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.

* @param[in,out] pText the text to intersect with the current clip path


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the page/group associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pText</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pText</b> is associated with a different document

 * - \ref ePtx_Error_Generic if for some of the requested characters to paint the font's encoding is not defined or no
glyph exists in the font


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ContentGenerator_ClipWithText(
    TPtxPdfContent_ContentGenerator* pContentGenerator, TPtxPdfContent_Text* pText);
/**
 * @brief Paint a group.
* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.

* @param[in,out] pGroup the group to be painted

* @param[in] pTargetRect the target rectangle in the current coordinate system.
If targetRect is `NULL`,
a default rectangle <b>[0, 0, width, height]</b> is used, where <b>width</b> and <b>height</b> are the dimensions of the
given <b>pGroup</b>'s \ref PtxPdfContent_Group_GetSize "Size"

* @param[in,out] pTransparency the transparency to be used when painting the group.
If Transparency is `NULL`, then the group is painted opaquely.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the page/group associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pGroup</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pGroup</b> is associated with a different document

 * - \ref ePtx_Error_IllegalArgument if the <b>pGroup</b> contains no content or the content generator has not been
closed yet

 * - \ref ePtx_Error_IllegalArgument if the <b>pGroup</b> contains interactive elements (see \ref
PtxPdfContent_Group_CopyFromPage "") and it has been painted before.

 * - \ref ePtx_Error_IllegalArgument if the <b>pGroup</b> contains interactive elements (see \ref
PtxPdfContent_Group_CopyFromPage "") and the content of the content generator belongs to an annotation.

 * - \ref ePtx_Error_Conformance if the <b>pTransparency</b> argument is not `NULL` and has properties \ref
PtxPdfContent_Transparency_GetAlpha "" other than 1.0 or \ref PtxPdfContent_Transparency_GetBlendMode "" other than \ref
ePtxPdfContent_BlendMode_Normal "", and the explicitly specified conformance does not support transparency (PDF/A-1,
PDF 1.0 - 1.3).


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ContentGenerator_PaintGroup(
    TPtxPdfContent_ContentGenerator* pContentGenerator, TPtxPdfContent_Group* pGroup,
    const TPtxGeomReal_Rectangle* pTargetRect, TPtxPdfContent_Transparency* pTransparency);
/**
 * @brief Paint a content element
* @param[in,out] pContentGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_ContentGenerator.

* @param[in,out] pContentElement the content element to be painted


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalState if the <b>pContentElement</b> object has already been closed

 * - \ref ePtx_Error_IllegalState if the content object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pContentElement</b> is associated with a different document


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ContentGenerator_AppendContentElement(
    TPtxPdfContent_ContentGenerator* pContentGenerator, TPtxPdfContent_ContentElement* pContentElement);

/**
 * @brief Close object.
 *
 * Close disposable objects by invoking this function.
 *
 * @param[in] pObject Disposable object.
 *
 * @return   \ref TRUE if the object was closed successfully; \ref FALSE if an error occured while closing the object.
 *           Retrieve the error code by calling \ref Ptx_GetLastError .
 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentGenerator_Close(TPtxPdfContent_ContentGenerator* pObject);
/******************************************************************************
 * PathGenerator
 *****************************************************************************/
/**
 * @brief Create a new path generator for appending to a path.
* @param[in,out] pPath the path object


 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_PathGenerator* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathGenerator_New(TPtxPdfContent_Path* pPath);

/**
 * @brief Move the current position.
Begin a new subpath by moving the current point to the specified coordinates,
omitting any connecting line segment.
If the previous path construction operator in the current path was also MoveTo,
the new MoveTo overrides it;
no vestige of the previous MoveTo operation remains in the path.

* @param[in,out] pPathGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_PathGenerator.

* @param[in] pTarget the target coordinates


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the path object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathGenerator_MoveTo(TPtxPdfContent_PathGenerator* pPathGenerator, const TPtxGeomReal_Point* pTarget);
/**
 * @brief Draw a line.
Append a straight line segment from the current point to the target coordinates.
The current position is changed to the target position.

* @param[in,out] pPathGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_PathGenerator.

* @param[in] pTarget the target coordinates


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the path object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathGenerator_LineTo(TPtxPdfContent_PathGenerator* pPathGenerator, const TPtxGeomReal_Point* pTarget);
/**
 * @brief Draw a bezier curve.

Append a cubic Bézier curve to the current path.
The curve extends from the current point to the <b>pTarget</b> position,
using <b>pControlPoint1</b> and <b>pControlPoint2</b> as the Bézier control points.

The current position is changed to the target position.

* @param[in,out] pPathGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_PathGenerator.

* @param[in] pControlPoint1 the first bezier control point

* @param[in] pControlPoint2 the second bezier control point

* @param[in] pTarget the target coordinates


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the path object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_PathGenerator_BezierTo(
    TPtxPdfContent_PathGenerator* pPathGenerator, const TPtxGeomReal_Point* pControlPoint1,
    const TPtxGeomReal_Point* pControlPoint2, const TPtxGeomReal_Point* pTarget);
/**
 * @brief Close the current subpath.
Close the current subpath by appending a straight line segment from the
current point to the starting point of the subpath.
This operator terminates the current subpath;
appending another segment to the current path will begin a new subpath,
even if the new segment begins at the endpoint reached by the closeSubpath() operation.
If the current subpath has already been closed, CloseSubpath() does nothing.

* @param[in,out] pPathGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_PathGenerator.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the path object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathGenerator_CloseSubpath(TPtxPdfContent_PathGenerator* pPathGenerator);
/**
 * @brief Append a rectangle to the current path as a complete subpath.
* @param[in,out] pPathGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_PathGenerator.

* @param[in] pRectangle the rectangle to be added to the path


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the path object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_PathGenerator_AddRectangle(
    TPtxPdfContent_PathGenerator* pPathGenerator, const TPtxGeomReal_Rectangle* pRectangle);
/**
 * @brief Append a circle to the current path as a complete subpath.
* @param[in,out] pPathGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_PathGenerator.

* @param[in] pCenter the center of the circle

* @param[in] dRadius the radius of the circle


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the path object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_PathGenerator_AddCircle(
    TPtxPdfContent_PathGenerator* pPathGenerator, const TPtxGeomReal_Point* pCenter, double dRadius);
/**
 * @brief Add an ellipse to the current path as a complete subpath.
* @param[in,out] pPathGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_PathGenerator.

* @param[in] pRectangle the surrounding rectangle of the ellipse


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the path object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_PathGenerator_AddEllipse(
    TPtxPdfContent_PathGenerator* pPathGenerator, const TPtxGeomReal_Rectangle* pRectangle);
/**
 * @brief Add an elliptical arc to the current path.
* @param[in,out] pPathGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_PathGenerator.

* @param[in] pRectangle the surrounding rectangle of the ellipse

* @param[in] dAlpha1 the angle between the x-axis and the begin of the arc

* @param[in] dAlpha2 the angle between the x-axis and the end of the arc


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the path object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathGenerator_AddArc(TPtxPdfContent_PathGenerator* pPathGenerator,
                                   const TPtxGeomReal_Rectangle* pRectangle, double dAlpha1, double dAlpha2);
/**
 * @brief Add an elliptical piece of pie to the current path as a complete subpath.
* @param[in,out] pPathGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_PathGenerator.

* @param[in] pRectangle the surrounding rectangle of the ellipse

* @param[in] dAlpha1 the angle between the x-axis and the begin of the arc

* @param[in] dAlpha2 the angle between the x-axis and the end of the arc


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the path object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathGenerator_AddPie(TPtxPdfContent_PathGenerator* pPathGenerator,
                                   const TPtxGeomReal_Rectangle* pRectangle, double dAlpha1, double dAlpha2);

/**
 * @brief Close object.
 *
 * Close disposable objects by invoking this function.
 *
 * @param[in] pObject Disposable object.
 *
 * @return   \ref TRUE if the object was closed successfully; \ref FALSE if an error occured while closing the object.
 *           Retrieve the error code by calling \ref Ptx_GetLastError .
 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathGenerator_Close(TPtxPdfContent_PathGenerator* pObject);
/******************************************************************************
 * TextGenerator
 *****************************************************************************/
/**
 * @brief Create a new text generator for appending text to a text content object.
All parameters that cannot be set in the constructor are set to default values:
  - Rendering: fill text with black paint
  - CharacterSpacing: 0
  - WordSpacing: 0
  - HorizontalScaling: 1
  - Leading: 1.2 times the <b>dFontSize</b>
(The <b>pFont</b>'s \ref PtxPdfContent_Font_GetLeading "" is not used.)
  - Rise: 0
  - Stroke: `NULL`
  - IntersectClipping: \ref "FALSE"

* @param[in,out] pText the text object

* @param[in,out] pFont the initial font

* @param[in] dFontSize the initial font size and leading

* @param[in] pLocation the initial position. If position is `NULL`,
the default position <b>[0, 0]</b> is used.


 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pText</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pText</b> and <b>pFont</b> objects are associated with different
documents

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pFont</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pFont</b> has been obtained from \ref PtxPdfContent_TextFragment_GetFont
""


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_TextGenerator* PDFTOOLS_TOOLBOX_CALL PtxPdfContent_TextGenerator_New(
    TPtxPdfContent_Text* pText, TPtxPdfContent_Font* pFont, double dFontSize, const TPtxGeomReal_Point* pLocation);

/**
 * @brief Get the width of a text string.
The width of a text string as if it would be shown with the current settings.

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] szText the text fragment


 * @return   the width of the text

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>szText</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_GetWidthA(TPtxPdfContent_TextGenerator* pTextGenerator, const char* szText);
/**
 * @brief Get the width of a text string.
The width of a text string as if it would be shown with the current settings.

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] szText the text fragment


 * @return   the width of the text

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>szText</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_GetWidthW(TPtxPdfContent_TextGenerator* pTextGenerator, const WCHAR* szText);
/**
 * @brief Show a text string.
The text is shown using the current settings.

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] szText the text to be shown


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>szText</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_ShowA(TPtxPdfContent_TextGenerator* pTextGenerator, const char* szText);
/**
 * @brief Show a text string.
The text is shown using the current settings.

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] szText the text to be shown


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>szText</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_ShowW(TPtxPdfContent_TextGenerator* pTextGenerator, const WCHAR* szText);
/**
 * @brief Show a text string and go to the next line.
* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] szText the text to be shown


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>szText</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_ShowLineA(TPtxPdfContent_TextGenerator* pTextGenerator, const char* szText);
/**
 * @brief Show a text string and go to the next line.
* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] szText the text to be shown


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>szText</b> argument is `NULL`


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_ShowLineW(TPtxPdfContent_TextGenerator* pTextGenerator, const WCHAR* szText);
/**
 * @brief Move the current position.
This also also sets the beginning of the current line to the specified position,
which will affect the \ref PtxPdfContent_TextGenerator_ShowLine "" method.

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] pTarget the target position


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_MoveTo(TPtxPdfContent_TextGenerator* pTextGenerator, const TPtxGeomReal_Point* pTarget);

/**
 * @brief The paint for filling
The fill paint or `NULL` if the text should not be filled.

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in,out] pFill Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_Paint "" object has already been closed.

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_Paint "" object belongs to a different document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_SetFill(TPtxPdfContent_TextGenerator* pTextGenerator, TPtxPdfContent_Paint* pFill);
/**
 * @brief The stroke properties
The stroke properties or `NULL` if the text should not be stroked.

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in,out] pStroke Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed.

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed.

 * - \ref ePtx_Error_IllegalArgument if the document associated with the given \ref TPtxPdfContent_Stroke "" object has
already been closed

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_Stroke "" object argument belongs to a different
document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_SetStroke(TPtxPdfContent_TextGenerator* pTextGenerator, TPtxPdfContent_Stroke* pStroke);
/**
 * @brief the current font.
The font is used for all subsequent \ref PtxPdfContent_TextGenerator_Show ""
and \ref PtxPdfContent_TextGenerator_ShowLine "" calls.

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in,out] pFont Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the document associated with the given \ref TPtxPdfContent_Font "" object has
already been closed

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_Font "" object is associated with a different
document

 * - \ref ePtx_Error_IllegalArgument if the given \ref TPtxPdfContent_Font "" object has been obtained from \ref
PtxPdfContent_TextFragment_GetFont ""


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_SetFont(TPtxPdfContent_TextGenerator* pTextGenerator, TPtxPdfContent_Font* pFont);
/**
 * @brief the current font size.

The font size is used for all subsequent \ref PtxPdfContent_TextGenerator_Show ""
and \ref PtxPdfContent_TextGenerator_ShowLine "" calls.

Note that this sets the font size only.
Also use \ref PtxPdfContent_TextGenerator_SetLeading "" to set the leading.

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] dFontSize Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the given value is smaller than 0.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_SetFontSize(TPtxPdfContent_TextGenerator* pTextGenerator, double dFontSize);
/**
 * @brief the current character spacing.

When the glyph for each character in the string is rendered,
the character spacing is added to the horizontal or vertical
component of the glyph’s displacement, depending on the writing mode.
It is subject to scaling by the horizontal scaling if the writing mode is horizontal.

Default value: 0

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] dCharacterSpacing Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_SetCharacterSpacing(TPtxPdfContent_TextGenerator* pTextGenerator, double dCharacterSpacing);
/**
 * @brief the current word spacing.

Word spacing works the same way as character spacing,
but applies only to the space character, code 32.

Default value: 0

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] dWordSpacing Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_SetWordSpacing(TPtxPdfContent_TextGenerator* pTextGenerator, double dWordSpacing);
/**
 * @brief the current horizontal scaling.

The horizontal scaling parameter adjusts the width of glyphs by stretching
or compressing them in the horizontal direction.
Its value is specified relative to the normal width of the glyphs,
with 1 being the normal width.

Default value: 1

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] dHorizontalScaling Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_TextGenerator_SetHorizontalScaling(
    TPtxPdfContent_TextGenerator* pTextGenerator, double dHorizontalScaling);
/**
 * @brief the current leading.

The leading parameter specifies the vertical distance between the
baselines of adjacent lines of text.
It affects only the method \ref PtxPdfContent_TextGenerator_ShowLine "".

Default value: 1.2 times the initial font size.

See also \ref PtxPdfContent_Font_GetLeading "".

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] dLeading Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_SetLeading(TPtxPdfContent_TextGenerator* pTextGenerator, double dLeading);
/**
 * @brief the current rise of the baseline.

Text rise specifies the distance to move the baseline up or down from its default location.
Positive values of text rise move the baseline up.
Adjustments to the baseline are useful for drawing superscripts or subscripts.

Default: 0

* @param[in,out] pTextGenerator Acts as a handle to the native object of type \ref TPtxPdfContent_TextGenerator.

* @param[in] dRise Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the underlying text object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_SetRise(TPtxPdfContent_TextGenerator* pTextGenerator, double dRise);

/**
 * @brief Close object.
 *
 * Close disposable objects by invoking this function.
 *
 * @param[in] pObject Disposable object.
 *
 * @return   \ref TRUE if the object was closed successfully; \ref FALSE if an error occured while closing the object.
 *           Retrieve the error code by calling \ref Ptx_GetLastError .
 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextGenerator_Close(TPtxPdfContent_TextGenerator* pObject);
/******************************************************************************
 * Paint
 *****************************************************************************/
/**
 * @brief Create an new paint.
Transparency is supported by PDF 1.4 or higher and by PDF/A-2 or higher.
The returned paint object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pColorSpace the color space of the paint

* @param[in] pColor the color components

* @param[in] nColors Size of the array `pColor`.

* @param[in,out] pTransparency the transparency parameters.
Use `NULL` to create an opaque paint.


 * @return   newly created paint object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_Conformance if the <b>pTransparency</b> argument is not `NULL`
and \ref PtxPdfContent_Transparency_GetAlpha "" of <b>pTransparency</b> is not 1.0
and the explicitly specified conformance does not support transparency (PDF/A-1, PDF 1.0 - 1.3).

 * - \ref ePtx_Error_Conformance if the <b>pTransparency</b> argument is not `NULL`
and \ref PtxPdfContent_Transparency_GetBlendMode "" is not \ref ePtxPdfContent_BlendMode_Normal ""
and the explicitly specified conformance does not support transparency (PDF/A-1, PDF 1.0 - 1.3).

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pColorSpace</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pColorSpace</b> is associated with a different document

 * - \ref ePtx_Error_IllegalArgument if the <b>pColor</b> argument contains too few elements

 * - \ref ePtx_Error_IllegalArgument if an element of the <b>pColor</b> argument is out of range


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Paint_Create(TPtxPdf_Document* pTargetDocument, TPtxPdfContent_ColorSpace* pColorSpace,
                           const double* pColor, size_t nColors, TPtxPdfContent_Transparency* pTransparency);

/**
The color space of this paint.

* @param[in,out] pPaint Acts as a handle to the native object of type \ref TPtxPdfContent_Paint.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed

 * - \ref ePtx_Error_IllegalState the object has been created with any creation method of the document.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ColorSpace* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Paint_GetColorSpace(TPtxPdfContent_Paint* pPaint);
/**
The color values of this paint.

* @param[in,out] pPaint Acts as a handle to the native object of type \ref TPtxPdfContent_Paint.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved array `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `-1` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed

 * - \ref ePtx_Error_IllegalState the object has been created with any creation method of the document.


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Paint_GetColor(TPtxPdfContent_Paint* pPaint,
                                                                                  double* pBuffer, size_t nBufferSize);
/**
The transparency parameters of this paint or `NULL` if this paint is opaque.

* @param[in,out] pPaint Acts as a handle to the native object of type \ref TPtxPdfContent_Paint.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed

 * - \ref ePtx_Error_IllegalState the object has been created with any creation method of the document.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Transparency* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Paint_GetTransparency(TPtxPdfContent_Paint* pPaint);

/******************************************************************************
 * Glyph
 *****************************************************************************/
/**
 * @brief glyph text
This is the glyph's associated text.

* @param[in,out] pGlyph Acts as a handle to the native object of type \ref TPtxPdfContent_Glyph.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Glyph_GetTextA(TPtxPdfContent_Glyph* pGlyph,
                                                                                  char* pBuffer, size_t nBufferSize);
/**
 * @brief glyph text
This is the glyph's associated text.

* @param[in,out] pGlyph Acts as a handle to the native object of type \ref TPtxPdfContent_Glyph.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Glyph_GetTextW(TPtxPdfContent_Glyph* pGlyph,
                                                                                  WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief glyph position

This is the position of the glyph within its \ref TPtxPdfContent_TextFragment "".
To find the point on the page this position has to be transformed with
\ref PtxPdfContent_TextFragment_GetTransform "".

The extent of the glyph with respect to its position depends on the text
fragment's \ref PtxPdfContent_TextFragment_GetWritingMode "".
In \ref ePtxPdfContent_WritingMode_Horizontal "" writing mode,
the glyph's position is at the left of the glyph on the height of the base line.
In \ref ePtxPdfContent_WritingMode_Vertical "" writing mode,
the glyph's position is at the middle of the glyph's horizontal extent,
vertically at the top.

* @param[in,out] pGlyph Acts as a handle to the native object of type \ref TPtxPdfContent_Glyph.

* @param[out] pPosition Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Glyph_GetPosition(TPtxPdfContent_Glyph* pGlyph,
                                                                                   TPtxGeomReal_Point*   pPosition);
/**
 * @brief glyph width
This is the width of the glyph.

* @param[in,out] pGlyph Acts as a handle to the native object of type \ref TPtxPdfContent_Glyph.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `-1.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Glyph_GetWidth(TPtxPdfContent_Glyph* pGlyph);

/******************************************************************************
 * TextFragment
 *****************************************************************************/
/**
 * @brief Get the number of elements in the list.
* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetCount(TPtxPdfContent_TextFragment* pTextFragment);
/**
 * @brief Returns the element at the specified position in the given list.
* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.

* @param[in] iIndex

 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Glyph* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_Get(TPtxPdfContent_TextFragment* pTextFragment, int iIndex);
/**
 * @brief Remove certain element from list.
* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.

* @param[in] iIndex

 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_Remove(TPtxPdfContent_TextFragment* pTextFragment, int iIndex);

/**
 * @brief the bounding box
This is a rectangle that encompasses all parts of the text fragment.

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.

* @param[out] pBoundingBox Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_TextFragment_GetBoundingBox(
    TPtxPdfContent_TextFragment* pTextFragment, TPtxGeomReal_Rectangle* pBoundingBox);
/**
 * @brief the transform to be applied to the bounding box rectangle
Use this transform matrix to compute the actual location of the text fragment's bounding box.

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.

* @param[out] pTransform Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_TextFragment_GetTransform(
    TPtxPdfContent_TextFragment* pTextFragment, TPtxGeomReal_AffineTransform* pTransform);
/**
The string painted by this text fragment

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetTextA(TPtxPdfContent_TextFragment* pTextFragment, char* pBuffer, size_t nBufferSize);
/**
The string painted by this text fragment

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
  `0` if either an error occurred or the returned buffer is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `0` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetTextW(TPtxPdfContent_TextFragment* pTextFragment, WCHAR* pBuffer, size_t nBufferSize);
/**
This text fragment's parameters for stroking the text or `NULL` if the text is not stroked

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Stroke* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetStroke(TPtxPdfContent_TextFragment* pTextFragment);
/**
This text fragment's parameters for filling the text or `NULL` if the text is not filled

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Fill* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetFill(TPtxPdfContent_TextFragment* pTextFragment);
/**
 * @brief The font size
* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetFontSize(TPtxPdfContent_TextFragment* pTextFragment);
/**
 * @brief The additional spacing between glyphs
When the glyph for each character in the text is rendered,
the character spacing is added to the horizontal or vertical
component of the glyph's displacement, depending on the writing mode.
It is subject to scaling by the horizontal scaling if the writing mode is horizontal.

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `-1.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetCharacterSpacing(TPtxPdfContent_TextFragment* pTextFragment);
/**
 * @brief The additional spacing between words
Word spacing works the same way as character spacing,
but applies only to the space character, code 32.

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `-1.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetWordSpacing(TPtxPdfContent_TextFragment* pTextFragment);
/**
 * @brief The horizontal scaling factor
The horizontal scaling parameter adjusts the width of glyphs by stretching
or compressing them in the horizontal direction.
Its value is specified relative to the normal width of the glyphs,
with 1 being the normal width.

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetHorizontalScaling(TPtxPdfContent_TextFragment* pTextFragment);
/**
 * @brief The rise of the baseline
The text rise specifies the distance to move the baseline up or down from its default location.
Positive values of text rise move the baseline up.
Adjustments to the baseline are useful for drawing superscripts or subscripts.

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `-1.0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT double PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetRise(TPtxPdfContent_TextFragment* pTextFragment);
/**
 * @brief The writing direction
This is the writing mode for the text fragment.
It applies to all contained \ref TPtxPdfContent_Glyph ""s.

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_WritingMode PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetWritingMode(TPtxPdfContent_TextFragment* pTextFragment);
/**
 * @brief The font
The returned \ref TPtxPdfContent_Font "" can only be used for extraction purposes.
Specifically, using this object in \ref PtxPdfContent_TextGenerator_New "" or in \ref
PtxPdfContent_TextGenerator_SetFont "" results in a \ref ePtx_Error_IllegalArgument "" error.

* @param[in,out] pTextFragment Acts as a handle to the native object of type \ref TPtxPdfContent_TextFragment.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Font* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextFragment_GetFont(TPtxPdfContent_TextFragment* pTextFragment);

/******************************************************************************
 * Text
 *****************************************************************************/
/**
 * @brief Create a text
The returned text object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument the output document with which the returned object is associated


 * @return   the newly created text object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Text* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Text_Create(TPtxPdf_Document* pTargetDocument);
/**
 * @brief Get the number of elements in the list.
* @param[in,out] pText Acts as a handle to the native object of type \ref TPtxPdfContent_Text.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Text_GetCount(TPtxPdfContent_Text* pText);
/**
 * @brief Returns the element at the specified position in the given list.
* @param[in,out] pText Acts as a handle to the native object of type \ref TPtxPdfContent_Text.

* @param[in] iIndex

 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_TextFragment* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_Text_Get(TPtxPdfContent_Text* pText, int iIndex);
/**
 * @brief Clear list.
* @param[in,out] pText Acts as a handle to the native object of type \ref TPtxPdfContent_Text.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Text_Clear(TPtxPdfContent_Text* pText);
/**
 * @brief Remove certain element from list.
* @param[in,out] pText Acts as a handle to the native object of type \ref TPtxPdfContent_Text.

* @param[in] iIndex

 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_Text_Remove(TPtxPdfContent_Text* pText, int iIndex);

/******************************************************************************
 * ContentElement
 *****************************************************************************/
/**
 * @brief Copy a content element
The returned object is associated with the given target document but not yet part of it.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pContentElement a content element of a different document


 * @return   the copied content element, associated with the current document

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the source document or writing to the target document

 * - \ref ePtx_Error_Conformance The conformance level of the source document is not compatible
with the conformance level of the target document.

 * - \ref ePtx_Error_Conformance The conformance of the <b>pTargetDocument</b> is PDF/A with a conformance level "a":
PDF/A-1a, PDF/A-2a, PDF/A-3a.

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument the <b>pContentElement</b> object is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument the document associated with the <b>pContentElement</b> object has already been
closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ContentElement* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentElement_Copy(TPtxPdf_Document* pTargetDocument, TPtxPdfContent_ContentElement* pContentElement);

/**
 * @brief the bounding box
This is a rectangle that encompasses all parts of an element.

* @param[in,out] pContentElement Acts as a handle to the native object of type \ref TPtxPdfContent_ContentElement.

* @param[out] pBoundingBox Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ContentElement_GetBoundingBox(
    TPtxPdfContent_ContentElement* pContentElement, TPtxGeomReal_Rectangle* pBoundingBox);
/**
 * @brief the transform to be applied to the alignment rectangle
Use this transform matrix to compute the actual location of the element's alignment rectangle.

* @param[in,out] pContentElement Acts as a handle to the native object of type \ref TPtxPdfContent_ContentElement.

* @param[out] pTransform Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the transform to be set is non-invertible

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ContentElement_GetTransform(
    TPtxPdfContent_ContentElement* pContentElement, TPtxGeomReal_AffineTransform* pTransform);
/**
 * @brief the transform to be applied to the alignment rectangle
Use this transform matrix to compute the actual location of the element's alignment rectangle.

* @param[in,out] pContentElement Acts as a handle to the native object of type \ref TPtxPdfContent_ContentElement.

* @param[in] pTransform Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the transform to be set is non-invertible

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ContentElement_SetTransform(
    TPtxPdfContent_ContentElement* pContentElement, const TPtxGeomReal_AffineTransform* pTransform);

/**
 * @brief Get actual derived type of base type \ref TPtxPdfContent_ContentElement.
 *
 * This function is invoked prior to downcasting to ascertain the derived object type.
 *
 * @param[in,out] pContentElement Acts as a handle to a native object.
 *
 * @return The item of the enumeration \ref TPtxPdfContent_ContentElementType that refers to the actual derived type.
 * `0` in case of an error.
 *
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 *       Get the error message with \ref Ptx_GetLastErrorMessage.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ContentElementType PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentElement_GetType(TPtxPdfContent_ContentElement* pContentElement);
/******************************************************************************
 * TextElement
 *****************************************************************************/
/**
This text element's text object.

* @param[in,out] pTextElement Acts as a handle to the native object of type \ref TPtxPdfContent_TextElement.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Text* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_TextElement_GetText(TPtxPdfContent_TextElement* pTextElement);

/******************************************************************************
 * GroupElement
 *****************************************************************************/
/**
 * @brief Copy a group element without copying its content
Create a new group element, taking a given group element as a template.
The newly created group has the same properties, such as geometric transform, clipping, and soft mask, but it's content
is empty. The returned object is associated with the given target document but not yet part of it.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pGroupElement a group element of a different document


 * @return   the new group element, associated with the current document

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the source document or writing to the target document

 * - \ref ePtx_Error_Conformance The conformance level of the source document is not compatible
with the conformance level of the target document.

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument the <b>pGroupElement</b> object is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument the document associated with the <b>pGroupElement</b> object has already been
closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_GroupElement* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_GroupElement_CopyWithoutContent(TPtxPdf_Document*            pTargetDocument,
                                              TPtxPdfContent_GroupElement* pGroupElement);

/**
This group element's group object.

* @param[in,out] pGroupElement Acts as a handle to the native object of type \ref TPtxPdfContent_GroupElement.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Group* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_GroupElement_GetGroup(TPtxPdfContent_GroupElement* pGroupElement);

/******************************************************************************
 * PathElement
 *****************************************************************************/
/**
 * @brief the box for alignment
This is a rectangle that may not encompass all parts of an element, but is usefull for alignment.

* @param[in,out] pPathElement Acts as a handle to the native object of type \ref TPtxPdfContent_PathElement.

* @param[out] pAlignmentBox Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_PathElement_GetAlignmentBox(
    TPtxPdfContent_PathElement* pPathElement, TPtxGeomReal_Rectangle* pAlignmentBox);
/**
This path element's path object.

* @param[in,out] pPathElement Acts as a handle to the native object of type \ref TPtxPdfContent_PathElement.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Path* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathElement_GetPath(TPtxPdfContent_PathElement* pPathElement);
/**
This path element's parameters for stroking the path or `NULL` if the path is not stroked.

* @param[in,out] pPathElement Acts as a handle to the native object of type \ref TPtxPdfContent_PathElement.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Stroke* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathElement_GetStroke(TPtxPdfContent_PathElement* pPathElement);
/**
This path element's parameters for filling the path or `NULL` if the path is not filled.

* @param[in,out] pPathElement Acts as a handle to the native object of type \ref TPtxPdfContent_PathElement.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Fill* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_PathElement_GetFill(TPtxPdfContent_PathElement* pPathElement);

/******************************************************************************
 * ImageElement
 *****************************************************************************/
/**
This element's image.

* @param[in,out] pImageElement Acts as a handle to the native object of type \ref TPtxPdfContent_ImageElement.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Image* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ImageElement_GetImage(TPtxPdfContent_ImageElement* pImageElement);

/******************************************************************************
 * ImageMaskElement
 *****************************************************************************/
/**
This element's image mask.

* @param[in,out] pImageMaskElement Acts as a handle to the native object of type \ref TPtxPdfContent_ImageMaskElement.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ImageMask* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ImageMaskElement_GetImageMask(TPtxPdfContent_ImageMaskElement* pImageMaskElement);
/**
The paint used to draw the image mask.

* @param[in,out] pImageMaskElement Acts as a handle to the native object of type \ref TPtxPdfContent_ImageMaskElement.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_Paint* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ImageMaskElement_GetPaint(TPtxPdfContent_ImageMaskElement* pImageMaskElement);

/******************************************************************************
 * ContentExtractor
 *****************************************************************************/
/**
 * @brief Create a new content extractor
* @param[in,out] pContent the content object of a page or group


 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IO Error reading from the document

 * - \ref ePtx_Error_Corrupt The document is corrupt

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pContent</b> object has already been closed

 * - \ref ePtx_Error_IllegalArgument if the document associated with the content has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pContent</b>'s document is an output document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ContentExtractor* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentExtractor_New(TPtxPdfContent_Content* pContent);

/**
 * @brief Get an iterator for this enumerable.
* @param[in,out] pContentExtractor Acts as a handle to the native object of type \ref TPtxPdfContent_ContentExtractor.


 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ContentExtractorIterator* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentExtractor_GetIterator(TPtxPdfContent_ContentExtractor* pContentExtractor);

/**
Configures the extractor's behavior regarding the selection of groups to be un-grouped.
Default value: \ref ePtxPdfContent_UngroupingSelection_None "".

* @param[in,out] pContentExtractor Acts as a handle to the native object of type \ref TPtxPdfContent_ContentExtractor.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_UngroupingSelection PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentExtractor_GetUngrouping(TPtxPdfContent_ContentExtractor* pContentExtractor);
/**
Configures the extractor's behavior regarding the selection of groups to be un-grouped.
Default value: \ref ePtxPdfContent_UngroupingSelection_None "".

* @param[in,out] pContentExtractor Acts as a handle to the native object of type \ref TPtxPdfContent_ContentExtractor.

* @param[in] iUngrouping Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfContent_ContentExtractor_SetUngrouping(
    TPtxPdfContent_ContentExtractor* pContentExtractor, TPtxPdfContent_UngroupingSelection iUngrouping);

/******************************************************************************
 * ContentExtractorIterator
 *****************************************************************************/
/**
 * @brief Get the current element or `NULL`if no elements are left.
* @param[in,out] pContentExtractorIterator Acts as a handle to the native object of type \ref
TPtxPdfContent_ContentExtractorIterator.


 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfContent_ContentElement* PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentExtractorIterator_GetValue(TPtxPdfContent_ContentExtractorIterator* pContentExtractorIterator);
/**
 * @brief Move the iterator to the next element.
* @param[in,out] pContentExtractorIterator Acts as a handle to the native object of type \ref
TPtxPdfContent_ContentExtractorIterator.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfContent_ContentExtractorIterator_MoveNext(TPtxPdfContent_ContentExtractorIterator* pContentExtractorIterator);

#ifdef __cplusplus
}
#endif

#endif /* PDFTOOLS_TOOLBOX_PTXPDFCONTENT_H__ */
