/******************************************************************************
 *
 * File:            PdfTools_Toolbox_PtxPdfNav.h
 *
 * Description:     Sub Header file for Toolbox SDK
 *
 * Author:          PDF Tools AG
 *
 * Copyright:       Copyright (C) 2024 - 2024 PDF Tools AG, Switzerland
 *                  All rights reserved.
 *
 * Notice:          By downloading and using this artifact, you accept PDF Tools AG's
 *                  [license agreement](https://www.pdf-tools.com/license-agreement/),
 *                  [privacy policy](https://www.pdf-tools.com/privacy-policy/),
 *                  and allow PDF Tools AG to track your usage data.
 *
 *****************************************************************************/

#ifndef PDFTOOLS_TOOLBOX_PTXPDFNAV_H__
#define PDFTOOLS_TOOLBOX_PTXPDFNAV_H__

#ifndef PDFTOOLS_TOOLBOX_CALL
#if defined(WIN32)
#define PDFTOOLS_TOOLBOX_CALL __stdcall
#else
#define PDFTOOLS_TOOLBOX_CALL
#endif
#endif

#include "PdfTools_Toolbox_Types.h"
#include "PdfTools_Toolbox_PtxSys.h"

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef _UNICODE
#define PtxPdfNav_OutlineItem_Create PtxPdfNav_OutlineItem_CreateW

#define PtxPdfNav_OutlineItem_GetTitle PtxPdfNav_OutlineItem_GetTitleW
#define PtxPdfNav_OutlineItem_SetTitle PtxPdfNav_OutlineItem_SetTitleW

#define PtxPdfNav_NamedDestination_Create PtxPdfNav_NamedDestination_CreateW

#define PtxPdfNav_NamedDestination_GetName PtxPdfNav_NamedDestination_GetNameW

#define PtxPdfNav_WebLink_Create                   PtxPdfNav_WebLink_CreateW
#define PtxPdfNav_WebLink_CreateFromQuadrilaterals PtxPdfNav_WebLink_CreateFromQuadrilateralsW

#define PtxPdfNav_WebLink_GetUri PtxPdfNav_WebLink_GetUriW
#define PtxPdfNav_WebLink_SetUri PtxPdfNav_WebLink_SetUriW

#else
#define PtxPdfNav_OutlineItem_Create PtxPdfNav_OutlineItem_CreateA

#define PtxPdfNav_OutlineItem_GetTitle PtxPdfNav_OutlineItem_GetTitleA
#define PtxPdfNav_OutlineItem_SetTitle PtxPdfNav_OutlineItem_SetTitleA

#define PtxPdfNav_NamedDestination_Create PtxPdfNav_NamedDestination_CreateA

#define PtxPdfNav_NamedDestination_GetName PtxPdfNav_NamedDestination_GetNameA

#define PtxPdfNav_WebLink_Create                   PtxPdfNav_WebLink_CreateA
#define PtxPdfNav_WebLink_CreateFromQuadrilaterals PtxPdfNav_WebLink_CreateFromQuadrilateralsA

#define PtxPdfNav_WebLink_GetUri PtxPdfNav_WebLink_GetUriA
#define PtxPdfNav_WebLink_SetUri PtxPdfNav_WebLink_SetUriA

#endif

/******************************************************************************
 * ViewerSettings
 *****************************************************************************/
/**
 * @brief Copy viewer settings
The newly created viewer settings are associated with the target document,
but not yet used as the document's viewer settings.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pViewerSettings the viewer settings of a different document


 * @return   a viewer settings object with the same content, but associated with the current document.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_Conformance The conformance level of the source document is not compatible
with the conformance level of the target document.

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pViewerSettings</b> argument is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pViewerSettings</b> object has already been
closed

 * - \ref ePtx_Error_IO Error reading from the stream


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_ViewerSettings* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_ViewerSettings_Copy(TPtxPdf_Document* pTargetDocument, TPtxPdfNav_ViewerSettings* pViewerSettings);

/**
The positional arrangment for displaying pages when opening the document in a viewer.
If `NULL` then the viewer acts according to it's default behavior.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.

* @param[out] pPageDisplay Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_ViewerSettings_GetPageDisplay(
    TPtxPdfNav_ViewerSettings* pViewerSettings, TPtxPdfNav_PageDisplay* pPageDisplay);
/**
The positional arrangment for displaying pages when opening the document in a viewer.
If `NULL` then the viewer acts according to it's default behavior.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.

* @param[in] pPageDisplay Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_ViewerSettings_SetPageDisplay(
    TPtxPdfNav_ViewerSettings* pViewerSettings, const TPtxPdfNav_PageDisplay* pPageDisplay);
/**
The initially visible side pane when opening the document in a viewer.
If `NULL` then the viewer acts according to it's default behavior.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.

* @param[out] pActivePane Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_ViewerSettings_GetActivePane(
    TPtxPdfNav_ViewerSettings* pViewerSettings, TPtxPdfNav_ViewerNavigationPane* pActivePane);
/**
The initially visible side pane when opening the document in a viewer.
If `NULL` then the viewer acts according to it's default behavior.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.

* @param[in] pActivePane Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_ViewerSettings_SetActivePane(
    TPtxPdfNav_ViewerSettings* pViewerSettings, const TPtxPdfNav_ViewerNavigationPane* pActivePane);
/**
If \ref "TRUE" then opening the document will make the viewer try to enter full screen mode.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_ViewerSettings_GetFullScreen(TPtxPdfNav_ViewerSettings* pViewerSettings);
/**
If \ref "TRUE" then opening the document will make the viewer try to enter full screen mode.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.

* @param[in] bFullScreen Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_ViewerSettings_SetFullScreen(TPtxPdfNav_ViewerSettings* pViewerSettings, BOOL bFullScreen);
/**
If \ref "TRUE" then opening the document will instruct the viewer to hide it's toolbar.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_ViewerSettings_GetHideToolbar(TPtxPdfNav_ViewerSettings* pViewerSettings);
/**
If \ref "TRUE" then opening the document will instruct the viewer to hide it's toolbar.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.

* @param[in] bHideToolbar Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_ViewerSettings_SetHideToolbar(TPtxPdfNav_ViewerSettings* pViewerSettings, BOOL bHideToolbar);
/**
If \ref "TRUE" then opening the document will instruct the viewer to hide it's menu bar.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_ViewerSettings_GetHideMenubar(TPtxPdfNav_ViewerSettings* pViewerSettings);
/**
If \ref "TRUE" then opening the document will instruct the viewer to hide it's menu bar.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.

* @param[in] bHideMenubar Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_ViewerSettings_SetHideMenubar(TPtxPdfNav_ViewerSettings* pViewerSettings, BOOL bHideMenubar);
/**
If \ref "TRUE" then opening the document will instruct the viewer to display the document's title from the metadata
instead of it's file name.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_ViewerSettings_GetDisplayDocumentTitle(TPtxPdfNav_ViewerSettings* pViewerSettings);
/**
If \ref "TRUE" then opening the document will instruct the viewer to display the document's title from the metadata
instead of it's file name.

* @param[in,out] pViewerSettings Acts as a handle to the native object of type \ref TPtxPdfNav_ViewerSettings.

* @param[in] bDisplayDocumentTitle Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the document has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_ViewerSettings_SetDisplayDocumentTitle(
    TPtxPdfNav_ViewerSettings* pViewerSettings, BOOL bDisplayDocumentTitle);

/******************************************************************************
 * OutlineItem
 *****************************************************************************/
/**
 * @brief Create a new outline item (bookmark).
The returned outline item is not yet part of the outline item tree, but it is associated with the given target document.
It can be inserted at any position in the tree.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in] szTitle The title of the newly created outline item.

* @param[in,out] pDestination The destination that this outline item refers to or
`NULL` if the item has no destination.


 * @return   The newly created outline item.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pDestination</b> argument has already been
closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pDestination</b> argument belongs to a different document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_OutlineItem* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineItem_CreateA(
    TPtxPdf_Document* pTargetDocument, const char* szTitle, TPtxPdfNav_Destination* pDestination);
/**
 * @brief Create a new outline item (bookmark).
The returned outline item is not yet part of the outline item tree, but it is associated with the given target document.
It can be inserted at any position in the tree.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in] szTitle The title of the newly created outline item.

* @param[in,out] pDestination The destination that this outline item refers to or
`NULL` if the item has no destination.


 * @return   The newly created outline item.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pDestination</b> argument has already been
closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pDestination</b> argument belongs to a different document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_OutlineItem* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineItem_CreateW(
    TPtxPdf_Document* pTargetDocument, const WCHAR* szTitle, TPtxPdfNav_Destination* pDestination);
/**
 * @brief Copy an outline item
Copy an outline item object including all descendants from an input document to the given <b>pTargetDocument</b>.
The returned object is associated with the given target document but not yet part of it.

* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pOutlineItem An outline item of a different document

* @param[in,out] pOptions The options used to copy the item


 * @return   The copied outline item, associated with the current document.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_Conformance The conformance level of the source document is not compatible
with the conformance level of the target document.

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is `NULL`

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pOutlineItem</b> argument is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument the target document contains implicitly copied outline items

 * - \ref ePtx_Error_IllegalArgument the <b>pOutlineItem</b> argument is `NULL`

 * - \ref ePtx_Error_IllegalArgument the <b>pOutlineItem</b> argument is not associated with an input document

 * - \ref ePtx_Error_IO Error reading from the source document or writing to the target document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_OutlineItem* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineItem_Copy(
    TPtxPdf_Document* pTargetDocument, TPtxPdfNav_OutlineItem* pOutlineItem, TPtxPdfNav_OutlineCopyOptions* pOptions);

/**
The title of the outline item.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItem_GetTitleA(TPtxPdfNav_OutlineItem* pOutlineItem, char* pBuffer, size_t nBufferSize);
/**
The title of the outline item.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItem_GetTitleW(TPtxPdfNav_OutlineItem* pOutlineItem, WCHAR* pBuffer, size_t nBufferSize);
/**
The title of the outline item.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.

* @param[in] szTitle Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineItem_SetTitleA(TPtxPdfNav_OutlineItem* pOutlineItem,
                                                                                   const char*             szTitle);
/**
The title of the outline item.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.

* @param[in] szTitle Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineItem_SetTitleW(TPtxPdfNav_OutlineItem* pOutlineItem,
                                                                                   const WCHAR*            szTitle);
/**
If \ref "TRUE", the outline item is displayed in bold font.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineItem_GetBold(TPtxPdfNav_OutlineItem* pOutlineItem);
/**
If \ref "TRUE", the outline item is displayed in bold font.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.

* @param[in] bBold Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineItem_SetBold(TPtxPdfNav_OutlineItem* pOutlineItem,
                                                                                 BOOL                    bBold);
/**
If \ref "TRUE", the outline item is displayed in italic font.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItem_GetItalic(TPtxPdfNav_OutlineItem* pOutlineItem);
/**
If \ref "TRUE", the outline item is displayed in italic font.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.

* @param[in] bItalic Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineItem_SetItalic(TPtxPdfNav_OutlineItem* pOutlineItem,
                                                                                   BOOL                    bItalic);
/**
The destination of the outline item.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.


 * @return   Retrieved value.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an input document.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_Destination* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItem_GetDestination(TPtxPdfNav_OutlineItem* pOutlineItem);
/**
The destination of the outline item.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.

* @param[in,out] pDestination Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an input document.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItem_SetDestination(TPtxPdfNav_OutlineItem* pOutlineItem, TPtxPdfNav_Destination* pDestination);
/**
  - If \ref "TRUE", the item is expanded.
  - If \ref "FALSE", the item is collapsed.
This is property is only meaningful if the item has child items.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineItem_IsOpen(TPtxPdfNav_OutlineItem* pOutlineItem);
/**
  - If \ref "TRUE", the item is expanded.
  - If \ref "FALSE", the item is collapsed.
This is property is only meaningful if the item has child items.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.

* @param[in] bIsOpen Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_UnsupportedOperation the object is not associated to an output document.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineItem_SetIsOpen(TPtxPdfNav_OutlineItem* pOutlineItem,
                                                                                   BOOL                    bIsOpen);
/**
The child items of this outline item.

* @param[in,out] pOutlineItem Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItem.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_OutlineItemList* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItem_GetChildren(TPtxPdfNav_OutlineItem* pOutlineItem);

/******************************************************************************
 * OutlineItemList
 *****************************************************************************/
/**
 * @brief Get the number of elements in the list.
* @param[in,out] pOutlineItemList Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItemList.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItemList_GetCount(TPtxPdfNav_OutlineItemList* pOutlineItemList);
/**
 * @brief Returns the element at the specified position in the given list.
* @param[in,out] pOutlineItemList Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItemList.

* @param[in] iIndex

 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_OutlineItem* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItemList_Get(TPtxPdfNav_OutlineItemList* pOutlineItemList, int iIndex);
/**
 * @brief Add an element to the end of the list.
* @param[in,out] pOutlineItemList Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItemList.

* @param[in,out] pOutlineItem

 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItemList_Add(TPtxPdfNav_OutlineItemList* pOutlineItemList, TPtxPdfNav_OutlineItem* pOutlineItem);
/**
 * @brief Clear list.
* @param[in,out] pOutlineItemList Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItemList.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItemList_Clear(TPtxPdfNav_OutlineItemList* pOutlineItemList);
/**
 * @brief Remove certain element from list.
* @param[in,out] pOutlineItemList Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItemList.

* @param[in] iIndex

 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItemList_Remove(TPtxPdfNav_OutlineItemList* pOutlineItemList, int iIndex);
/**
 * @brief Update certain element in list.
* @param[in,out] pOutlineItemList Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineItemList.

* @param[in] iIndex
* @param[in,out] pValue

 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineItemList_Set(TPtxPdfNav_OutlineItemList* pOutlineItemList, int iIndex, TPtxPdfNav_OutlineItem* pValue);

/******************************************************************************
 * Destination
 *****************************************************************************/
/**
 * @brief The target destination
For direct destinations,
this is just the destination itself.

* @param[in,out] pDestination Acts as a handle to the native object of type \ref TPtxPdfNav_Destination.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_DirectDestination* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_Destination_GetTarget(TPtxPdfNav_Destination* pDestination);

/**
 * @brief Get actual derived type of base type \ref TPtxPdfNav_Destination.
 *
 * This function is invoked prior to downcasting to ascertain the derived object type.
 *
 * @param[in,out] pDestination Acts as a handle to a native object.
 *
 * @return The item of the enumeration \ref TPtxPdfNav_DestinationType that refers to the actual derived type. `0` in
 * case of an error.
 *
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 *       Get the error message with \ref Ptx_GetLastErrorMessage.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_DestinationType PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_Destination_GetType(TPtxPdfNav_Destination* pDestination);
/******************************************************************************
 * NamedDestination
 *****************************************************************************/
/**
 * @brief Create a named destination
The returned object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in] szName The name by which the destination is referred to.

* @param[in,out] pTarget The target destination


 * @return   The newly created named destination.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pTarget</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTarget</b> argument belongs to a different document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_NamedDestination* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_NamedDestination_CreateA(
    TPtxPdf_Document* pTargetDocument, const char* szName, TPtxPdfNav_DirectDestination* pTarget);
/**
 * @brief Create a named destination
The returned object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in] szName The name by which the destination is referred to.

* @param[in,out] pTarget The target destination


 * @return   The newly created named destination.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pTarget</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTarget</b> argument belongs to a different document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_NamedDestination* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_NamedDestination_CreateW(
    TPtxPdf_Document* pTargetDocument, const WCHAR* szName, TPtxPdfNav_DirectDestination* pTarget);

/**
The name by which the destination is referred to.

* @param[in,out] pNamedDestination Acts as a handle to the native object of type \ref TPtxPdfNav_NamedDestination.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_NamedDestination_GetNameA(TPtxPdfNav_NamedDestination* pNamedDestination, char* pBuffer, size_t nBufferSize);
/**
The name by which the destination is referred to.

* @param[in,out] pNamedDestination Acts as a handle to the native object of type \ref TPtxPdfNav_NamedDestination.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_NamedDestination_GetNameW(TPtxPdfNav_NamedDestination* pNamedDestination, WCHAR* pBuffer, size_t nBufferSize);

/******************************************************************************
 * DirectDestination
 *****************************************************************************/
/**
The page in the document that this destination is pointing to.

* @param[in,out] pDirectDestination Acts as a handle to the native object of type \ref TPtxPdfNav_DirectDestination.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.

 * - \ref ePtx_Error_Corrupt the page could not be found or does not exist in the document.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdf_Page* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_DirectDestination_GetPage(TPtxPdfNav_DirectDestination* pDirectDestination);

/**
 * @brief Get actual derived type of base type \ref TPtxPdfNav_DirectDestination.
 *
 * This function is invoked prior to downcasting to ascertain the derived object type.
 *
 * @param[in,out] pDirectDestination Acts as a handle to a native object.
 *
 * @return The item of the enumeration \ref TPtxPdfNav_DirectDestinationType that refers to the actual derived type. `0`
 * in case of an error.
 *
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 *       Get the error message with \ref Ptx_GetLastErrorMessage.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_DirectDestinationType PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_DirectDestination_GetType(TPtxPdfNav_DirectDestination* pDirectDestination);
/******************************************************************************
 * LocationZoomDestination
 *****************************************************************************/
/**
 * @brief Create a new LocationZoomDestination
The returned object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in,out] pPage The page in the document that this destination is pointing to.

* @param[in] pLeft The location of the page that is displayed at the left border
of the viewport or `NULL`.
See property \ref PtxPdfNav_LocationZoomDestination_GetLeft "" for more information.

* @param[in] pTop The location of the page that is displayed at the top
of the viewport or `NULL`.
See property \ref PtxPdfNav_LocationZoomDestination_GetTop "" for more information.

* @param[in] pZoom The zoom factor that is applied when jumping to the destination
or `NULL`.
See property \ref PtxPdfNav_LocationZoomDestination_GetZoom "" for more information.


 * @return   The newly created destination object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b>differs from the document associated with <b>pPage</b>

 * - \ref ePtx_Error_IllegalArgument If the document associated with the <b>pPage</b> argument has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_LocationZoomDestination* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_LocationZoomDestination_Create(TPtxPdf_Document* pTargetDocument, TPtxPdf_Page* pPage, const double* pLeft,
                                         const double* pTop, const double* pZoom);

/**

The location of the page that is displayed at the left border of the viewport (if possible).

If the property is `NULL`, the value from before the jump is retained.

Note: Due to the current zoom factor,
it is usually not possible for viewers to scroll as far to the right side,
as would be necessary to place the location at the left corner of the viewport.
However, viewers will ensure, that the location is at least visible.

In practice this means, that this value is mostly irrelevant.

* @param[in,out] pLocationZoomDestination Acts as a handle to the native object of type \ref
TPtxPdfNav_LocationZoomDestination.

* @param[out] pLeft Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_LocationZoomDestination_GetLeft(TPtxPdfNav_LocationZoomDestination* pLocationZoomDestination, double* pLeft);
/**

The location of the page that is displayed at the top of the viewport (if possible).

If the property is `NULL`, the value from before the jump is retained.

* @param[in,out] pLocationZoomDestination Acts as a handle to the native object of type \ref
TPtxPdfNav_LocationZoomDestination.

* @param[out] pTop Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_LocationZoomDestination_GetTop(TPtxPdfNav_LocationZoomDestination* pLocationZoomDestination, double* pTop);
/**

The zoom factor that is applied when jumping to the destination.

A value of `NULL` means that the current zoom level is retained.

Note: Many PDF viewers support different viewing modes like "fit page" or "fit width".

A \ref TPtxPdfNav_LocationZoomDestination "" with a \ref PtxPdfNav_LocationZoomDestination_GetZoom "" value of `NULL`
will usually not change the current viewing mode in most viewers.

For other \ref PtxPdfNav_LocationZoomDestination_GetZoom "" values however,
the viewer must switch to the standard mode,
i.e. deactivate special modes like "fit page" or "fit width".

Because of this,
using a \ref PtxPdfNav_LocationZoomDestination_GetZoom "" value other than `NULL` is discouraged.

* @param[in,out] pLocationZoomDestination Acts as a handle to the native object of type \ref
TPtxPdfNav_LocationZoomDestination.

* @param[out] pZoom Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_LocationZoomDestination_GetZoom(TPtxPdfNav_LocationZoomDestination* pLocationZoomDestination, double* pZoom);

/******************************************************************************
 * FitPageDestination
 *****************************************************************************/
/**
 * @brief Create a new FitPageDestination
The returned object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in,out] pPage The page in the document that this destination is pointing to.

* @param[in] bFitActualContent If \ref "TRUE", the viewport is fitted to the actual content of the page,
instead of the size of the page.
See property \ref PtxPdfNav_FitPageDestination_GetFitActualContent "" for more information.


 * @return   The newly created destination object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b>differs from the document associated with <b>pPage</b>

 * - \ref ePtx_Error_IllegalArgument If the document associated with the <b>pPage</b> argument has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_FitPageDestination* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_FitPageDestination_Create(TPtxPdf_Document* pTargetDocument, TPtxPdf_Page* pPage, BOOL bFitActualContent);

/**

If \ref "TRUE", the viewport is fitted to the actual content of the page,
instead of the size of the page.

Note: Many PDF viewers simply ignore this property and always treat it
as \ref "FALSE", i.e. switching to "fit page" mode anyway.

* @param[in,out] pFitPageDestination Acts as a handle to the native object of type \ref TPtxPdfNav_FitPageDestination.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_FitPageDestination_GetFitActualContent(TPtxPdfNav_FitPageDestination* pFitPageDestination);

/******************************************************************************
 * FitWidthDestination
 *****************************************************************************/
/**
 * @brief Create a new FitWidthDestination
The returned object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in,out] pPage The page in the document that this destination is pointing to.

* @param[in] bFitActualContent If \ref "TRUE", the viewport is fitted to the actual content of the page,
instead of the size of the page.
See property \ref PtxPdfNav_FitWidthDestination_GetFitActualContent "" for more information.


 * @return   The newly created destination object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b>differs from the document associated with <b>pPage</b>

 * - \ref ePtx_Error_IllegalArgument If the document associated with the <b>pPage</b> argument has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_FitWidthDestination* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_FitWidthDestination_Create(TPtxPdf_Document* pTargetDocument, TPtxPdf_Page* pPage, BOOL bFitActualContent);

/**

If \ref "TRUE", the viewport is fitted to the width of the actual content of the page,
instead of the width of the page.

Note: Many PDF viewers simply ignore this property and always treat it
as \ref "FALSE", i.e. switching to "fit page" mode anyway.

* @param[in,out] pFitWidthDestination Acts as a handle to the native object of type \ref TPtxPdfNav_FitWidthDestination.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_FitWidthDestination_GetFitActualContent(TPtxPdfNav_FitWidthDestination* pFitWidthDestination);

/******************************************************************************
 * FitHeightDestination
 *****************************************************************************/
/**
 * @brief Create a new FitHeightDestination
The returned object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in,out] pPage The page in the document that this destination is pointing to.

* @param[in] bFitActualContent If \ref "TRUE", the viewport is fitted to the actual content of the page,
instead of the size of the page.
See property \ref PtxPdfNav_FitHeightDestination_GetFitActualContent "" for more information.


 * @return   The newly created destination object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b>differs from the document associated with <b>pPage</b>

 * - \ref ePtx_Error_IllegalArgument If the document associated with the <b>pPage</b> argument has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_FitHeightDestination* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_FitHeightDestination_Create(TPtxPdf_Document* pTargetDocument, TPtxPdf_Page* pPage, BOOL bFitActualContent);

/**

If \ref "TRUE", the viewport is fitted to the width of the actual content of the page,
instead of the width of the page.

Note: Many PDF viewers simply ignore this property and always treat it
as \ref "FALSE", i.e. switching to "fit page" mode anyway.

* @param[in,out] pFitHeightDestination Acts as a handle to the native object of type \ref
TPtxPdfNav_FitHeightDestination.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_FitHeightDestination_GetFitActualContent(TPtxPdfNav_FitHeightDestination* pFitHeightDestination);

/******************************************************************************
 * FitRectangleDestination
 *****************************************************************************/
/**
 * @brief Create a new FitRectangleDestination
The returned object is not yet used on any page, but it is associated with the given target document.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in,out] pPage The page in the document that this destination is pointing to.

* @param[in] pRectangle The rectangle that is displayed in the viewport.
See property \ref PtxPdfNav_FitRectangleDestination_GetRectangle "" for more information.


 * @return   The newly created destination object.

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b>differs from the document associated with <b>pPage</b>

 * - \ref ePtx_Error_IllegalArgument If the document associated with the <b>pPage</b> argument has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_FitRectangleDestination* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_FitRectangleDestination_Create(TPtxPdf_Document* pTargetDocument, TPtxPdf_Page* pPage,
                                         const TPtxGeomReal_Rectangle* pRectangle);

/**
The rectangle that is displayed in the viewport.

* @param[in,out] pFitRectangleDestination Acts as a handle to the native object of type \ref
TPtxPdfNav_FitRectangleDestination.

* @param[out] pRectangle Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState the object has already been closed.

 * - \ref ePtx_Error_IllegalState the associated document has already been closed.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_FitRectangleDestination_GetRectangle(
    TPtxPdfNav_FitRectangleDestination* pFitRectangleDestination, TPtxGeomReal_Rectangle* pRectangle);

/******************************************************************************
 * Link
 *****************************************************************************/
/**
 * @brief Copy a link from an input document to a output document.
* @param[in,out] pTargetDocument the output document with which the returned object is associated

* @param[in,out] pLink the link to be copied to the <b>pTargetDocument</b>


 * @return   the copied link, associated with the <b>pTargetDocument</b>

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pLink</b> is not associated with an input document

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pLink</b> object has already been closed

 * - \ref ePtx_Error_Conformance The conformance level of the input document is not compatible
with the conformance level of the <b>pTargetDocument</b>.

 * - \ref ePtx_Error_IO Error reading from the input document or writing to the output document


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_Link* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_Link_Copy(TPtxPdf_Document* pTargetDocument,
                                                                                   TPtxPdfNav_Link*  pLink);

/**
 * @brief The link area
The link is activated when a mouse click falls within the area defined by this list of \ref TPtxGeomReal_Quadrilateral
""s.

* @param[in,out] pLink Acts as a handle to the native object of type \ref TPtxPdfNav_Link.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxGeomReal_QuadrilateralList* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_Link_GetActiveArea(TPtxPdfNav_Link* pLink);
/**
 * @brief The location on the page
* @param[in,out] pLink Acts as a handle to the native object of type \ref TPtxPdfNav_Link.

* @param[out] pBoundingBox Retrieved value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_Corrupt if the PDF is corrupt


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_Link_GetBoundingBox(TPtxPdfNav_Link*        pLink,
                                                                                 TPtxGeomReal_Rectangle* pBoundingBox);
/**
 * @brief The link's visibility
If \ref "TRUE" then the link is present, but is invisible and not available for user interaction.
Depending on the \ref PtxPdfNav_Link_GetNoPrint "" property it will, however, still be visible when printing.

* @param[in,out] pLink Acts as a handle to the native object of type \ref TPtxPdfNav_Link.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_Link_GetHidden(TPtxPdfNav_Link* pLink);
/**
 * @brief The link's visibility when printing
If \ref "TRUE" then the link is not present in a print output of the document.

* @param[in,out] pLink Acts as a handle to the native object of type \ref TPtxPdfNav_Link.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_Link_GetNoPrint(TPtxPdfNav_Link* pLink);
/**
 * @brief The link's border
This property defines if and how a rectangular border is drawn for the link.

* @param[in,out] pLink Acts as a handle to the native object of type \ref TPtxPdfNav_Link.

* @param[in,out] pBorderStyle Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument the given \ref TPtxPdfContent_Stroke "" object is associated with a different
document

 * - \ref ePtx_Error_IllegalArgument the given \ref TPtxPdfContent_Stroke "" object has a \ref
PtxPdfContent_Stroke_GetDashPhase "" differing from 0

 * - \ref ePtx_Error_IllegalArgument the given \ref TPtxPdfContent_Stroke "" object has a \ref
PtxPdfContent_Stroke_GetMiterLimit "" differing from 10

 * - \ref ePtx_Error_IllegalArgument the given \ref TPtxPdfContent_Stroke "" object has a \ref
PtxPdfContent_Stroke_GetLineCapStyle "" other than \ref ePtxPdfContent_LineCapStyle_Butt ""

 * - \ref ePtx_Error_IllegalArgument the given \ref TPtxPdfContent_Stroke "" object has a \ref
PtxPdfContent_Stroke_GetLineJoinStyle "" other than \ref ePtxPdfContent_LineJoinStyle_Miter ""

 * - \ref ePtx_Error_IllegalArgument the given \ref TPtxPdfContent_Stroke "" object's \ref PtxPdfContent_Stroke_GetPaint
"" has a color space other than a device color space, or a calibrated color space.

 * - \ref ePtx_Error_UnsupportedFeature the given \ref TPtxPdfContent_Stroke "" object's \ref
PtxPdfContent_Stroke_GetPaint "" has a DeviceGray, DeviceCMYK, or CalGray color space.


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_Link_SetBorderStyle(TPtxPdfNav_Link*       pLink,
                                                                                 TPtxPdfContent_Stroke* pBorderStyle);

/**
 * @brief Get actual derived type of base type \ref TPtxPdfNav_Link.
 *
 * This function is invoked prior to downcasting to ascertain the derived object type.
 *
 * @param[in,out] pLink Acts as a handle to a native object.
 *
 * @return The item of the enumeration \ref TPtxPdfNav_LinkType that refers to the actual derived type. `0` in case of
 * an error.
 *
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 *       Get the error message with \ref Ptx_GetLastErrorMessage.
 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_LinkType PDFTOOLS_TOOLBOX_CALL PtxPdfNav_Link_GetType(TPtxPdfNav_Link* pLink);
/******************************************************************************
 * LinkList
 *****************************************************************************/
/**
 * @brief Get the number of elements in the list.
* @param[in,out] pLinkList Acts as a handle to the native object of type \ref TPtxPdfNav_LinkList.


 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState

 */
PDFTOOLS_TOOLBOX_EXPORT int PDFTOOLS_TOOLBOX_CALL PtxPdfNav_LinkList_GetCount(TPtxPdfNav_LinkList* pLinkList);
/**
 * @brief Returns the element at the specified position in the given list.
* @param[in,out] pLinkList Acts as a handle to the native object of type \ref TPtxPdfNav_LinkList.

* @param[in] iIndex

 * @return    `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_Link* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_LinkList_Get(TPtxPdfNav_LinkList* pLinkList,
                                                                                      int                  iIndex);
/**
 * @brief Add an element to the end of the list.
* @param[in,out] pLinkList Acts as a handle to the native object of type \ref TPtxPdfNav_LinkList.

* @param[in,out] pLink

 * @return    May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState
 * - \ref ePtx_Error_IllegalArgument
 * - \ref ePtx_Error_UnsupportedOperation

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_LinkList_Add(TPtxPdfNav_LinkList* pLinkList,
                                                                          TPtxPdfNav_Link*     pLink);

/******************************************************************************
 * InternalLink
 *****************************************************************************/
/**
 * @brief Create a document-internal link
The link is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of links.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in] pBoundingBox The location on the page.

* @param[in,out] pTarget The link target


 * @return   The newly created object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pTarget</b> argument is not associated with the <b>pTargetDocument</b>


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_InternalLink* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_InternalLink_Create(
    TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Rectangle* pBoundingBox, TPtxPdfNav_Destination* pTarget);
/**
 * @brief Create a document-internal link with defined link area
The link has an active area defined by the given <b>pActiveArea</b>.
The link is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of links.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in,out] pActiveArea The active link area on the page.

* @param[in,out] pTarget The link target


 * @return   The newly created object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pTarget</b> argument is not associated with the <b>pTargetDocument</b>

 * - \ref ePtx_Error_IllegalArgument if the <b>pActiveArea</b> is empty


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_InternalLink* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_InternalLink_CreateFromQuadrilaterals(
    TPtxPdf_Document* pTargetDocument, TPtxGeomReal_QuadrilateralList* pActiveArea, TPtxPdfNav_Destination* pTarget);

/**
 * @brief The link target
* @param[in,out] pInternalLink Acts as a handle to the native object of type \ref TPtxPdfNav_InternalLink.


 * @return   Retrieved value.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_IllegalState if the link has no destination


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_Destination* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_InternalLink_GetDestination(TPtxPdfNav_InternalLink* pInternalLink);

/******************************************************************************
 * WebLink
 *****************************************************************************/
/**
 * @brief Create an external link
The link is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of links.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in] pBoundingBox The location on the page.

* @param[in] szUri The link target


 * @return   The newly created object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>szUri</b> is empty


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_WebLink* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_WebLink_CreateA(
    TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Rectangle* pBoundingBox, const char* szUri);
/**
 * @brief Create an external link
The link is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of links.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in] pBoundingBox The location on the page.

* @param[in] szUri The link target


 * @return   The newly created object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>szUri</b> is empty


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_WebLink* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_WebLink_CreateW(
    TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Rectangle* pBoundingBox, const WCHAR* szUri);
/**
 * @brief Create an external link with defined link area
The link has an active area defined by the given <b>pActiveArea</b>.
The link is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of links.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in,out] pActiveArea The active link area on the page.

* @param[in] szUri The link target


 * @return   The newly created object

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pActiveArea</b> is empty

 * - \ref ePtx_Error_IllegalArgument if the <b>szUri</b> is empty


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_WebLink* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_WebLink_CreateFromQuadrilateralsA(
    TPtxPdf_Document* pTargetDocument, TPtxGeomReal_QuadrilateralList* pActiveArea, const char* szUri);
/**
 * @brief Create an external link with defined link area
The link has an active area defined by the given <b>pActiveArea</b>.
The link is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of links.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in,out] pActiveArea The active link area on the page.

* @param[in] szUri The link target


 * @return   The newly created object

  `NULL` if either an error occurred or the returned object is actually `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when `NULL` was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> argument is read-only

 * - \ref ePtx_Error_IllegalArgument if the <b>pActiveArea</b> is empty

 * - \ref ePtx_Error_IllegalArgument if the <b>szUri</b> is empty


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_WebLink* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_WebLink_CreateFromQuadrilateralsW(
    TPtxPdf_Document* pTargetDocument, TPtxGeomReal_QuadrilateralList* pActiveArea, const WCHAR* szUri);

/**
 * @brief The link target
* @param[in,out] pWebLink Acts as a handle to the native object of type \ref TPtxPdfNav_WebLink.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the given value is empty

 * - \ref ePtx_Error_IllegalState if the link has already been appended to a page's list of links


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdfNav_WebLink_GetUriA(TPtxPdfNav_WebLink* pWebLink,
                                                                               char* pBuffer, size_t nBufferSize);
/**
 * @brief The link target
* @param[in,out] pWebLink Acts as a handle to the native object of type \ref TPtxPdfNav_WebLink.

* @param[out] pBuffer Retrieved value.
To determine the required buffer size, the function has to be called with `NULL`.
The return value of this function specifies the buffer size.

* @param[in] nBufferSize The buffer size of the retrieved string `pBuffer`.


 * @return   The amount of data written to the buffer `pBuffer`.
   `0` if there is an error.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the given value is empty

 * - \ref ePtx_Error_IllegalState if the link has already been appended to a page's list of links


 */
PDFTOOLS_TOOLBOX_EXPORT size_t PDFTOOLS_TOOLBOX_CALL PtxPdfNav_WebLink_GetUriW(TPtxPdfNav_WebLink* pWebLink,
                                                                               WCHAR* pBuffer, size_t nBufferSize);
/**
 * @brief The link target
* @param[in,out] pWebLink Acts as a handle to the native object of type \ref TPtxPdfNav_WebLink.

* @param[in] szUri Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the given value is empty

 * - \ref ePtx_Error_IllegalState if the link has already been appended to a page's list of links


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_WebLink_SetUriA(TPtxPdfNav_WebLink* pWebLink,
                                                                             const char*         szUri);
/**
 * @brief The link target
* @param[in,out] pWebLink Acts as a handle to the native object of type \ref TPtxPdfNav_WebLink.

* @param[in] szUri Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalArgument if the given value is empty

 * - \ref ePtx_Error_IllegalState if the link has already been appended to a page's list of links


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_WebLink_SetUriW(TPtxPdfNav_WebLink* pWebLink,
                                                                             const WCHAR*        szUri);

/******************************************************************************
 * EmbeddedPdfLink
 *****************************************************************************/
/**
 * @brief Create a link to an embedded PDF document
The link is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of links.

* @param[in,out] pTargetDocument The output document with which the returned object is associated

* @param[in] pBoundingBox The location on the page

* @param[in,out] pFileReference The embedded PDF file


 * @return   The newly created object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pFileReference</b> argument has already been
closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pFileReference</b> argument does not contain a PDF document

 * - \ref ePtx_Error_IllegalArgument if the <b>pFileReference</b> argument is neither used in a file attachment
annotation nor has it been appended to the <b>pTargetDocument</b>'s list of plain embedded or associated files.

 * - \ref ePtx_Error_IllegalArgument if the <b>pFileReference</b> argument is used in a file attachment annotation and
this annotation has not been appended to a page's list of annotations.


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_EmbeddedPdfLink* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_EmbeddedPdfLink_Create(TPtxPdf_Document* pTargetDocument, const TPtxGeomReal_Rectangle* pBoundingBox,
                                 TPtxPdf_FileReference* pFileReference);
/**
 * @brief Create a link to an embedded PDF document with defined link area
The link has an active area defined by the given <b>pActiveArea</b>.
The link is associated with the <b>pTargetDocument</b> but not yet part of any page.
It can be added to a page's list of links.

* @param[in,out] pTargetDocument The document in which the links is used

* @param[in,out] pActiveArea The active link area on the page.

* @param[in,out] pFileReference The embedded PDF file


 * @return   The newly created object

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> has already been closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pTargetDocument</b> is read-only

 * - \ref ePtx_Error_IllegalArgument if the document associated with the <b>pFileReference</b> argument has already been
closed

 * - \ref ePtx_Error_IllegalArgument if the <b>pFileReference</b> argument does not contain a PDF document

 * - \ref ePtx_Error_IllegalArgument if the <b>pFileReference</b> argument is neither used in a file attachment
annotation nor has it been appended to the <b>pTargetDocument</b>'s list of plain embedded or associated files.

 * - \ref ePtx_Error_IllegalArgument if the <b>pFileReference</b> argument is used in a file attachment annotation and
this annotation has not been appended to a page's list of annotations.

 * - \ref ePtx_Error_IllegalArgument if the <b>pActiveArea</b> is empty


 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_EmbeddedPdfLink* PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_EmbeddedPdfLink_CreateFromQuadrilaterals(TPtxPdf_Document*               pTargetDocument,
                                                   TPtxGeomReal_QuadrilateralList* pActiveArea,
                                                   TPtxPdf_FileReference*          pFileReference);

/**
 * @brief The opening behavior
This defines the viewer's behavior when opening the target PDF document.
  - `NULL`: The viewer uses its default behavior.
  - \ref "TRUE": Open the document in an additional window.
  - \ref "FALSE": Replace the parent document with the embedded document.

* @param[in,out] pEmbeddedPdfLink Acts as a handle to the native object of type \ref TPtxPdfNav_EmbeddedPdfLink.

* @param[out] pNewWindow Retrieved value.


 * @return   \ref FALSE if either an error occurred or the `[out]` argument returns `NULL`.
   To determine if an error has occurred, check the error code as described in the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_EmbeddedPdfLink_GetNewWindow(TPtxPdfNav_EmbeddedPdfLink* pEmbeddedPdfLink, BOOL* pNewWindow);
/**
 * @brief The opening behavior
This defines the viewer's behavior when opening the target PDF document.
  - `NULL`: The viewer uses its default behavior.
  - \ref "TRUE": Open the document in an additional window.
  - \ref "FALSE": Replace the parent document with the embedded document.

* @param[in,out] pEmbeddedPdfLink Acts as a handle to the native object of type \ref TPtxPdfNav_EmbeddedPdfLink.

* @param[in] pNewWindow Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.
 * Possible error codes:
 * - \ref ePtx_Error_IllegalState if the object has already been closed

 * - \ref ePtx_Error_UnsupportedOperation if the document is read-only

 * - \ref ePtx_Error_IllegalState if the annotation has already been appended to a page's list of annotations


 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_EmbeddedPdfLink_SetNewWindow(TPtxPdfNav_EmbeddedPdfLink* pEmbeddedPdfLink, const BOOL* pNewWindow);

/******************************************************************************
 * OutlineCopyOptions
 *****************************************************************************/
/**
 * @return   Handle to the newly created native object.

   `NULL` if there is an error.
 * @note An error occurred when `NULL` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_OutlineCopyOptions* PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineCopyOptions_New(void);

/**
 * @brief Copy the logical structure and tagging information.

Specifies whether the logical structure and tagging information associated
with a an outline item is also copied when copying the item.

This is required if the target document conformance is PDF/A Level a.

Default value: \ref "TRUE"

* @param[in,out] pOutlineCopyOptions Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when \ref FALSE was returned <b>and</b> the error code returned by \ref Ptx_GetLastError is
different from \ref ePtx_Error_Success.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineCopyOptions_GetCopyLogicalStructure(TPtxPdfNav_OutlineCopyOptions* pOutlineCopyOptions);
/**
 * @brief Copy the logical structure and tagging information.

Specifies whether the logical structure and tagging information associated
with a an outline item is also copied when copying the item.

This is required if the target document conformance is PDF/A Level a.

Default value: \ref "TRUE"

* @param[in,out] pOutlineCopyOptions Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineCopyOptions.

* @param[in] bCopyLogicalStructure Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineCopyOptions_SetCopyLogicalStructure(
    TPtxPdfNav_OutlineCopyOptions* pOutlineCopyOptions, BOOL bCopyLogicalStructure);
/**
 * @brief Copy strategy for named destinations

Specify whether named destinations are resolved when copying an outline item.

Default value: \ref ePtxPdfNav_NamedDestinationCopyStrategy_Copy ""

* @param[in,out] pOutlineCopyOptions Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineCopyOptions.


 * @return   Retrieved value.

   May indicate an error in certain scenarios. For further information see the note section below.
 * @note An error occurred when `0` was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT TPtxPdfNav_NamedDestinationCopyStrategy PDFTOOLS_TOOLBOX_CALL
PtxPdfNav_OutlineCopyOptions_GetNamedDestinations(TPtxPdfNav_OutlineCopyOptions* pOutlineCopyOptions);
/**
 * @brief Copy strategy for named destinations

Specify whether named destinations are resolved when copying an outline item.

Default value: \ref ePtxPdfNav_NamedDestinationCopyStrategy_Copy ""

* @param[in,out] pOutlineCopyOptions Acts as a handle to the native object of type \ref TPtxPdfNav_OutlineCopyOptions.

* @param[in] iNamedDestinations Set value.


 * @return    \ref TRUE if the operation is successful; \ref FALSE if there is an error.
 * @note An error occurred when \ref FALSE was returned. Retrieve specific error code by calling \ref Ptx_GetLastError.
 * Get the error message with \ref Ptx_GetLastErrorMessage.

 */
PDFTOOLS_TOOLBOX_EXPORT BOOL PDFTOOLS_TOOLBOX_CALL PtxPdfNav_OutlineCopyOptions_SetNamedDestinations(
    TPtxPdfNav_OutlineCopyOptions* pOutlineCopyOptions, TPtxPdfNav_NamedDestinationCopyStrategy iNamedDestinations);

#ifdef __cplusplus
}
#endif

#endif /* PDFTOOLS_TOOLBOX_PTXPDFNAV_H__ */
